(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(define-cpp-converter cl_univpoly_complex_ring* static)
(define-cpp-converter cl_heap_univpoly_complex_ring* static)
;; Definition of class cl_univpoly_complex_ring
;; At /home/bruno/cln/include/cl_univpoly_complex.h:23
(define-cpp-class cl_univpoly_complex_ring (cl_univpoly_ring)
  ((operator= (const-cl_univpoly_complex_ring&)
              cl_univpoly_complex_ring&
              "operator =")
   (operator-> () cl_heap_univpoly_complex_ring* "operator ->"))
  constructor: ()
  copy-constructor: (const-cl_univpoly_complex_ring&)
  type: cl_univpoly_complex_ring*
  class: "cl_univpoly_complex_ring")
(define-cpp-converter cl_UP_N* static)
;; Definition of class cl_UP_N
;; At /home/bruno/cln/include/cl_univpoly_complex.h:39
(define-cpp-class cl_UP_N (cl_UP)
  ((ring () const-cl_univpoly_complex_ring& "ring")
   (set_coeff (uintL const-cl_N&) void "set_coeff")
   (finalize () void "finalize")
   (|operator()| (const-cl_N&) cl_N "operator ()")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_UP_N*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  copy-constructor: (cl_UP_N&)
  type: cl_UP_N*
  class: "cl_UP_N")
;; Definition of class cl_heap_univpoly_complex_ring
;; At /home/bruno/cln/include/cl_univpoly_complex.h:55
(define-cpp-class cl_heap_univpoly_complex_ring (cl_heap_univpoly_ring)
  ((operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (fprint (ostream& const-cl_UP_N&) void "fprint")
   (equal (const-cl_UP_N& const-cl_UP_N&) cl_boolean "equal")
   (zero () cl_UP_N "zero")
   (zerop (const-cl_UP_N&) cl_boolean "zerop")
   (plus (const-cl_UP_N& const-cl_UP_N&) cl_UP_N "plus")
   (minus (const-cl_UP_N& const-cl_UP_N&) cl_UP_N "minus")
   (uminus (const-cl_UP_N&) cl_UP_N "uminus")
   (one () cl_UP_N "one")
   (canonhom (const-cl_I&) cl_UP_N "canonhom")
   (mul (const-cl_UP_N& const-cl_UP_N&) cl_UP_N "mul")
   (square (const-cl_UP_N&) cl_UP_N "square")
   (expt_pos (const-cl_UP_N& const-cl_I&) cl_UP_N "expt_pos")
   (degree (const-cl_UP_N&) sintL "degree")
   (monomial (const-cl_N& uintL) cl_UP_N "monomial")
   (coeff (const-cl_UP_N& uintL) cl_N "coeff")
   (create (sintL) cl_UP_N "create")
   (set_coeff (cl_UP_N& uintL const-cl_N&) void "set_coeff")
   (finalize (cl_UP_N&) void "finalize")
   (eval (const-cl_UP_N& const-cl_N&) cl_N "eval")
   (operator= (cl_heap_univpoly_complex_ring&)
              cl_heap_univpoly_complex_ring&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_heap_univpoly_complex_ring&)
  type: cl_heap_univpoly_complex_ring*
  class: "cl_heap_univpoly_complex_ring")
(defoverloaded operator+ (const-cl_UP_N& const-cl_UP_N&) cl_UP_N "operator +")
(defoverloaded operator- (const-cl_UP_N&) cl_UP_N "operator -")
(defoverloaded operator- (const-cl_UP_N& const-cl_UP_N&) cl_UP_N "operator -")
(defoverloaded operator* (const-cl_UP_N& const-cl_UP_N&) cl_UP_N "operator *")
(defoverloaded square (const-cl_UP_N&) cl_UP_N "square")
(defoverloaded expt_pos (const-cl_UP_N& const-cl_I&) cl_UP_N "expt_pos")
(defoverloaded operator* (const-cl_I& const-cl_UP_N&) cl_UP_N "operator *")
(defoverloaded operator* (const-cl_UP_N& const-cl_I&) cl_UP_N "operator *")
(defoverloaded coeff (const-cl_UP_N& uintL) cl_N "coeff")
(defoverloaded set_coeff (cl_UP_N& uintL const-cl_N&) void "set_coeff")
(defoverloaded finalize (cl_UP_N&) void "finalize")
(defoverloaded deriv (const-cl_UP_N&) cl_UP_N "deriv")
(define-cpp-converter cl_univpoly_modint_ring* static)
(define-cpp-converter cl_heap_univpoly_modint_ring* static)
;; Definition of class cl_univpoly_modint_ring
;; At /home/bruno/cln/include/cl_univpoly_modint.h:15
(define-cpp-class cl_univpoly_modint_ring (cl_univpoly_ring)
  ((operator= (const-cl_univpoly_modint_ring&)
              cl_univpoly_modint_ring&
              "operator =")
   (operator-> () cl_heap_univpoly_modint_ring* "operator ->"))
  constructor: ()
  copy-constructor: (const-cl_univpoly_modint_ring&)
  type: cl_univpoly_modint_ring*
  class: "cl_univpoly_modint_ring")
(define-cpp-converter cl_UP_MI* static)
;; Definition of class cl_UP_MI
;; At /home/bruno/cln/include/cl_univpoly_modint.h:31
(define-cpp-class cl_UP_MI (cl_UP)
  ((ring () const-cl_univpoly_modint_ring& "ring")
   (set_coeff (uintL const-cl_MI&) void "set_coeff")
   (finalize () void "finalize")
   (|operator()| (const-cl_MI&) cl_MI "operator ()")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_UP_MI*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  copy-constructor: (cl_UP_MI&)
  type: cl_UP_MI*
  class: "cl_UP_MI")
;; Definition of class cl_heap_univpoly_modint_ring
;; At /home/bruno/cln/include/cl_univpoly_modint.h:47
(define-cpp-class cl_heap_univpoly_modint_ring (cl_heap_univpoly_ring)
  ((operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (fprint (ostream& const-cl_UP_MI&) void "fprint")
   (equal (const-cl_UP_MI& const-cl_UP_MI&) cl_boolean "equal")
   (zero () cl_UP_MI "zero")
   (zerop (const-cl_UP_MI&) cl_boolean "zerop")
   (plus (const-cl_UP_MI& const-cl_UP_MI&) cl_UP_MI "plus")
   (minus (const-cl_UP_MI& const-cl_UP_MI&) cl_UP_MI "minus")
   (uminus (const-cl_UP_MI&) cl_UP_MI "uminus")
   (one () cl_UP_MI "one")
   (canonhom (const-cl_I&) cl_UP_MI "canonhom")
   (mul (const-cl_UP_MI& const-cl_UP_MI&) cl_UP_MI "mul")
   (square (const-cl_UP_MI&) cl_UP_MI "square")
   (expt_pos (const-cl_UP_MI& const-cl_I&) cl_UP_MI "expt_pos")
   (degree (const-cl_UP_MI&) sintL "degree")
   (monomial (const-cl_MI& uintL) cl_UP_MI "monomial")
   (coeff (const-cl_UP_MI& uintL) cl_MI "coeff")
   (create (sintL) cl_UP_MI "create")
   (set_coeff (cl_UP_MI& uintL const-cl_MI&) void "set_coeff")
   (finalize (cl_UP_MI&) void "finalize")
   (eval (const-cl_UP_MI& const-cl_MI&) cl_MI "eval")
   (operator= (cl_heap_univpoly_modint_ring&)
              cl_heap_univpoly_modint_ring&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_heap_univpoly_modint_ring&)
  type: cl_heap_univpoly_modint_ring*
  class: "cl_heap_univpoly_modint_ring")
(defoverloaded cl_find_univpoly_ring (const-cl_modint_ring&)
  cl_univpoly_modint_ring
  "cl_find_univpoly_ring")
(defoverloaded cl_find_univpoly_ring (const-cl_modint_ring& const-cl_symbol&)
  cl_univpoly_modint_ring
  "cl_find_univpoly_ring")
(defoverloaded operator+ (const-cl_UP_MI& const-cl_UP_MI&)
  cl_UP_MI
  "operator +")
(defoverloaded operator- (const-cl_UP_MI&) cl_UP_MI "operator -")
(defoverloaded operator- (const-cl_UP_MI& const-cl_UP_MI&)
  cl_UP_MI
  "operator -")
(defoverloaded operator* (const-cl_UP_MI& const-cl_UP_MI&)
  cl_UP_MI
  "operator *")
(defoverloaded square (const-cl_UP_MI&) cl_UP_MI "square")
(defoverloaded expt_pos (const-cl_UP_MI& const-cl_I&) cl_UP_MI "expt_pos")
(defoverloaded operator* (const-cl_I& const-cl_UP_MI&) cl_UP_MI "operator *")
(defoverloaded operator* (const-cl_UP_MI& const-cl_I&) cl_UP_MI "operator *")
(defoverloaded coeff (const-cl_UP_MI& uintL) cl_MI "coeff")
(defoverloaded set_coeff (cl_UP_MI& uintL const-cl_MI&) void "set_coeff")
(defoverloaded finalize (cl_UP_MI&) void "finalize")
(defoverloaded deriv (const-cl_UP_MI&) cl_UP_MI "deriv")
(define-cpp-converter cl_timespec* static)
;; Definition of class cl_timespec
;; At /home/bruno/cln/include/cl_timing.h:12
(define-cpp-class cl_timespec (<class-proxy>)
  ((tv_sec type: uintL cpp-name: "tv_sec")
   (tv_nsec type: sintL cpp-name: "tv_nsec")
   (operator= (cl_timespec&) cl_timespec& "operator =" implicitp: t))
  constructor: ()
  constructor: (uintL sintL)
  copy-constructor: (cl_timespec&)
  type: cl_timespec*
  class: "cl_timespec")
(define-cpp-converter cl_time_duration* static)
;; Definition of class cl_time_duration
;; At /home/bruno/cln/include/cl_timing.h:21
(define-cpp-class cl_time_duration (<class-proxy>)
  ((tv_sec type: uintL cpp-name: "tv_sec")
   (tv_nsec type: uintL cpp-name: "tv_nsec")
   (operator= (cl_time_duration&) cl_time_duration& "operator =" implicitp: t))
  constructor: ()
  constructor: (uintL)
  constructor: (uintL uintL)
  copy-constructor: (cl_time_duration&)
  type: cl_time_duration*
  class: "cl_time_duration")
(define-cpp-converter cl_time_consumption* static)
;; Definition of class cl_time_consumption
;; At /home/bruno/cln/include/cl_timing.h:32
(define-cpp-class cl_time_consumption (<class-proxy>)
  ((realtime type: cl_time_duration cpp-name: "realtime")
   (usertime type: cl_time_duration cpp-name: "usertime")
   (operator= (cl_time_consumption&)
              cl_time_consumption&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_time_consumption&)
  constructor: ()
  type: cl_time_consumption*
  class: "cl_time_consumption")
(defoverloaded operator- (const-cl_timespec& const-cl_timespec&)
  cl_time_duration
  "operator -")
(defoverloaded operator+ (const-cl_timespec& const-cl_time_duration&)
  cl_timespec
  "operator +")
(defoverloaded operator- (const-cl_timespec& const-cl_time_duration&)
  cl_timespec
  "operator -")
(defoverloaded operator+ (const-cl_time_duration& const-cl_time_duration&)
  cl_time_duration
  "operator +")
(defoverloaded operator- (const-cl_time_duration& const-cl_time_duration&)
  cl_time_duration
  "operator -")
(defoverloaded cl_current_time () cl_timespec "cl_current_time")
