(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded operator* (const-cl_R& const-cl_R&) cl_R "operator *")
(defoverloaded operator* (long const-cl_R&) cl_R "operator *")
(defoverloaded operator* (const-cl_R& long) cl_R "operator *")
(defoverloaded square (const-cl_R&) cl_R "square")
(defoverloaded operator/ (const-cl_R& const-cl_R&) cl_R "operator /")
(defoverloaded operator/ (const-cl_F& const-cl_R&) cl_F "operator /")
(defoverloaded operator/ (long const-cl_R&) cl_R "operator /")
(defoverloaded operator/ (const-cl_R& long) cl_R "operator /")
(defoverloaded operator/ (const-cl_F& long) cl_F "operator /")
(defoverloaded recip (const-cl_R&) cl_R "recip")
(defoverloaded plus1 (const-cl_R&) cl_R "plus1")
(defoverloaded minus1 (const-cl_R&) cl_R "minus1")
(define-cpp-converter cl_R_div_t* static)
;; Definition of class cl_R_div_t
;; At /home/bruno/cln/include/cl_real.h:255
(define-cpp-class cl_R_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_R cpp-name: "remainder")
   (operator= (cl_R_div_t&) cl_R_div_t& "operator =" implicitp: t))
  type: cl_R_div_t*
  class: "cl_R_div_t")
(defoverloaded floor2 (const-cl_R&) cl_R_div_t "floor2")
(defoverloaded floor1 (const-cl_R&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_R&) cl_R_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_R&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_R&) cl_R_div_t "truncate2")
(defoverloaded truncate1 (const-cl_R&) cl_I "truncate1")
(defoverloaded round2 (const-cl_R&) cl_R_div_t "round2")
(defoverloaded round1 (const-cl_R&) cl_I "round1")
(defoverloaded floor2 (const-cl_R& const-cl_R&) cl_R_div_t "floor2")
(defoverloaded floor1 (const-cl_R& const-cl_R&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_R& const-cl_R&) cl_R_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_R& const-cl_R&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_R& const-cl_R&) cl_R_div_t "truncate2")
(defoverloaded truncate1 (const-cl_R& const-cl_R&) cl_I "truncate1")
(defoverloaded round2 (const-cl_R& const-cl_R&) cl_R_div_t "round2")
(defoverloaded round1 (const-cl_R& const-cl_R&) cl_I "round1")
(define-cpp-converter cl_R_fdiv_t* static)
;; Definition of class cl_R_fdiv_t
;; At /home/bruno/cln/include/cl_real.h:301
(define-cpp-class cl_R_fdiv_t (<class-proxy>)
  ((quotient type: cl_F cpp-name: "quotient")
   (remainder type: cl_R cpp-name: "remainder")
   (operator= (cl_R_fdiv_t&) cl_R_fdiv_t& "operator =" implicitp: t))
  type: cl_R_fdiv_t*
  class: "cl_R_fdiv_t")
(defoverloaded ffloor2 (const-cl_R&) cl_R_fdiv_t "ffloor2")
(defoverloaded ffloor (const-cl_R&) cl_F "ffloor")
(defoverloaded fceiling2 (const-cl_R&) cl_R_fdiv_t "fceiling2")
(defoverloaded fceiling (const-cl_R&) cl_F "fceiling")
(defoverloaded ftruncate2 (const-cl_R&) cl_R_fdiv_t "ftruncate2")
(defoverloaded ftruncate (const-cl_R&) cl_F "ftruncate")
(defoverloaded fround2 (const-cl_R&) cl_R_fdiv_t "fround2")
(defoverloaded fround (const-cl_R&) cl_F "fround")
(defoverloaded ffloor2 (const-cl_R& const-cl_R&) cl_R_fdiv_t "ffloor2")
(defoverloaded ffloor (const-cl_R& const-cl_R&) cl_F "ffloor")
(defoverloaded fceiling2 (const-cl_R& const-cl_R&) cl_R_fdiv_t "fceiling2")
(defoverloaded fceiling (const-cl_R& const-cl_R&) cl_F "fceiling")
(defoverloaded ftruncate2 (const-cl_R& const-cl_R&) cl_R_fdiv_t "ftruncate2")
(defoverloaded ftruncate (const-cl_R& const-cl_R&) cl_F "ftruncate")
(defoverloaded fround2 (const-cl_R& const-cl_R&) cl_R_fdiv_t "fround2")
(defoverloaded fround (const-cl_R& const-cl_R&) cl_F "fround")
(defoverloaded mod (const-cl_R& const-cl_R&) cl_R "mod")
(defoverloaded rem (const-cl_R& const-cl_R&) cl_R "rem")
