// Conditions.

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_condition.h"


// Implementation.

// This tells the compiler to put the `cl_condition' vtable into this file.
void cl_condition::dummy () {}

// The destructor must be defined although it is virtual and abstract.
#ifndef __GNUC__
cl_condition::~cl_condition () {}
#endif
