// copy().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_GV_complex.h"


// Implementation.

cl_GV_N copy (const cl_GV_N& v)
{
	var uintL len = v.length();
	var cl_GV_N w = cl_GV_N(len);
	for (var uintL i = 0; i < len; i++)
		w[i] = v[i];
	return w;
}
