(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded rational (const-cl_F&) cl_RA "rational")
(defoverloaded rational (const-cl_R&) cl_RA "rational")
(defoverloaded rational (const-cl_RA&) cl_RA "rational")
(defoverloaded cl_equal (const-cl_R& const-cl_R&) cl_boolean "cl_equal")
(defoverloaded cl_compare (const-cl_R& const-cl_R&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_R& const-cl_R&) bool "operator ==")
(defoverloaded operator!= (const-cl_R& const-cl_R&) bool "operator !=")
(defoverloaded operator<= (const-cl_R& const-cl_R&) bool "operator <=")
(defoverloaded operator< (const-cl_R& const-cl_R&) bool "operator <")
(defoverloaded operator>= (const-cl_R& const-cl_R&) bool "operator >=")
(defoverloaded operator> (const-cl_R& const-cl_R&) bool "operator >")
(defoverloaded signum (const-cl_R&) cl_R "signum")
(defoverloaded sqrt (const-cl_R&) cl_R "sqrt")
(defoverloaded expt (const-cl_R& sintL) cl_R "expt")
(defoverloaded expt (const-cl_R& const-cl_I&) cl_R "expt")
(defoverloaded rationalize (const-cl_R&) cl_RA "rationalize")
(defoverloaded cl_float_approx (const-cl_R&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_R&) double "cl_double_approx")
(defoverloaded cl_pi (const-cl_F&) cl_F "cl_pi")
(defoverloaded cl_pi (cl_float_format_t) cl_F "cl_pi")
(defoverloaded cl_pi () cl_F "cl_pi")
(defoverloaded atan (const-cl_R& const-cl_R&) cl_R "atan")
(defoverloaded atan (const-cl_R& const-cl_F&) cl_F "atan")
(defoverloaded atan (const-cl_R& long) cl_R "atan")
(defoverloaded atan (const-cl_R&) cl_R "atan")
(defoverloaded atan (const-cl_F&) cl_F "atan")
(defoverloaded atan (long) cl_R "atan")
(defoverloaded sin (const-cl_R&) cl_R "sin")
(defoverloaded sin (const-cl_F&) cl_F "sin")
(defoverloaded sin (long) cl_R "sin")
(defoverloaded cos (const-cl_R&) cl_R "cos")
(defoverloaded cos (const-cl_F&) cl_F "cos")
(defoverloaded cos (long) cl_R "cos")
(define-cpp-converter cl_cos_sin_t* static)
;; Definition of class cl_cos_sin_t
;; At /home/bruno/cln/include/cl_real.h:470
(define-cpp-class cl_cos_sin_t (<class-proxy>)
  ((cos type: cl_R cpp-name: "cos")
   (sin type: cl_R cpp-name: "sin")
   (operator= (cl_cos_sin_t&) cl_cos_sin_t& "operator =" implicitp: t))
  constructor: (const-cl_R& const-cl_R&)
  copy-constructor: (cl_cos_sin_t&)
  type: cl_cos_sin_t*
  class: "cl_cos_sin_t")
(defoverloaded cl_cos_sin (const-cl_R&) cl_cos_sin_t "cl_cos_sin")
(defoverloaded tan (const-cl_R&) cl_R "tan")
(defoverloaded tan (const-cl_F&) cl_F "tan")
(defoverloaded tan (long) cl_R "tan")
(defoverloaded cl_exp1 (const-cl_F&) cl_F "cl_exp1")
(defoverloaded cl_exp1 (cl_float_format_t) cl_F "cl_exp1")
(defoverloaded cl_exp1 () cl_F "cl_exp1")
(defoverloaded ln (const-cl_R&) cl_R "ln")
(defoverloaded ln (const-cl_F&) cl_F "ln")
(defoverloaded ln (const-cl_LF&) cl_LF "ln")
(defoverloaded ln (long) cl_R "ln")
(defoverloaded log (const-cl_R& const-cl_R&) cl_R "log")
(defoverloaded exp (const-cl_R&) cl_R "exp")
(defoverloaded exp (const-cl_F&) cl_F "exp")
(defoverloaded exp (long) cl_R "exp")
