// print_number().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_output.h"


// Implementation.

#include "cl_print.h"
#include "cl_complex.h"
#include "cl_C.h"

void print_number (cl_ostream stream, const cl_print_number_flags& flags, const cl_number& z)
{
 {	DeclareType(cl_N,z);
	if (realp(z)) {
		DeclareType(cl_R,z);
		print_real(stream,flags,z);
	} else {
		DeclareType(cl_C,z);
		print_complex(stream,flags,z);
	}
}}
