// cl_SV_N debugging support.

// General includes.
#include "cl_sysdep.h"

// Specification.


// Implementation.

#include "cl_SV_complex.h"
#include "cl_io.h"

static void dprint (cl_heap* pointer)
{
	var const cl_SV_N& obj = *(const cl_SV_N*)&pointer;
	fprint(cl_debugout, "(cl_SV_N) ");
	fprint(cl_debugout, obj);
}
AT_INITIALIZATION(dprint_SV_N)
{ cl_register_typetag_printer(cl_typetag_svector_number,dprint); }

// This dummy links in this module when <cl_SV_complex.h> requires it.
int cl_SV_N_debug_module;
