/* Integers of type char have 8 bits. */
#define char_bitsize 8

/* Integers of type short have 16 bits. */
#define short_bitsize 16

/* Integers of type int have 32 bits. */
#define int_bitsize 32

/* Integers of type long have 32 bits. */
#define long_bitsize 32

/* Integers of type long long have 64 bits. */
#define long_long_bitsize 64

/* Integers of type unsigned char have 8 bits. */

/* Integers of type unsigned short have 16 bits. */

/* Integers of type unsigned int have 32 bits. */

/* Integers of type unsigned long have 32 bits. */

/* Integers of type unsigned long long have 64 bits. */

/* Integer types char and unsigned char have the same binary representation. */
/* Integer types short and unsigned short have the same binary representation. */
/* Integer types int and unsigned int have the same binary representation. */
/* Integer types long and unsigned long have the same binary representation. */
/* Integer types long long and unsigned long long have the same binary representation. */

/* Pointers of type char * have 32 bits. */
#define pointer_bitsize 32

/* Casts from long * to char * is OK (does nothing). */
/* Casts from char * to long * is OK (does nothing). */
/* Casts from function * to char * is OK (does nothing). */
/* Casts from char * to function * is OK (does nothing). */

/* Type unsigned short is stored BIG-ENDIAN in memory (i.e. like mc68000 or sparc). */
#define short_big_endian
/* Type unsigned int is stored BIG-ENDIAN in memory (i.e. like mc68000 or sparc). */
#define int_big_endian
/* Type unsigned long is stored BIG-ENDIAN in memory (i.e. like mc68000 or sparc). */
#define long_big_endian
/* Type unsigned long long is stored BIG-ENDIAN in memory (i.e. like mc68000 or sparc). */
#define long_long_big_endian

/* Stack grows up, ca. 64 bytes per function call. */
#define stack_grows_up
