// cl_ln2().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_R_tran.h"


// Implementation.

#include "cl_F.h"
#include "cl_LF.h"

cl_F cl_ln2 (const cl_F& y)
{
	floattypecase(y
	,	return cl_SF_ln2;
	,	return cl_FF_ln2;
	,	return cl_DF_ln2;
	,	return cl_ln2(TheLfloat(y)->len);
	);
}
