(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(define-callback-pool |_cl_MI(*)(cl_heap_modint_ring*,cl_random_state&)| 5)
(define-cpp-converter _cl_modint_setops* static)
;; Definition of class _cl_modint_setops
;; At /home/bruno/cln/include/cl_modinteger.h:151
(define-cpp-class _cl_modint_setops (<class-proxy>)
  ((fprint type:
           |void(*)(cl_heap_modint_ring*,ostream&,const_cl_MI&)|
           cpp-name: "fprint")
   (equal type:
          |cl_boolean(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
          cpp-name: "equal")
   (random type:
           |_cl_MI(*)(cl_heap_modint_ring*,cl_random_state&)|
           cpp-name: "random")
   (operator= (_cl_modint_setops&)
              _cl_modint_setops&
              "operator ="
              implicitp: t))
  copy-constructor: (_cl_modint_setops&)
  constructor: ()
  type: _cl_modint_setops*
  class: "_cl_modint_setops")
(define-cpp-converter |_cl_MI*(*)(cl_heap_modint_ring**)| function-type)
(define-callback-pool |_cl_MI(*)(cl_heap_modint_ring*)| 5)
(define-cpp-converter
  |cl_boolean(*)(cl_heap_modint_ring**,_cl_MI**)|
  function-type)
(define-callback-pool |cl_boolean(*)(cl_heap_modint_ring*,const_cl_MI&)| 5)
(define-cpp-converter
  |_cl_MI*(*)(cl_heap_modint_ring**,_cl_MI**,_cl_MI**)|
  function-type)
(define-callback-pool
  |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
  5)
(define-cpp-converter
  |_cl_MI*(*)(cl_heap_modint_ring**,_cl_MI**)|
  function-type)
(define-callback-pool |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&)| 5)
(define-cpp-converter _cl_modint_addops* static)
;; Definition of class _cl_modint_addops
;; At /home/bruno/cln/include/cl_modinteger.h:159
(define-cpp-class _cl_modint_addops (<class-proxy>)
  ((zero type: |_cl_MI(*)(cl_heap_modint_ring*)| cpp-name: "zero")
   (zerop type:
          |cl_boolean(*)(cl_heap_modint_ring*,const_cl_MI&)|
          cpp-name: "zerop")
   (plus type:
         |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
         cpp-name: "plus")
   (minus type:
          |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
          cpp-name: "minus")
   (uminus type:
           |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&)|
           cpp-name: "uminus")
   (operator= (_cl_modint_addops&)
              _cl_modint_addops&
              "operator ="
              implicitp: t))
  copy-constructor: (_cl_modint_addops&)
  constructor: ()
  type: _cl_modint_addops*
  class: "_cl_modint_addops")
(define-cpp-converter |_cl_MI*(*)(cl_heap_modint_ring**,cl_I**)| function-type)
(define-callback-pool |_cl_MI(*)(cl_heap_modint_ring*,const-cl_I&)| 5)
(define-cpp-converter
  |_cl_MI*(*)(cl_heap_modint_ring**,_cl_MI**,cl_I**)|
  function-type)
(define-callback-pool
  |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&,const-cl_I&)|
  5)
(define-cpp-converter
  |cl_MI_x*(*)(cl_heap_modint_ring**,_cl_MI**)|
  function-type)
(define-callback-pool |cl_MI_x(*)(cl_heap_modint_ring*,const_cl_MI&)| 5)
(define-cpp-converter
  |cl_MI_x*(*)(cl_heap_modint_ring**,_cl_MI**,_cl_MI**)|
  function-type)
(define-callback-pool
  |cl_MI_x(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
  5)
(define-cpp-converter
  |cl_MI_x*(*)(cl_heap_modint_ring**,_cl_MI**,cl_I**)|
  function-type)
(define-callback-pool
  |cl_MI_x(*)(cl_heap_modint_ring*,const_cl_MI&,const-cl_I&)|
  5)
(define-cpp-converter |cl_I*(*)(cl_heap_modint_ring**,cl_I**)| function-type)
(define-callback-pool |cl_I(*)(cl_heap_modint_ring*,const-cl_I&)| 5)
(define-cpp-converter |cl_I*(*)(cl_heap_modint_ring**,_cl_MI**)| function-type)
(define-callback-pool |cl_I(*)(cl_heap_modint_ring*,const_cl_MI&)| 5)
(define-cpp-converter _cl_modint_mulops* static)
;; Definition of class _cl_modint_mulops
;; At /home/bruno/cln/include/cl_modinteger.h:170
(define-cpp-class _cl_modint_mulops (<class-proxy>)
  ((one type: |_cl_MI(*)(cl_heap_modint_ring*)| cpp-name: "one")
   (canonhom type:
             |_cl_MI(*)(cl_heap_modint_ring*,const-cl_I&)|
             cpp-name: "canonhom")
   (mul type:
        |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
        cpp-name: "mul")
   (square type:
           |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&)|
           cpp-name: "square")
   (expt_pos type:
             |_cl_MI(*)(cl_heap_modint_ring*,const_cl_MI&,const-cl_I&)|
             cpp-name: "expt_pos")
   (recip type:
          |cl_MI_x(*)(cl_heap_modint_ring*,const_cl_MI&)|
          cpp-name: "recip")
   (div type:
        |cl_MI_x(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
        cpp-name: "div")
   (expt type:
         |cl_MI_x(*)(cl_heap_modint_ring*,const_cl_MI&,const-cl_I&)|
         cpp-name: "expt")
   (reduce_modulo
     type: |cl_I(*)(cl_heap_modint_ring*,const-cl_I&)|
     cpp-name: "reduce_modulo")
   (retract type:
            |cl_I(*)(cl_heap_modint_ring*,const_cl_MI&)|
            cpp-name: "retract")
   (operator= (_cl_modint_mulops&)
              _cl_modint_mulops&
              "operator ="
              implicitp: t))
  copy-constructor: (_cl_modint_mulops&)
  constructor: ()
  type: _cl_modint_mulops*
  class: "_cl_modint_mulops")
(defoverloaded cl_find_modint_ring (const-cl_I&)
  cl_modint_ring
  "cl_find_modint_ring")
(defoverloaded fprint (ostream& const-cl_MI&) void "fprint")
(defoverloaded operator<< (ostream& const-cl_MI&) ostream& "operator <<")
(defoverloaded operator+ (const-cl_MI& const-cl_MI&) cl_MI "operator +")
(defoverloaded operator+ (const-cl_MI& const-cl_I&) cl_MI "operator +")
(defoverloaded operator+ (const-cl_I& const-cl_MI&) cl_MI "operator +")
(defoverloaded operator- (const-cl_MI&) cl_MI "operator -")
(defoverloaded operator- (const-cl_MI& const-cl_MI&) cl_MI "operator -")
(defoverloaded operator- (const-cl_MI& const-cl_I&) cl_MI "operator -")
(defoverloaded operator- (const-cl_I& const-cl_MI&) cl_MI "operator -")
(defoverloaded operator<< (const-cl_MI& sintL) cl_MI "operator <<")
(defoverloaded operator>> (const-cl_MI& sintL) cl_MI "operator >>")
(defoverloaded operator== (const-cl_MI& const-cl_MI&) bool "operator ==")
(defoverloaded operator!= (const-cl_MI& const-cl_MI&) bool "operator !=")
(defoverloaded operator== (const-cl_MI& const-cl_I&) bool "operator ==")
(defoverloaded operator!= (const-cl_MI& const-cl_I&) bool "operator !=")
(defoverloaded operator== (const-cl_I& const-cl_MI&) bool "operator ==")
(defoverloaded operator!= (const-cl_I& const-cl_MI&) bool "operator !=")
(defoverloaded zerop (const-cl_MI&) cl_boolean "zerop")
(defoverloaded operator* (const-cl_MI& const-cl_MI&) cl_MI "operator *")
(defoverloaded square (const-cl_MI&) cl_MI "square")
