(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded operator<= (const-cl_DF& const-cl_DF&) bool "operator <=")
(defoverloaded operator< (const-cl_DF& const-cl_DF&) bool "operator <")
(defoverloaded operator>= (const-cl_DF& const-cl_DF&) bool "operator >=")
(defoverloaded operator> (const-cl_DF& const-cl_DF&) bool "operator >")
(defoverloaded minusp (const-cl_DF&) cl_boolean "minusp")
(defoverloaded zerop (const-cl_DF&) cl_boolean "zerop")
(defoverloaded operator+ (const-cl_DF& const-cl_DF&) cl_DF "operator +")
(defoverloaded operator- (const-cl_DF& const-cl_DF&) cl_DF "operator -")
(defoverloaded operator* (const-cl_DF& const-cl_DF&) cl_DF "operator *")
(defoverloaded square (const-cl_DF&) cl_DF "square")
(defoverloaded operator/ (const-cl_DF& const-cl_DF&) cl_DF "operator /")
(defoverloaded sqrt (const-cl_DF&) cl_DF "sqrt")
(defoverloaded recip (const-cl_DF&) cl_DF "recip")
(defoverloaded plus1 (const-cl_DF&) cl_DF "plus1")
(defoverloaded minus1 (const-cl_DF&) cl_DF "minus1")
(defoverloaded ftruncate (const-cl_DF&) cl_DF "ftruncate")
(defoverloaded fround (const-cl_DF&) cl_DF "fround")
(defoverloaded ffloor (const-cl_DF&) cl_DF "ffloor")
(defoverloaded fceiling (const-cl_DF&) cl_DF "fceiling")
(define-cpp-converter cl_DF_fdiv_t* static)
;; Definition of class cl_DF_fdiv_t
;; At /home/bruno/cln/include/cl_dfloat.h:98
(define-cpp-class cl_DF_fdiv_t (<class-proxy>)
  ((quotient type: cl_DF cpp-name: "quotient")
   (remainder type: cl_DF cpp-name: "remainder")
   (operator= (cl_DF_fdiv_t&) cl_DF_fdiv_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_DF& const-cl_DF&)
  copy-constructor: (cl_DF_fdiv_t&)
  type: cl_DF_fdiv_t*
  class: "cl_DF_fdiv_t")
(defoverloaded ffloor2 (const-cl_DF&) cl_DF_fdiv_t "ffloor2")
(defoverloaded fceiling2 (const-cl_DF&) cl_DF_fdiv_t "fceiling2")
(defoverloaded ftruncate2 (const-cl_DF&) cl_DF_fdiv_t "ftruncate2")
(defoverloaded fround2 (const-cl_DF&) cl_DF_fdiv_t "fround2")
(define-cpp-converter cl_DF_div_t* static)
;; Definition of class cl_DF_div_t
;; At /home/bruno/cln/include/cl_dfloat.h:125
(define-cpp-class cl_DF_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_DF cpp-name: "remainder")
   (operator= (cl_DF_div_t&) cl_DF_div_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_DF&)
  copy-constructor: (cl_DF_div_t&)
  type: cl_DF_div_t*
  class: "cl_DF_div_t")
(defoverloaded floor2 (const-cl_DF&) cl_DF_div_t "floor2")
(defoverloaded floor1 (const-cl_DF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_DF&) cl_DF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_DF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_DF&) cl_DF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_DF&) cl_I "truncate1")
(defoverloaded round2 (const-cl_DF&) cl_DF_div_t "round2")
(defoverloaded round1 (const-cl_DF&) cl_I "round1")
(defoverloaded floor2 (const-cl_DF& const-cl_DF&) cl_DF_div_t "floor2")
(defoverloaded floor1 (const-cl_DF& const-cl_DF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_DF& const-cl_DF&) cl_DF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_DF& const-cl_DF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_DF& const-cl_DF&) cl_DF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_DF& const-cl_DF&) cl_I "truncate1")
(defoverloaded round2 (const-cl_DF& const-cl_DF&) cl_DF_div_t "round2")
(defoverloaded round1 (const-cl_DF& const-cl_DF&) cl_I "round1")
(define-cpp-converter cl_decoded_dfloat* static)
;; Definition of class cl_decoded_dfloat
;; At /home/bruno/cln/include/cl_dfloat.h:203
(define-cpp-class cl_decoded_dfloat (<class-proxy>)
  ((mantissa type: cl_DF cpp-name: "mantissa")
   (exponent type: cl_I cpp-name: "exponent")
   (sign type: cl_DF cpp-name: "sign")
   (operator= (cl_decoded_dfloat&)
              cl_decoded_dfloat&
              "operator ="
              implicitp: t))
  constructor: ()
  constructor: (const-cl_DF& const-cl_I& const-cl_DF&)
  copy-constructor: (cl_decoded_dfloat&)
  type: cl_decoded_dfloat*
  class: "cl_decoded_dfloat")
(defoverloaded decode_float (const-cl_DF&) cl_decoded_dfloat "decode_float")
(defoverloaded float_exponent (const-cl_DF&) sintL "float_exponent")
(defoverloaded float_radix (const-cl_DF&) sintL "float_radix")
(defoverloaded float_sign (const-cl_DF&) cl_DF "float_sign")
(defoverloaded float_digits (const-cl_DF&) uintL "float_digits")
