(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded cl_current_time_consumption ()
  cl_time_consumption
  "cl_current_time_consumption")
(defoverloaded cl_timing_report (ostream& const-cl_time_consumption&)
  void
  "cl_timing_report")
(define-cpp-converter |void(*)(cl_timing**)| function-type)
(define-callback-pool |void(*)(const-cl_timing&)| 5)
(define-cpp-converter cl_timing* static)
;; Definition of class cl_timing
;; At /home/bruno/cln/include/cl_timing.h:50
(define-cpp-class cl_timing (<class-proxy>)
  ((tmp type: cl_time_consumption cpp-name: "tmp")
   (report_fn type: |void(*)(const-cl_timing&)| cpp-name: "report_fn")
   (report_destination type: void* cpp-name: "report_destination")
   (comment type: const-char* cpp-name: "comment")
   (operator= (cl_timing&) cl_timing& "operator =" implicitp: t))
  constructor: (cl_time_consumption&)
  constructor: (optional: ostream&)
  constructor: (const-char* optional: ostream&)
  copy-constructor: (cl_timing&)
  type: cl_timing*
  class: "cl_timing")
