(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded round2 (const-cl_LF&) cl_LF_div_t "round2")
(defoverloaded round1 (const-cl_LF&) cl_I "round1")
(defoverloaded floor2 (const-cl_LF& const-cl_LF&) cl_LF_div_t "floor2")
(defoverloaded floor1 (const-cl_LF& const-cl_LF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_LF& const-cl_LF&) cl_LF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_LF& const-cl_LF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_LF& const-cl_LF&) cl_LF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_LF& const-cl_LF&) cl_I "truncate1")
(defoverloaded round2 (const-cl_LF& const-cl_LF&) cl_LF_div_t "round2")
(defoverloaded round1 (const-cl_LF& const-cl_LF&) cl_I "round1")
(defoverloaded cl_float (const-cl_F& const-cl_LF&) cl_LF "cl_float")
(defoverloaded cl_float (const-cl_I& const-cl_LF&) cl_LF "cl_float")
(defoverloaded cl_float (const-cl_RA& const-cl_LF&) cl_LF "cl_float")
(defoverloaded cl_float (int const-cl_LF&) cl_LF "cl_float")
(define-cpp-converter cl_decoded_lfloat* static)
;; Definition of class cl_decoded_lfloat
;; At /home/bruno/cln/include/cl_lfloat.h:280
(define-cpp-class cl_decoded_lfloat (<class-proxy>)
  ((mantissa type: cl_LF cpp-name: "mantissa")
   (exponent type: cl_I cpp-name: "exponent")
   (sign type: cl_LF cpp-name: "sign")
   (operator= (cl_decoded_lfloat&)
              cl_decoded_lfloat&
              "operator ="
              implicitp: t))
  constructor: ()
  constructor: (const-cl_LF& const-cl_I& const-cl_LF&)
  copy-constructor: (cl_decoded_lfloat&)
  type: cl_decoded_lfloat*
  class: "cl_decoded_lfloat")
(defoverloaded decode_float (const-cl_LF&) cl_decoded_lfloat "decode_float")
(defoverloaded float_exponent (const-cl_LF&) sintL "float_exponent")
(defoverloaded float_radix (const-cl_LF&) sintL "float_radix")
(defoverloaded float_sign (const-cl_LF&) cl_LF "float_sign")
(defoverloaded float_digits (const-cl_LF&) uintL "float_digits")
(defoverloaded float_precision (const-cl_LF&) uintL "float_precision")
(defoverloaded integer_decode_float (const-cl_LF&)
  cl_idecoded_float
  "integer_decode_float")
(defoverloaded scale_float (const-cl_LF& sintL) cl_LF "scale_float")
(defoverloaded scale_float (const-cl_LF& const-cl_I&) cl_LF "scale_float")
(defoverloaded signum (const-cl_LF&) cl_LF "signum")
(defoverloaded cl_float_approx (const-cl_LF&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_LF&) double "cl_double_approx")
(defoverloaded operator- (const-cl_RA&) cl_RA "operator -")
(defoverloaded operator+ (const-cl_RA& const-cl_RA&) cl_RA "operator +")
(defoverloaded operator+ (long const-cl_RA&) cl_RA "operator +")
(defoverloaded operator+ (const-cl_RA& long) cl_RA "operator +")
(defoverloaded operator- (const-cl_RA& const-cl_RA&) cl_RA "operator -")
(defoverloaded operator- (long const-cl_RA&) cl_RA "operator -")
(defoverloaded operator- (const-cl_RA& long) cl_RA "operator -")
(defoverloaded plus1 (const-cl_RA&) cl_RA "plus1")
(defoverloaded minus1 (const-cl_RA&) cl_RA "minus1")
(defoverloaded cl_equal (const-cl_RA& const-cl_RA&) cl_boolean "cl_equal")
(defoverloaded cl_compare (const-cl_RA& const-cl_RA&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_RA& const-cl_RA&) bool "operator ==")
(defoverloaded operator!= (const-cl_RA& const-cl_RA&) bool "operator !=")
(defoverloaded operator<= (const-cl_RA& const-cl_RA&) bool "operator <=")
(defoverloaded operator< (const-cl_RA& const-cl_RA&) bool "operator <")
(defoverloaded operator>= (const-cl_RA& const-cl_RA&) bool "operator >=")
(defoverloaded operator> (const-cl_RA& const-cl_RA&) bool "operator >")
(defoverloaded minusp (const-cl_RA&) cl_boolean "minusp")
(defoverloaded zerop (const-cl_RA&) cl_boolean "zerop")
(defoverloaded recip (const-cl_RA&) cl_RA "recip")
(defoverloaded operator* (const-cl_RA& const-cl_RA&) cl_RA "operator *")
(defoverloaded operator* (long const-cl_RA&) cl_RA "operator *")
