(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded scale_float (const-cl_F& sintL) cl_F "scale_float")
(defoverloaded scale_float (const-cl_F& const-cl_I&) cl_F "scale_float")
(defoverloaded signum (const-cl_F&) cl_F "signum")
(defoverloaded most_positive_float (cl_float_format_t)
  cl_F
  "most_positive_float")
(defoverloaded most_negative_float (cl_float_format_t)
  cl_F
  "most_negative_float")
(defoverloaded least_positive_float (cl_float_format_t)
  cl_F
  "least_positive_float")
(defoverloaded least_negative_float (cl_float_format_t)
  cl_F
  "least_negative_float")
(defoverloaded float_epsilon (cl_float_format_t) cl_F "float_epsilon")
(defoverloaded float_negative_epsilon (cl_float_format_t)
  cl_F
  "float_negative_epsilon")
(defoverloaded cl_float_approx (const-cl_F&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_F&) double "cl_double_approx")
(define-cpp-variable
  cl_inhibit_floating_point_underflow
  cl_boolean
  "cl_inhibit_floating_point_underflow")
(defoverloaded zerop (const-cl_R&) cl_boolean "zerop")
(defoverloaded minusp (const-cl_R&) cl_boolean "minusp")
(defoverloaded plusp (const-cl_R&) cl_boolean "plusp")
(defoverloaded cl_R_to_SF (const-cl_R&) cl_SF "cl_R_to_SF")
(defoverloaded cl_R_to_FF (const-cl_R&) cl_FF "cl_R_to_FF")
(defoverloaded cl_R_to_DF (const-cl_R&) cl_DF "cl_R_to_DF")
(defoverloaded cl_R_to_LF (const-cl_R& uintC) cl_LF "cl_R_to_LF")
(defoverloaded cl_float (const-cl_R& const-cl_F&) cl_F "cl_float")
(defoverloaded cl_float (const-cl_R& cl_float_format_t) cl_F "cl_float")
(defoverloaded cl_float (const-cl_I&) cl_F "cl_float")
(defoverloaded cl_float (const-cl_RA&) cl_F "cl_float")
(defoverloaded cl_float (const-cl_R&) cl_F "cl_float")
(defoverloaded cl_float (const-cl_F&) cl_F "cl_float")
(defoverloaded cl_float (int) cl_F "cl_float")
(defoverloaded cl_float (double) cl_F "cl_float")
(defoverloaded operator- (const-cl_R&) cl_R "operator -")
(defoverloaded operator+ (const-cl_R& const-cl_R&) cl_R "operator +")
(defoverloaded operator+ (const-cl_R& const-cl_F&) cl_F "operator +")
(defoverloaded operator+ (const-cl_F& const-cl_R&) cl_F "operator +")
(defoverloaded operator+ (const-cl_RA& const-cl_F&) cl_F "operator +")
(defoverloaded operator+ (const-cl_I& const-cl_F&) cl_F "operator +")
(defoverloaded operator+ (const-cl_F& const-cl_RA&) cl_F "operator +")
(defoverloaded operator+ (const-cl_F& const-cl_I&) cl_F "operator +")
(defoverloaded operator+ (long const-cl_R&) cl_R "operator +")
(defoverloaded operator+ (const-cl_R& long) cl_R "operator +")
(defoverloaded operator+ (long const-cl_F&) cl_F "operator +")
(defoverloaded operator+ (const-cl_F& long) cl_F "operator +")
(defoverloaded operator- (const-cl_R& const-cl_R&) cl_R "operator -")
(defoverloaded operator- (const-cl_R& const-cl_F&) cl_F "operator -")
(defoverloaded operator- (const-cl_F& const-cl_R&) cl_F "operator -")
(defoverloaded operator- (const-cl_RA& const-cl_F&) cl_F "operator -")
(defoverloaded operator- (const-cl_I& const-cl_F&) cl_F "operator -")
(defoverloaded operator- (const-cl_F& const-cl_RA&) cl_F "operator -")
(defoverloaded operator- (const-cl_F& const-cl_I&) cl_F "operator -")
(defoverloaded operator- (long const-cl_R&) cl_R "operator -")
(defoverloaded operator- (const-cl_R& long) cl_R "operator -")
(defoverloaded operator- (long const-cl_F&) cl_F "operator -")
(defoverloaded operator- (const-cl_F& long) cl_F "operator -")
