(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded sqrt (const-cl_N&) cl_N "sqrt")
(defoverloaded cl_equal (const-cl_N& const-cl_N&) cl_boolean "cl_equal")
(defoverloaded operator== (const-cl_N& const-cl_N&) bool "operator ==")
(defoverloaded operator!= (const-cl_N& const-cl_N&) bool "operator !=")
(defoverloaded phase (const-cl_N&) cl_R "phase")
(defoverloaded exp (const-cl_N&) cl_N "exp")
(defoverloaded log (const-cl_N&) cl_N "log")
(defoverloaded log (const-cl_N& const-cl_N&) cl_N "log")
(defoverloaded expt (const-cl_N& sintL) cl_N "expt")
(defoverloaded expt (const-cl_N& const-cl_I&) cl_N "expt")
(defoverloaded expt (const-cl_N& const-cl_N&) cl_N "expt")
(defoverloaded sin (const-cl_N&) cl_N "sin")
(defoverloaded cos (const-cl_N&) cl_N "cos")
(defoverloaded tan (const-cl_N&) cl_N "tan")
(defoverloaded cis (const-cl_R&) cl_N "cis")
(defoverloaded cis (const-cl_N&) cl_N "cis")
(defoverloaded sinh (const-cl_N&) cl_N "sinh")
(defoverloaded cosh (const-cl_N&) cl_N "cosh")
(defoverloaded tanh (const-cl_N&) cl_N "tanh")
(defoverloaded atan (const-cl_N&) cl_N "atan")
(defoverloaded atanh (const-cl_N&) cl_N "atanh")
(defoverloaded asin (const-cl_N&) cl_N "asin")
(defoverloaded asinh (const-cl_N&) cl_N "asinh")
(defoverloaded acos (const-cl_N&) cl_N "acos")
(defoverloaded acosh (const-cl_N&) cl_N "acosh")
(define-cpp-converter cl_random_state* static)
;; Definition of class cl_random_state
;; At /home/bruno/cln/include/cl_random.h:8
(define-cpp-class cl_random_state (<class-proxy>)
  ()
  constructor: ()
  type: cl_random_state*
  class: "cl_random_state")
(defoverloaded random32 (cl_random_state&) uint32 "random32")
(defoverloaded random_I (cl_random_state& const-cl_I&) cl_I "random_I")
(defoverloaded random_F (cl_random_state& const-cl_F&) cl_F "random_F")
(defoverloaded random_R (cl_random_state& const-cl_R&) cl_R "random_R")
(defoverloaded random32 () uint32 "random32")
(defoverloaded random_I (const-cl_I&) cl_I "random_I")
(defoverloaded random_F (const-cl_F&) cl_F "random_F")
(defoverloaded random_R (const-cl_R&) cl_R "random_R")
(defoverloaded cl_abort () void "cl_abort")
(define-cpp-variable cl_debugout_stream ostream* "cl_debugout_stream")
(defoverloaded freadchar (istream&) int "freadchar")
(defoverloaded funreadchar (istream& int) int "funreadchar")
(defoverloaded fprintchar (ostream& char) void "fprintchar")
(defoverloaded fprint (ostream& const-char*) void "fprint")
(defoverloaded fprintdecimal (ostream& int) void "fprintdecimal")
(defoverloaded fprint (ostream& const-cl_number&) void "fprint")
(defoverloaded fprint (ostream& const-cl_R&) void "fprint")
(defoverloaded fprint (ostream& const-cl_RA&) void "fprint")
(defoverloaded fprint (ostream& const-cl_I&) void "fprint")
(defoverloaded fprint (ostream& long) void "fprint")
(defoverloaded fprintdecimal (ostream& const-cl_I&) void "fprintdecimal")
(defoverloaded fprintbinary (ostream& const-cl_I&) void "fprintbinary")
(defoverloaded fprintoctal (ostream& const-cl_I&) void "fprintoctal")
