(define-cpp-type cl_print_complex_flags* static
  id: cl_print_complex_flags*
  class: cl_print_complex_flags
  cpp-type-dcl: "cl_print_complex_flags *%s"
  cast: "((cl_print_complex_flags *)(%s))"
  callback-res-converter: "(cl_print_complex_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_print_number_flags complex_readably)
(define-cpp-data-slot-setter cl_print_number_flags float_readably)
(define-cpp-data-slot-setter cl_print_number_flags rational_base)
(define-cpp-data-slot-setter cl_print_number_flags default_float_format)
(define-cpp-data-slot-setter cl_print_number_flags rational_readably)
(define-cpp-type cl_print_number_flags& static
  id: cl_print_number_flags*
  class: cl_print_number_flags
  cpp-type-dcl: "cl_print_number_flags &%s"
  cast: "((cl_print_number_flags &)(%s))"
  arg-type: "cl_print_number_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_number_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_number_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_number_flags static
  id: cl_print_number_flags*
  class: cl_print_number_flags
  cpp-type-dcl: "cl_print_number_flags %s"
  cast: "((cl_print_number_flags)(%s))"
  arg-type: "cl_print_number_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_number_flags *%s"
  res-converter: "new cl_print_number_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_number_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_number_flags* static
  id: cl_print_number_flags*
  class: cl_print_number_flags
  cpp-type-dcl: "cl_print_number_flags *%s"
  cast: "((cl_print_number_flags *)(%s))"
  callback-res-converter: "(cl_print_number_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_vector_syntax_t enum
  cpp-arg-type: "((cl_print_vector_syntax_t)(%s))"
  id: cl_print_vector_syntax_t)
(define-cpp-data-slot-setter cl_print_vector_flags vector_syntax)
(define-cpp-type cl_print_vector_flags& static
  id: cl_print_vector_flags*
  class: cl_print_vector_flags
  cpp-type-dcl: "cl_print_vector_flags &%s"
  cast: "((cl_print_vector_flags &)(%s))"
  arg-type: "cl_print_vector_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_vector_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_vector_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_vector_flags static
  id: cl_print_vector_flags*
  class: cl_print_vector_flags
  cpp-type-dcl: "cl_print_vector_flags %s"
  cast: "((cl_print_vector_flags)(%s))"
  arg-type: "cl_print_vector_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_vector_flags *%s"
  res-converter: "new cl_print_vector_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_vector_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_vector_flags* static
  id: cl_print_vector_flags*
  class: cl_print_vector_flags
  cpp-type-dcl: "cl_print_vector_flags *%s"
  cast: "((cl_print_vector_flags *)(%s))"
  callback-res-converter: "(cl_print_vector_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_print_univpoly_flags univpoly_varname)
(define-cpp-type cl_print_univpoly_flags& static
  id: cl_print_univpoly_flags*
  class: cl_print_univpoly_flags
  cpp-type-dcl: "cl_print_univpoly_flags &%s"
  cast: "((cl_print_univpoly_flags &)(%s))"
  arg-type: "cl_print_univpoly_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_univpoly_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_univpoly_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_univpoly_flags static
  id: cl_print_univpoly_flags*
  class: cl_print_univpoly_flags
  cpp-type-dcl: "cl_print_univpoly_flags %s"
  cast: "((cl_print_univpoly_flags)(%s))"
  arg-type: "cl_print_univpoly_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_univpoly_flags *%s"
  res-converter: "new cl_print_univpoly_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_univpoly_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_univpoly_flags* static
  id: cl_print_univpoly_flags*
  class: cl_print_univpoly_flags
  cpp-type-dcl: "cl_print_univpoly_flags *%s"
  cast: "((cl_print_univpoly_flags *)(%s))"
  callback-res-converter: "(cl_print_univpoly_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_print_flags vector_syntax)
(define-cpp-data-slot-setter cl_print_flags complex_readably)
(define-cpp-data-slot-setter cl_print_flags float_readably)
(define-cpp-data-slot-setter cl_print_flags rational_base)
(define-cpp-data-slot-setter cl_print_flags rational_readably)
(define-cpp-data-slot-setter cl_print_flags univpoly_varname)
(define-cpp-data-slot-setter cl_print_flags default_float_format)
(define-cpp-type cl_print_flags& static
  id: cl_print_flags*
  class: cl_print_flags
  cpp-type-dcl: "cl_print_flags &%s"
  cast: "((cl_print_flags &)(%s))"
  arg-type: "cl_print_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_flags static
  id: cl_print_flags*
  class: cl_print_flags
  cpp-type-dcl: "cl_print_flags %s"
  cast: "((cl_print_flags)(%s))"
  arg-type: "cl_print_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_flags *%s"
  res-converter: "new cl_print_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_flags* static
  id: cl_print_flags*
  class: cl_print_flags
  cpp-type-dcl: "cl_print_flags *%s"
  cast: "((cl_print_flags *)(%s))"
  callback-res-converter: "(cl_print_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-variable-setter cl_default_print_flags)
(define-cpp-type const-cl_print_flags& static
  id: cl_print_flags*
  class: cl_print_flags
  cpp-type-dcl: "const cl_print_flags &%s"
  cast: "((const cl_print_flags &)(%s))"
  arg-type: "const cl_print_flags *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_print_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_print_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_print_number_flags& static
  id: cl_print_number_flags*
  class: cl_print_number_flags
  cpp-type-dcl: "const cl_print_number_flags &%s"
  cast: "((const cl_print_number_flags &)(%s))"
  arg-type: "const cl_print_number_flags *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_print_number_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_print_number_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_print_real_flags& static
  id: cl_print_real_flags*
  class: cl_print_real_flags
  cpp-type-dcl: "const cl_print_real_flags &%s"
  cast: "((const cl_print_real_flags &)(%s))"
  arg-type: "const cl_print_real_flags *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_print_real_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_print_real_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_print_rational_flags& static
  id: cl_print_rational_flags*
  class: cl_print_rational_flags
  cpp-type-dcl: "const cl_print_rational_flags &%s"
  cast: "((const cl_print_rational_flags &)(%s))"
  arg-type: "const cl_print_rational_flags *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_print_rational_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_print_rational_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_condition& static
  id: cl_condition*
  class: cl_condition
  cpp-type-dcl: "cl_condition &%s"
  cast: "((cl_condition &)(%s))"
  arg-type: "cl_condition *%s"
  arg-converter: "*(%s)"
  res-type: "cl_condition *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_condition *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_condition static
  id: cl_condition*
  class: cl_condition
  cpp-type-dcl: "cl_condition %s"
  cast: "((cl_condition)(%s))"
  arg-type: "cl_condition *%s"
  arg-converter: "*(%s)"
  res-type: "cl_condition *%s"
  res-converter: "new cl_condition (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_condition *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_condition* static
  id: cl_condition*
  class: cl_condition
  cpp-type-dcl: "cl_condition *%s"
  cast: "((cl_condition *)(%s))"
  callback-res-converter: "(cl_condition *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_symbol& static
  id: cl_symbol*
  class: cl_symbol
  cpp-type-dcl: "const cl_symbol &%s"
  cast: "((const cl_symbol &)(%s))"
  arg-type: "const cl_symbol *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_symbol *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_symbol *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_symbol& static
  id: cl_symbol*
  class: cl_symbol
  cpp-type-dcl: "cl_symbol &%s"
  cast: "((cl_symbol &)(%s))"
  arg-type: "cl_symbol *%s"
  arg-converter: "*(%s)"
  res-type: "cl_symbol *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_symbol *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_symbol static
  id: cl_symbol*
  class: cl_symbol
  cpp-type-dcl: "cl_symbol %s"
  cast: "((cl_symbol)(%s))"
  arg-type: "cl_symbol *%s"
  arg-converter: "*(%s)"
  res-type: "cl_symbol *%s"
  res-converter: "new cl_symbol (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_symbol *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_symbol* static
  id: cl_symbol*
  class: cl_symbol
  cpp-type-dcl: "cl_symbol *%s"
  cast: "((cl_symbol *)(%s))"
  callback-res-converter: "(cl_symbol *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_property key)
(define-cpp-type cl_property& static
  id: cl_property*
  class: cl_property
  cpp-type-dcl: "cl_property &%s"
  cast: "((cl_property &)(%s))"
  arg-type: "cl_property *%s"
  arg-converter: "*(%s)"
  res-type: "cl_property *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_property *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_property static
  id: cl_property*
  class: cl_property
  cpp-type-dcl: "cl_property %s"
  cast: "((cl_property)(%s))"
  arg-type: "cl_property *%s"
  arg-converter: "*(%s)"
  res-type: "cl_property *%s"
  res-converter: "new cl_property (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_property *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_property* static
  id: cl_property*
  class: cl_property
  cpp-type-dcl: "cl_property *%s"
  cast: "((cl_property *)(%s))"
  callback-res-converter: "(cl_property *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_property_list& static
  id: cl_property_list*
  class: cl_property_list
  cpp-type-dcl: "cl_property_list &%s"
  cast: "((cl_property_list &)(%s))"
  arg-type: "cl_property_list *%s"
  arg-converter: "*(%s)"
  res-type: "cl_property_list *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_property_list *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_property_list static
  id: cl_property_list*
  class: cl_property_list
  cpp-type-dcl: "cl_property_list %s"
  cast: "((cl_property_list)(%s))"
  arg-type: "cl_property_list *%s"
  arg-converter: "*(%s)"
  res-type: "cl_property_list *%s"
  res-converter: "new cl_property_list (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_property_list *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_property_list* static
  id: cl_property_list*
  class: cl_property_list
  cpp-type-dcl: "cl_property_list *%s"
  cast: "((cl_property_list *)(%s))"
  callback-res-converter: "(cl_property_list *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_ring& static
  id: cl_ring*
  class: cl_ring
  cpp-type-dcl: "const cl_ring &%s"
  cast: "((const cl_ring &)(%s))"
  arg-type: "const cl_ring *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_ring& static
  id: cl_ring*
  class: cl_ring
  cpp-type-dcl: "cl_ring &%s"
  cast: "((cl_ring &)(%s))"
  arg-type: "cl_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_ring* static
  id: cl_heap_ring*
  class: cl_heap_ring
  cpp-type-dcl: "cl_heap_ring *%s"
  cast: "((cl_heap_ring *)(%s))"
  callback-res-converter: "(cl_heap_ring *)(%s)"
  c-callback-arg-type: "void *%s")
