(define-cpp-type cl_F& static
  id: cl_F*
  class: cl_F
  cpp-type-dcl: "cl_F &%s"
  cast: "((cl_F &)(%s))"
  arg-type: "cl_F *%s"
  arg-converter: "*(%s)"
  res-type: "cl_F *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_F *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_F* static
  id: cl_F*
  class: cl_F
  cpp-type-dcl: "cl_F *%s"
  cast: "((cl_F *)(%s))"
  callback-res-converter: "(cl_F *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_F static
  id: cl_F*
  class: cl_F
  cpp-type-dcl: "cl_F %s"
  cast: "((cl_F)(%s))"
  arg-type: "cl_F *%s"
  arg-converter: "*(%s)"
  res-type: "cl_F *%s"
  res-converter: "new cl_F (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_F *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_SF& static
  id: cl_SF*
  class: cl_SF
  cpp-type-dcl: "cl_SF &%s"
  cast: "((cl_SF &)(%s))"
  arg-type: "cl_SF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_SF *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_SF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_SF* static
  id: cl_SF*
  class: cl_SF
  cpp-type-dcl: "cl_SF *%s"
  cast: "((cl_SF *)(%s))"
  callback-res-converter: "(cl_SF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_SF static
  id: cl_SF*
  class: cl_SF
  cpp-type-dcl: "cl_SF %s"
  cast: "((cl_SF)(%s))"
  arg-type: "cl_SF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_SF *%s"
  res-converter: "new cl_SF (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_SF *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_FF& static
  id: cl_FF*
  class: cl_FF
  cpp-type-dcl: "cl_FF &%s"
  cast: "((cl_FF &)(%s))"
  arg-type: "cl_FF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_FF *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_FF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_FF* static
  id: cl_FF*
  class: cl_FF
  cpp-type-dcl: "cl_FF *%s"
  cast: "((cl_FF *)(%s))"
  callback-res-converter: "(cl_FF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_FF static
  id: cl_FF*
  class: cl_FF
  cpp-type-dcl: "cl_FF %s"
  cast: "((cl_FF)(%s))"
  arg-type: "cl_FF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_FF *%s"
  res-converter: "new cl_FF (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_FF *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_DF& static
  id: cl_DF*
  class: cl_DF
  cpp-type-dcl: "cl_DF &%s"
  cast: "((cl_DF &)(%s))"
  arg-type: "cl_DF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_DF *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_DF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_DF* static
  id: cl_DF*
  class: cl_DF
  cpp-type-dcl: "cl_DF *%s"
  cast: "((cl_DF *)(%s))"
  callback-res-converter: "(cl_DF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_DF static
  id: cl_DF*
  class: cl_DF
  cpp-type-dcl: "cl_DF %s"
  cast: "((cl_DF)(%s))"
  arg-type: "cl_DF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_DF *%s"
  res-converter: "new cl_DF (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_DF *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_LF& static
  id: cl_LF*
  class: cl_LF
  cpp-type-dcl: "cl_LF &%s"
  cast: "((cl_LF &)(%s))"
  arg-type: "cl_LF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_LF *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_LF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_LF* static
  id: cl_LF*
  class: cl_LF
  cpp-type-dcl: "cl_LF *%s"
  cast: "((cl_LF *)(%s))"
  callback-res-converter: "(cl_LF *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_LF static
  id: cl_LF*
  class: cl_LF
  cpp-type-dcl: "cl_LF %s"
  cast: "((cl_LF)(%s))"
  arg-type: "cl_LF *%s"
  arg-converter: "*(%s)"
  res-type: "cl_LF *%s"
  res-converter: "new cl_LF (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_LF *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_RA& static
  id: cl_RA*
  class: cl_RA
  cpp-type-dcl: "cl_RA &%s"
  cast: "((cl_RA &)(%s))"
  arg-type: "cl_RA *%s"
  arg-converter: "*(%s)"
  res-type: "cl_RA *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_RA *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_RA* static
  id: cl_RA*
  class: cl_RA
  cpp-type-dcl: "cl_RA *%s"
  cast: "((cl_RA *)(%s))"
  callback-res-converter: "(cl_RA *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_RA static
  id: cl_RA*
  class: cl_RA
  cpp-type-dcl: "cl_RA %s"
  cast: "((cl_RA)(%s))"
  arg-type: "cl_RA *%s"
  arg-converter: "*(%s)"
  res-type: "cl_RA *%s"
  res-converter: "new cl_RA (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_RA *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_I& static
  id: cl_I*
  class: cl_I
  cpp-type-dcl: "cl_I &%s"
  cast: "((cl_I &)(%s))"
  arg-type: "cl_I *%s"
  arg-converter: "*(%s)"
  res-type: "cl_I *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_I *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_I* static
  id: cl_I*
  class: cl_I
  cpp-type-dcl: "cl_I *%s"
  cast: "((cl_I *)(%s))"
  callback-res-converter: "(cl_I *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_I static
  id: cl_I*
  class: cl_I
  cpp-type-dcl: "cl_I %s"
  cast: "((cl_I)(%s))"
  arg-type: "cl_I *%s"
  arg-converter: "*(%s)"
  res-type: "cl_I *%s"
  res-converter: "new cl_I (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_I *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-variable-setter cl_FF_0)
(define-cpp-variable-setter cl_DF_0)
(define-cpp-variable-setter cl_LF_0)
(define-cpp-data-slot-setter cl_idecoded_float mantissa)
(define-cpp-data-slot-setter cl_idecoded_float exponent)
(define-cpp-data-slot-setter cl_idecoded_float sign)
(define-cpp-type cl_idecoded_float& static
  id: cl_idecoded_float*
  class: cl_idecoded_float
  cpp-type-dcl: "cl_idecoded_float &%s"
  cast: "((cl_idecoded_float &)(%s))"
  arg-type: "cl_idecoded_float *%s"
  arg-converter: "*(%s)"
  res-type: "cl_idecoded_float *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_idecoded_float *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_idecoded_float static
  id: cl_idecoded_float*
  class: cl_idecoded_float
  cpp-type-dcl: "cl_idecoded_float %s"
  cast: "((cl_idecoded_float)(%s))"
  arg-type: "cl_idecoded_float *%s"
  arg-converter: "*(%s)"
  res-type: "cl_idecoded_float *%s"
  res-converter: "new cl_idecoded_float (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_idecoded_float *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_idecoded_float* static
  id: cl_idecoded_float*
  class: cl_idecoded_float
  cpp-type-dcl: "cl_idecoded_float *%s"
  cast: "((cl_idecoded_float *)(%s))"
  callback-res-converter: "(cl_idecoded_float *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_float_format_t enum
  cpp-arg-type: "((cl_float_format_t)(%s))"
  id: cl_float_format_t)
(define-cpp-type cl_signean int
  cpp-type-dcl: "cl_signean %s"
  cpp-arg-type: "((cl_signean)(%s))"
  cast: "((cl_signean)(%s))")
(define-cpp-data-slot-setter cl_F_fdiv_t quotient)
(define-cpp-data-slot-setter cl_F_fdiv_t remainder)
(define-cpp-type cl_F_fdiv_t& static
  id: cl_F_fdiv_t*
  class: cl_F_fdiv_t
  cpp-type-dcl: "cl_F_fdiv_t &%s"
  cast: "((cl_F_fdiv_t &)(%s))"
  arg-type: "cl_F_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_F_fdiv_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_F_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_F_fdiv_t static
  id: cl_F_fdiv_t*
  class: cl_F_fdiv_t
  cpp-type-dcl: "cl_F_fdiv_t %s"
  cast: "((cl_F_fdiv_t)(%s))"
  arg-type: "cl_F_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_F_fdiv_t *%s"
  res-converter: "new cl_F_fdiv_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_F_fdiv_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_F_fdiv_t* static
  id: cl_F_fdiv_t*
  class: cl_F_fdiv_t
  cpp-type-dcl: "cl_F_fdiv_t *%s"
  cast: "((cl_F_fdiv_t *)(%s))"
  callback-res-converter: "(cl_F_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_F_div_t quotient)
(define-cpp-data-slot-setter cl_F_div_t remainder)
(define-cpp-type cl_F_div_t& static
  id: cl_F_div_t*
  class: cl_F_div_t
  cpp-type-dcl: "cl_F_div_t &%s"
  cast: "((cl_F_div_t &)(%s))"
  arg-type: "cl_F_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_F_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_F_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_F_div_t static
  id: cl_F_div_t*
  class: cl_F_div_t
  cpp-type-dcl: "cl_F_div_t %s"
  cast: "((cl_F_div_t)(%s))"
  arg-type: "cl_F_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_F_div_t *%s"
  res-converter: "new cl_F_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_F_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_F_div_t* static
  id: cl_F_div_t*
  class: cl_F_div_t
  cpp-type-dcl: "cl_F_div_t *%s"
  cast: "((cl_F_div_t *)(%s))"
  callback-res-converter: "(cl_F_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-variable-setter cl_default_float_format)
(define-cpp-type uintL unsigned-long
  cpp-type-dcl: "uintL %s"
  cpp-arg-type: "((uintL)(%s))"
  cast: "((uintL)(%s))")
(define-cpp-data-slot-setter cl_decoded_float mantissa)
(define-cpp-data-slot-setter cl_decoded_float exponent)
(define-cpp-data-slot-setter cl_decoded_float sign)
(define-cpp-type cl_decoded_float& static
  id: cl_decoded_float*
  class: cl_decoded_float
  cpp-type-dcl: "cl_decoded_float &%s"
  cast: "((cl_decoded_float &)(%s))"
  arg-type: "cl_decoded_float *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_float *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_float *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_float static
  id: cl_decoded_float*
  class: cl_decoded_float
  cpp-type-dcl: "cl_decoded_float %s"
  cast: "((cl_decoded_float)(%s))"
  arg-type: "cl_decoded_float *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_float *%s"
  res-converter: "new cl_decoded_float (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_float *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_float* static
  id: cl_decoded_float*
  class: cl_decoded_float
  cpp-type-dcl: "cl_decoded_float *%s"
  cast: "((cl_decoded_float *)(%s))"
  callback-res-converter: "(cl_decoded_float *)(%s)"
  c-callback-arg-type: "void *%s")
