(define-cpp-type const-cl_univpoly_integer_ring& static
  id: cl_univpoly_integer_ring*
  class: cl_univpoly_integer_ring
  cpp-type-dcl: "const cl_univpoly_integer_ring &%s"
  cast: "((const cl_univpoly_integer_ring &)(%s))"
  arg-type: "const cl_univpoly_integer_ring *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_univpoly_integer_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_univpoly_integer_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_integer_ring& static
  id: cl_univpoly_integer_ring*
  class: cl_univpoly_integer_ring
  cpp-type-dcl: "cl_univpoly_integer_ring &%s"
  cast: "((cl_univpoly_integer_ring &)(%s))"
  arg-type: "cl_univpoly_integer_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_integer_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_integer_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_integer_ring* static
  id: cl_heap_univpoly_integer_ring*
  class: cl_heap_univpoly_integer_ring
  cpp-type-dcl: "cl_heap_univpoly_integer_ring *%s"
  cast: "((cl_heap_univpoly_integer_ring *)(%s))"
  callback-res-converter: "(cl_heap_univpoly_integer_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_integer_ring static
  id: cl_univpoly_integer_ring*
  class: cl_univpoly_integer_ring
  cpp-type-dcl: "cl_univpoly_integer_ring %s"
  cast: "((cl_univpoly_integer_ring)(%s))"
  arg-type: "cl_univpoly_integer_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_integer_ring *%s"
  res-converter: "new cl_univpoly_integer_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_integer_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_integer_ring* static
  id: cl_univpoly_integer_ring*
  class: cl_univpoly_integer_ring
  cpp-type-dcl: "cl_univpoly_integer_ring *%s"
  cast: "((cl_univpoly_integer_ring *)(%s))"
  callback-res-converter: "(cl_univpoly_integer_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_I* static
  id: cl_UP_I*
  class: cl_UP_I
  cpp-type-dcl: "cl_UP_I *%s"
  cast: "((cl_UP_I *)(%s))"
  callback-res-converter: "(cl_UP_I *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_I static
  id: cl_UP_I*
  class: cl_UP_I
  cpp-type-dcl: "cl_UP_I %s"
  cast: "((cl_UP_I)(%s))"
  arg-type: "cl_UP_I *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_I *%s"
  res-converter: "new cl_UP_I (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_I *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_I& static
  id: cl_UP_I*
  class: cl_UP_I
  cpp-type-dcl: "cl_UP_I &%s"
  cast: "((cl_UP_I &)(%s))"
  arg-type: "cl_UP_I *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_I *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_I *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_UP_I& static
  id: cl_UP_I*
  class: cl_UP_I
  cpp-type-dcl: "const cl_UP_I &%s"
  cast: "((const cl_UP_I &)(%s))"
  arg-type: "const cl_UP_I *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_UP_I *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_UP_I *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_integer_ring& static
  id: cl_heap_univpoly_integer_ring*
  class: cl_heap_univpoly_integer_ring
  cpp-type-dcl: "cl_heap_univpoly_integer_ring &%s"
  cast: "((cl_heap_univpoly_integer_ring &)(%s))"
  arg-type: "cl_heap_univpoly_integer_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_integer_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_integer_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_integer_ring static
  id: cl_heap_univpoly_integer_ring*
  class: cl_heap_univpoly_integer_ring
  cpp-type-dcl: "cl_heap_univpoly_integer_ring %s"
  cast: "((cl_heap_univpoly_integer_ring)(%s))"
  arg-type: "cl_heap_univpoly_integer_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_integer_ring *%s"
  res-converter: "new cl_heap_univpoly_integer_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_integer_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_univpoly_rational_ring& static
  id: cl_univpoly_rational_ring*
  class: cl_univpoly_rational_ring
  cpp-type-dcl: "const cl_univpoly_rational_ring &%s"
  cast: "((const cl_univpoly_rational_ring &)(%s))"
  arg-type: "const cl_univpoly_rational_ring *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_univpoly_rational_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_univpoly_rational_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_rational_ring& static
  id: cl_univpoly_rational_ring*
  class: cl_univpoly_rational_ring
  cpp-type-dcl: "cl_univpoly_rational_ring &%s"
  cast: "((cl_univpoly_rational_ring &)(%s))"
  arg-type: "cl_univpoly_rational_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_rational_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_rational_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_rational_ring* static
  id: cl_heap_univpoly_rational_ring*
  class: cl_heap_univpoly_rational_ring
  cpp-type-dcl: "cl_heap_univpoly_rational_ring *%s"
  cast: "((cl_heap_univpoly_rational_ring *)(%s))"
  callback-res-converter: "(cl_heap_univpoly_rational_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_rational_ring static
  id: cl_univpoly_rational_ring*
  class: cl_univpoly_rational_ring
  cpp-type-dcl: "cl_univpoly_rational_ring %s"
  cast: "((cl_univpoly_rational_ring)(%s))"
  arg-type: "cl_univpoly_rational_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_rational_ring *%s"
  res-converter: "new cl_univpoly_rational_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_rational_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_rational_ring* static
  id: cl_univpoly_rational_ring*
  class: cl_univpoly_rational_ring
  cpp-type-dcl: "cl_univpoly_rational_ring *%s"
  cast: "((cl_univpoly_rational_ring *)(%s))"
  callback-res-converter: "(cl_univpoly_rational_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_RA* static
  id: cl_UP_RA*
  class: cl_UP_RA
  cpp-type-dcl: "cl_UP_RA *%s"
  cast: "((cl_UP_RA *)(%s))"
  callback-res-converter: "(cl_UP_RA *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_RA static
  id: cl_UP_RA*
  class: cl_UP_RA
  cpp-type-dcl: "cl_UP_RA %s"
  cast: "((cl_UP_RA)(%s))"
  arg-type: "cl_UP_RA *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_RA *%s"
  res-converter: "new cl_UP_RA (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_RA *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_RA& static
  id: cl_UP_RA*
  class: cl_UP_RA
  cpp-type-dcl: "cl_UP_RA &%s"
  cast: "((cl_UP_RA &)(%s))"
  arg-type: "cl_UP_RA *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_RA *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_RA *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_UP_RA& static
  id: cl_UP_RA*
  class: cl_UP_RA
  cpp-type-dcl: "const cl_UP_RA &%s"
  cast: "((const cl_UP_RA &)(%s))"
  arg-type: "const cl_UP_RA *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_UP_RA *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_UP_RA *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_rational_ring& static
  id: cl_heap_univpoly_rational_ring*
  class: cl_heap_univpoly_rational_ring
  cpp-type-dcl: "cl_heap_univpoly_rational_ring &%s"
  cast: "((cl_heap_univpoly_rational_ring &)(%s))"
  arg-type: "cl_heap_univpoly_rational_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_rational_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_rational_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_rational_ring static
  id: cl_heap_univpoly_rational_ring*
  class: cl_heap_univpoly_rational_ring
  cpp-type-dcl: "cl_heap_univpoly_rational_ring %s"
  cast: "((cl_heap_univpoly_rational_ring)(%s))"
  arg-type: "cl_heap_univpoly_rational_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_rational_ring *%s"
  res-converter: "new cl_heap_univpoly_rational_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_rational_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_univpoly_real_ring& static
  id: cl_univpoly_real_ring*
  class: cl_univpoly_real_ring
  cpp-type-dcl: "const cl_univpoly_real_ring &%s"
  cast: "((const cl_univpoly_real_ring &)(%s))"
  arg-type: "const cl_univpoly_real_ring *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_univpoly_real_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_univpoly_real_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_real_ring& static
  id: cl_univpoly_real_ring*
  class: cl_univpoly_real_ring
  cpp-type-dcl: "cl_univpoly_real_ring &%s"
  cast: "((cl_univpoly_real_ring &)(%s))"
  arg-type: "cl_univpoly_real_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_real_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_real_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_real_ring* static
  id: cl_heap_univpoly_real_ring*
  class: cl_heap_univpoly_real_ring
  cpp-type-dcl: "cl_heap_univpoly_real_ring *%s"
  cast: "((cl_heap_univpoly_real_ring *)(%s))"
  callback-res-converter: "(cl_heap_univpoly_real_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_real_ring static
  id: cl_univpoly_real_ring*
  class: cl_univpoly_real_ring
  cpp-type-dcl: "cl_univpoly_real_ring %s"
  cast: "((cl_univpoly_real_ring)(%s))"
  arg-type: "cl_univpoly_real_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_real_ring *%s"
  res-converter: "new cl_univpoly_real_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_real_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_real_ring* static
  id: cl_univpoly_real_ring*
  class: cl_univpoly_real_ring
  cpp-type-dcl: "cl_univpoly_real_ring *%s"
  cast: "((cl_univpoly_real_ring *)(%s))"
  callback-res-converter: "(cl_univpoly_real_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_R* static
  id: cl_UP_R*
  class: cl_UP_R
  cpp-type-dcl: "cl_UP_R *%s"
  cast: "((cl_UP_R *)(%s))"
  callback-res-converter: "(cl_UP_R *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_R static
  id: cl_UP_R*
  class: cl_UP_R
  cpp-type-dcl: "cl_UP_R %s"
  cast: "((cl_UP_R)(%s))"
  arg-type: "cl_UP_R *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_R *%s"
  res-converter: "new cl_UP_R (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_R *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_R& static
  id: cl_UP_R*
  class: cl_UP_R
  cpp-type-dcl: "cl_UP_R &%s"
  cast: "((cl_UP_R &)(%s))"
  arg-type: "cl_UP_R *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_R *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_R *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_UP_R& static
  id: cl_UP_R*
  class: cl_UP_R
  cpp-type-dcl: "const cl_UP_R &%s"
  cast: "((const cl_UP_R &)(%s))"
  arg-type: "const cl_UP_R *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_UP_R *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_UP_R *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_real_ring& static
  id: cl_heap_univpoly_real_ring*
  class: cl_heap_univpoly_real_ring
  cpp-type-dcl: "cl_heap_univpoly_real_ring &%s"
  cast: "((cl_heap_univpoly_real_ring &)(%s))"
  arg-type: "cl_heap_univpoly_real_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_real_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_real_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_real_ring static
  id: cl_heap_univpoly_real_ring*
  class: cl_heap_univpoly_real_ring
  cpp-type-dcl: "cl_heap_univpoly_real_ring %s"
  cast: "((cl_heap_univpoly_real_ring)(%s))"
  arg-type: "cl_heap_univpoly_real_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_real_ring *%s"
  res-converter: "new cl_heap_univpoly_real_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_real_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_univpoly_complex_ring& static
  id: cl_univpoly_complex_ring*
  class: cl_univpoly_complex_ring
  cpp-type-dcl: "const cl_univpoly_complex_ring &%s"
  cast: "((const cl_univpoly_complex_ring &)(%s))"
  arg-type: "const cl_univpoly_complex_ring *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_univpoly_complex_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_univpoly_complex_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_complex_ring& static
  id: cl_univpoly_complex_ring*
  class: cl_univpoly_complex_ring
  cpp-type-dcl: "cl_univpoly_complex_ring &%s"
  cast: "((cl_univpoly_complex_ring &)(%s))"
  arg-type: "cl_univpoly_complex_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_complex_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_complex_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_complex_ring* static
  id: cl_heap_univpoly_complex_ring*
  class: cl_heap_univpoly_complex_ring
  cpp-type-dcl: "cl_heap_univpoly_complex_ring *%s"
  cast: "((cl_heap_univpoly_complex_ring *)(%s))"
  callback-res-converter: "(cl_heap_univpoly_complex_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_complex_ring static
  id: cl_univpoly_complex_ring*
  class: cl_univpoly_complex_ring
  cpp-type-dcl: "cl_univpoly_complex_ring %s"
  cast: "((cl_univpoly_complex_ring)(%s))"
  arg-type: "cl_univpoly_complex_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_complex_ring *%s"
  res-converter: "new cl_univpoly_complex_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_complex_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_complex_ring* static
  id: cl_univpoly_complex_ring*
  class: cl_univpoly_complex_ring
  cpp-type-dcl: "cl_univpoly_complex_ring *%s"
  cast: "((cl_univpoly_complex_ring *)(%s))"
  callback-res-converter: "(cl_univpoly_complex_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_N* static
  id: cl_UP_N*
  class: cl_UP_N
  cpp-type-dcl: "cl_UP_N *%s"
  cast: "((cl_UP_N *)(%s))"
  callback-res-converter: "(cl_UP_N *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_N static
  id: cl_UP_N*
  class: cl_UP_N
  cpp-type-dcl: "cl_UP_N %s"
  cast: "((cl_UP_N)(%s))"
  arg-type: "cl_UP_N *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_N *%s"
  res-converter: "new cl_UP_N (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_N *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_N& static
  id: cl_UP_N*
  class: cl_UP_N
  cpp-type-dcl: "cl_UP_N &%s"
  cast: "((cl_UP_N &)(%s))"
  arg-type: "cl_UP_N *%s"
  arg-converter: "*(%s)"
  res-type: "cl_UP_N *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_UP_N *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_UP_N& static
  id: cl_UP_N*
  class: cl_UP_N
  cpp-type-dcl: "const cl_UP_N &%s"
  cast: "((const cl_UP_N &)(%s))"
  arg-type: "const cl_UP_N *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_UP_N *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_UP_N *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_complex_ring& static
  id: cl_heap_univpoly_complex_ring*
  class: cl_heap_univpoly_complex_ring
  cpp-type-dcl: "cl_heap_univpoly_complex_ring &%s"
  cast: "((cl_heap_univpoly_complex_ring &)(%s))"
  arg-type: "cl_heap_univpoly_complex_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_complex_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_complex_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_complex_ring static
  id: cl_heap_univpoly_complex_ring*
  class: cl_heap_univpoly_complex_ring
  cpp-type-dcl: "cl_heap_univpoly_complex_ring %s"
  cast: "((cl_heap_univpoly_complex_ring)(%s))"
  arg-type: "cl_heap_univpoly_complex_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_heap_univpoly_complex_ring *%s"
  res-converter: "new cl_heap_univpoly_complex_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_heap_univpoly_complex_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_univpoly_modint_ring& static
  id: cl_univpoly_modint_ring*
  class: cl_univpoly_modint_ring
  cpp-type-dcl: "const cl_univpoly_modint_ring &%s"
  cast: "((const cl_univpoly_modint_ring &)(%s))"
  arg-type: "const cl_univpoly_modint_ring *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_univpoly_modint_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_univpoly_modint_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_modint_ring& static
  id: cl_univpoly_modint_ring*
  class: cl_univpoly_modint_ring
  cpp-type-dcl: "cl_univpoly_modint_ring &%s"
  cast: "((cl_univpoly_modint_ring &)(%s))"
  arg-type: "cl_univpoly_modint_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_modint_ring *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_modint_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_univpoly_modint_ring* static
  id: cl_heap_univpoly_modint_ring*
  class: cl_heap_univpoly_modint_ring
  cpp-type-dcl: "cl_heap_univpoly_modint_ring *%s"
  cast: "((cl_heap_univpoly_modint_ring *)(%s))"
  callback-res-converter: "(cl_heap_univpoly_modint_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_modint_ring static
  id: cl_univpoly_modint_ring*
  class: cl_univpoly_modint_ring
  cpp-type-dcl: "cl_univpoly_modint_ring %s"
  cast: "((cl_univpoly_modint_ring)(%s))"
  arg-type: "cl_univpoly_modint_ring *%s"
  arg-converter: "*(%s)"
  res-type: "cl_univpoly_modint_ring *%s"
  res-converter: "new cl_univpoly_modint_ring (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_univpoly_modint_ring *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_univpoly_modint_ring* static
  id: cl_univpoly_modint_ring*
  class: cl_univpoly_modint_ring
  cpp-type-dcl: "cl_univpoly_modint_ring *%s"
  cast: "((cl_univpoly_modint_ring *)(%s))"
  callback-res-converter: "(cl_univpoly_modint_ring *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_UP_MI* static
  id: cl_UP_MI*
  class: cl_UP_MI
  cpp-type-dcl: "cl_UP_MI *%s"
  cast: "((cl_UP_MI *)(%s))"
  callback-res-converter: "(cl_UP_MI *)(%s)"
  c-callback-arg-type: "void *%s")
