// Private modular integer operations.

#ifndef _CL_MI_H
#define _CL_MI_H

#include "cl_modinteger.h"

extern cl_composite_condition* cl_notify_composite (const cl_modint_ring& R, const cl_I& nonunit);

#endif /* _CL_MI_H */
