(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded sinh (const-cl_R&) cl_R "sinh")
(defoverloaded sinh (const-cl_F&) cl_F "sinh")
(defoverloaded sinh (long) cl_R "sinh")
(defoverloaded cosh (const-cl_R&) cl_R "cosh")
(defoverloaded cosh (const-cl_F&) cl_F "cosh")
(defoverloaded cosh (long) cl_R "cosh")
(define-cpp-converter cl_cosh_sinh_t* static)
;; Definition of class cl_cosh_sinh_t
;; At /home/bruno/cln/include/cl_real.h:550
(define-cpp-class cl_cosh_sinh_t (<class-proxy>)
  ((cosh type: cl_R cpp-name: "cosh")
   (sinh type: cl_R cpp-name: "sinh")
   (operator= (cl_cosh_sinh_t&) cl_cosh_sinh_t& "operator =" implicitp: t))
  constructor: (const-cl_R& const-cl_R&)
  copy-constructor: (cl_cosh_sinh_t&)
  type: cl_cosh_sinh_t*
  class: "cl_cosh_sinh_t")
(defoverloaded cl_cosh_sinh (const-cl_R&) cl_cosh_sinh_t "cl_cosh_sinh")
(defoverloaded tanh (const-cl_R&) cl_R "tanh")
(defoverloaded tanh (const-cl_F&) cl_F "tanh")
(defoverloaded tanh (long) cl_R "tanh")
(defoverloaded cl_eulerconst (const-cl_F&) cl_F "cl_eulerconst")
(defoverloaded cl_eulerconst (cl_float_format_t) cl_F "cl_eulerconst")
(defoverloaded cl_eulerconst () cl_F "cl_eulerconst")
(defoverloaded cl_catalanconst (const-cl_F&) cl_F "cl_catalanconst")
(defoverloaded cl_catalanconst (cl_float_format_t) cl_F "cl_catalanconst")
(defoverloaded cl_catalanconst () cl_F "cl_catalanconst")
(defoverloaded cl_zeta (int const-cl_F&) cl_F "cl_zeta")
(defoverloaded cl_zeta (int cl_float_format_t) cl_F "cl_zeta")
(defoverloaded cl_zeta (int) cl_F "cl_zeta")
(defoverloaded realpart (const-cl_R&) cl_R "realpart")
(defoverloaded imagpart (const-cl_R&) cl_R "imagpart")
(defoverloaded conjugate (const-cl_R&) cl_R "conjugate")
(defoverloaded operator- (const-cl_SF&) cl_SF "operator -")
(defoverloaded cl_compare (const-cl_SF& const-cl_SF&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_SF& const-cl_SF&) bool "operator ==")
(defoverloaded operator!= (const-cl_SF& const-cl_SF&) bool "operator !=")
(defoverloaded operator<= (const-cl_SF& const-cl_SF&) bool "operator <=")
(defoverloaded operator< (const-cl_SF& const-cl_SF&) bool "operator <")
(defoverloaded operator>= (const-cl_SF& const-cl_SF&) bool "operator >=")
(defoverloaded operator> (const-cl_SF& const-cl_SF&) bool "operator >")
(defoverloaded minusp (const-cl_SF&) cl_boolean "minusp")
(defoverloaded zerop (const-cl_SF&) cl_boolean "zerop")
(defoverloaded operator+ (const-cl_SF& const-cl_SF&) cl_SF "operator +")
(defoverloaded operator- (const-cl_SF& const-cl_SF&) cl_SF "operator -")
(defoverloaded operator* (const-cl_SF& const-cl_SF&) cl_SF "operator *")
(defoverloaded square (const-cl_SF&) cl_SF "square")
(defoverloaded operator/ (const-cl_SF& const-cl_SF&) cl_SF "operator /")
(defoverloaded sqrt (const-cl_SF&) cl_SF "sqrt")
(defoverloaded recip (const-cl_SF&) cl_SF "recip")
(defoverloaded plus1 (const-cl_SF&) cl_SF "plus1")
(defoverloaded minus1 (const-cl_SF&) cl_SF "minus1")
(defoverloaded ftruncate (const-cl_SF&) cl_SF "ftruncate")
(defoverloaded fround (const-cl_SF&) cl_SF "fround")
(defoverloaded ffloor (const-cl_SF&) cl_SF "ffloor")
(defoverloaded fceiling (const-cl_SF&) cl_SF "fceiling")
(define-cpp-converter cl_SF_fdiv_t* static)
;; Definition of class cl_SF_fdiv_t
;; At /home/bruno/cln/include/cl_sfloat.h:98
(define-cpp-class cl_SF_fdiv_t (<class-proxy>)
  ((quotient type: cl_SF cpp-name: "quotient")
   (remainder type: cl_SF cpp-name: "remainder")
   (operator= (cl_SF_fdiv_t&) cl_SF_fdiv_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_SF& const-cl_SF&)
  copy-constructor: (cl_SF_fdiv_t&)
  type: cl_SF_fdiv_t*
  class: "cl_SF_fdiv_t")
(defoverloaded ffloor2 (const-cl_SF&) cl_SF_fdiv_t "ffloor2")
