(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded fceiling2 (const-cl_SF&) cl_SF_fdiv_t "fceiling2")
(defoverloaded ftruncate2 (const-cl_SF&) cl_SF_fdiv_t "ftruncate2")
(defoverloaded fround2 (const-cl_SF&) cl_SF_fdiv_t "fround2")
(define-cpp-converter cl_SF_div_t* static)
;; Definition of class cl_SF_div_t
;; At /home/bruno/cln/include/cl_sfloat.h:125
(define-cpp-class cl_SF_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_SF cpp-name: "remainder")
   (operator= (cl_SF_div_t&) cl_SF_div_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_SF&)
  copy-constructor: (cl_SF_div_t&)
  type: cl_SF_div_t*
  class: "cl_SF_div_t")
(defoverloaded floor2 (const-cl_SF&) cl_SF_div_t "floor2")
(defoverloaded floor1 (const-cl_SF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_SF&) cl_SF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_SF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_SF&) cl_SF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_SF&) cl_I "truncate1")
(defoverloaded round2 (const-cl_SF&) cl_SF_div_t "round2")
(defoverloaded round1 (const-cl_SF&) cl_I "round1")
(defoverloaded floor2 (const-cl_SF& const-cl_SF&) cl_SF_div_t "floor2")
(defoverloaded floor1 (const-cl_SF& const-cl_SF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_SF& const-cl_SF&) cl_SF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_SF& const-cl_SF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_SF& const-cl_SF&) cl_SF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_SF& const-cl_SF&) cl_I "truncate1")
(defoverloaded round2 (const-cl_SF& const-cl_SF&) cl_SF_div_t "round2")
(defoverloaded round1 (const-cl_SF& const-cl_SF&) cl_I "round1")
(define-cpp-converter cl_decoded_sfloat* static)
;; Definition of class cl_decoded_sfloat
;; At /home/bruno/cln/include/cl_sfloat.h:203
(define-cpp-class cl_decoded_sfloat (<class-proxy>)
  ((mantissa type: cl_SF cpp-name: "mantissa")
   (exponent type: cl_I cpp-name: "exponent")
   (sign type: cl_SF cpp-name: "sign")
   (operator= (cl_decoded_sfloat&)
              cl_decoded_sfloat&
              "operator ="
              implicitp: t))
  constructor: ()
  constructor: (const-cl_SF& const-cl_I& const-cl_SF&)
  copy-constructor: (cl_decoded_sfloat&)
  type: cl_decoded_sfloat*
  class: "cl_decoded_sfloat")
(defoverloaded decode_float (const-cl_SF&) cl_decoded_sfloat "decode_float")
(defoverloaded float_exponent (const-cl_SF&) sintL "float_exponent")
(defoverloaded float_radix (const-cl_SF&) sintL "float_radix")
(defoverloaded float_sign (const-cl_SF&) cl_SF "float_sign")
(defoverloaded float_digits (const-cl_SF&) uintL "float_digits")
(defoverloaded float_precision (const-cl_SF&) uintL "float_precision")
(defoverloaded integer_decode_float (const-cl_SF&)
  cl_idecoded_float
  "integer_decode_float")
(defoverloaded scale_float (const-cl_SF& sintL) cl_SF "scale_float")
(defoverloaded scale_float (const-cl_SF& const-cl_I&) cl_SF "scale_float")
(defoverloaded signum (const-cl_SF&) cl_SF "signum")
(defoverloaded cl_float_approx (const-cl_SF&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_SF&) double "cl_double_approx")
(defoverloaded operator- (const-cl_FF&) cl_FF "operator -")
(defoverloaded cl_compare (const-cl_FF& const-cl_FF&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_FF& const-cl_FF&) bool "operator ==")
(defoverloaded operator!= (const-cl_FF& const-cl_FF&) bool "operator !=")
(defoverloaded operator<= (const-cl_FF& const-cl_FF&) bool "operator <=")
(defoverloaded operator< (const-cl_FF& const-cl_FF&) bool "operator <")
(defoverloaded operator>= (const-cl_FF& const-cl_FF&) bool "operator >=")
(defoverloaded operator> (const-cl_FF& const-cl_FF&) bool "operator >")
(defoverloaded minusp (const-cl_FF&) cl_boolean "minusp")
(defoverloaded zerop (const-cl_FF&) cl_boolean "zerop")
(defoverloaded operator+ (const-cl_FF& const-cl_FF&) cl_FF "operator +")
(defoverloaded operator- (const-cl_FF& const-cl_FF&) cl_FF "operator -")
(defoverloaded operator* (const-cl_FF& const-cl_FF&) cl_FF "operator *")
(defoverloaded square (const-cl_FF&) cl_FF "square")
(defoverloaded operator/ (const-cl_FF& const-cl_FF&) cl_FF "operator /")
