(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(define-cpp-converter cl_print_univpoly_flags* static)
;; Definition of class cl_print_univpoly_flags
;; At /home/bruno/cln/include/cl_output.h:59
(define-cpp-class cl_print_univpoly_flags (<class-proxy>)
  ((univpoly_varname type: cl_string cpp-name: "univpoly_varname")
   (operator= (cl_print_univpoly_flags&)
              cl_print_univpoly_flags&
              "operator ="
              implicitp: t))
  constructor: ()
  copy-constructor: (cl_print_univpoly_flags&)
  type: cl_print_univpoly_flags*
  class: "cl_print_univpoly_flags")
(define-cpp-converter cl_print_flags* static)
;; Definition of class cl_print_flags
;; At /home/bruno/cln/include/cl_output.h:66
(define-cpp-class cl_print_flags (cl_print_number_flags
                                   cl_print_vector_flags
                                   cl_print_univpoly_flags)
  ((vector_syntax type: cl_print_vector_syntax_t cpp-name: "vector_syntax")
   (complex_readably type: cl_boolean cpp-name: "complex_readably")
   (float_readably type: cl_boolean cpp-name: "float_readably")
   (rational_base type: unsigned cpp-name: "rational_base")
   (rational_readably type: cl_boolean cpp-name: "rational_readably")
   (univpoly_varname type: cl_string cpp-name: "univpoly_varname")
   (default_float_format
     type: cl_float_format_t
     cpp-name: "default_float_format")
   (operator= (cl_print_flags&) cl_print_flags& "operator =" implicitp: t))
  copy-constructor: (cl_print_flags&)
  constructor: ()
  type: cl_print_flags*
  class: "cl_print_flags"
  supertypes: (cl_print_number_flags*
                cl_print_vector_flags*
                cl_print_univpoly_flags*))
(define-cpp-variable
  cl_default_print_flags
  cl_print_flags
  "cl_default_print_flags")
(defoverloaded print_number (ostream& const-cl_print_flags& const-cl_number&)
  void
  "print_number")
(defoverloaded print_number (ostream& const-cl_print_number_flags&
                                      const-cl_number&)
  void
  "print_number")
(defoverloaded print_real (ostream& const-cl_print_flags& const-cl_R&)
  void
  "print_real")
(defoverloaded print_real (ostream& const-cl_print_number_flags& const-cl_R&)
  void
  "print_real")
(defoverloaded print_real (ostream& const-cl_print_real_flags& const-cl_R&)
  void
  "print_real")
(defoverloaded print_rational (ostream& const-cl_print_flags& const-cl_RA&)
  void
  "print_rational")
(defoverloaded print_rational (ostream& const-cl_print_number_flags&
                                        const-cl_RA&)
  void
  "print_rational")
(defoverloaded print_rational (ostream& const-cl_print_real_flags&
                                        const-cl_RA&)
  void
  "print_rational")
(defoverloaded print_rational (ostream& const-cl_print_rational_flags&
                                        const-cl_RA&)
  void
  "print_rational")
(define-cpp-converter cl_condition* static)
;; Definition of class cl_condition
;; At /home/bruno/cln/include/cl_condition.h:9
(define-cpp-class cl_condition (<class-proxy>)
  ((operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (name () const-char* "name")
   (print (ostream&) void "print")
   (operator= (cl_condition&) cl_condition& "operator =" implicitp: t))
  type: cl_condition*
  class: "cl_condition")
(define-cpp-converter cl_symbol* static)
;; Definition of class cl_symbol
;; At /home/bruno/cln/include/cl_symbol.h:14
(define-cpp-class cl_symbol (cl_rcpointer)
  ((operator= (const-cl_symbol&) cl_symbol& "operator ="))
  constructor: (const-cl_string&)
  copy-constructor: (const-cl_symbol&)
  type: cl_symbol*
  class: "cl_symbol")
(defoverloaded hashcode (const-cl_symbol&) unsigned-long "hashcode")
(define-cpp-converter cl_property* static)
;; Definition of class cl_property
;; At /home/bruno/cln/include/cl_proplist.h:17
(define-cpp-class cl_property (<class-proxy>)
  ((key type: cl_symbol cpp-name: "key")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (operator= (cl_property&) cl_property& "operator =" implicitp: t))
  constructor: (const-cl_symbol&)
  copy-constructor: (cl_property&)
  type: cl_property*
  class: "cl_property")
(define-cpp-converter cl_property_list* static)
;; Definition of class cl_property_list
;; At /home/bruno/cln/include/cl_proplist.h:38
(define-cpp-class cl_property_list (<class-proxy>)
  ((get_property (const-cl_symbol&) cl_property* "get_property")
   (add_property (cl_property*) void "add_property")
   (operator= (cl_property_list&) cl_property_list& "operator =" implicitp: t))
  constructor: ()
  copy-constructor: (cl_property_list&)
  type: cl_property_list*
  class: "cl_property_list")
(define-cpp-converter cl_ring* static)
(define-cpp-converter cl_heap_ring* static)
;; Definition of class cl_ring
;; At /home/bruno/cln/include/cl_ring.h:40
(define-cpp-class cl_ring (cl_rcpointer)
  ((operator= (const-cl_ring&) cl_ring& "operator =")
   (operator-> () cl_heap_ring* "operator ->"))
  constructor: (cl_heap_ring*)
  copy-constructor: (const-cl_ring&)
  constructor: ()
  type: cl_ring*
  class: "cl_ring")
(defoverloaded operator== (const-cl_ring& const-cl_ring&) bool "operator ==")
(defoverloaded operator!= (const-cl_ring& const-cl_ring&) bool "operator !=")
(defoverloaded operator== (const-cl_ring& cl_heap_ring*) bool "operator ==")
(defoverloaded operator!= (const-cl_ring& cl_heap_ring*) bool "operator !=")
(define-cpp-converter _cl_ring_element* static)
;; Definition of class _cl_ring_element
;; At /home/bruno/cln/include/cl_ring.h:109
(define-cpp-class _cl_ring_element (<class-proxy>)
  ((rep type: cl_gcobject cpp-name: "rep")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t _cl_ring_element*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (operator= (_cl_ring_element&) _cl_ring_element& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_heap_ring* const-cl_gcobject&)
  constructor: (const-cl_ring& const-cl_gcobject&)
  copy-constructor: (_cl_ring_element&)
  type: _cl_ring_element*
  class: "_cl_ring_element")
(define-cpp-converter cl_ring_element* static)
;; Definition of class cl_ring_element
;; At /home/bruno/cln/include/cl_ring.h:124
(define-cpp-class cl_ring_element (_cl_ring_element)
  ((ring () const-cl_ring& "ring")
   (debug_print () void "debug_print")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_ring_element*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (operator= (cl_ring_element&) cl_ring_element& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_ring& const-cl_gcobject&)
  constructor: (const-cl_ring& const_cl_ring_element&)
  copy-constructor: (cl_ring_element&)
  type: cl_ring_element*
  class: "cl_ring_element")
(define-cpp-converter
  |void(*)(cl_heap_ring**,ostream**,_cl_ring_element**)|
  function-type)
(define-callback-pool
  |void(*)(cl_heap_ring*,ostream&,const_cl_ring_element&)|
  5)
(define-cpp-converter
  |cl_boolean(*)(cl_heap_ring**,_cl_ring_element**,_cl_ring_element**)|
  function-type)
(define-callback-pool
  |cl_boolean(*)(cl_heap_ring*,const_cl_ring_element&,const_cl_ring_element&)|
  5)
(define-cpp-converter _cl_ring_setops* static)
;; Definition of class _cl_ring_setops
;; At /home/bruno/cln/include/cl_ring.h:152
(define-cpp-class _cl_ring_setops (<class-proxy>)
  ((fprint type:
           |void(*)(cl_heap_ring*,ostream&,const_cl_ring_element&)|
           cpp-name: "fprint")
   (equal type:
          |cl_boolean(*)(cl_heap_ring*,const_cl_ring_element&,const_cl_ring_element&)|
          cpp-name: "equal")
   (operator= (_cl_ring_setops&) _cl_ring_setops& "operator =" implicitp: t))
  copy-constructor: (_cl_ring_setops&)
  constructor: ()
  type: _cl_ring_setops*
  class: "_cl_ring_setops")
(define-cpp-converter |_cl_ring_element*(*)(cl_heap_ring**)| function-type)
(define-callback-pool |_cl_ring_element(*)(cl_heap_ring*)| 5)
(define-cpp-converter
  |cl_boolean(*)(cl_heap_ring**,_cl_ring_element**)|
  function-type)
(define-callback-pool |cl_boolean(*)(cl_heap_ring*,const_cl_ring_element&)| 5)
(define-cpp-converter
  |_cl_ring_element*(*)(cl_heap_ring**,_cl_ring_element**,_cl_ring_element**)|
  function-type)
(define-callback-pool
  |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&,const_cl_ring_element&)|
  5)
(define-cpp-converter
  |_cl_ring_element*(*)(cl_heap_ring**,_cl_ring_element**)|
  function-type)
(define-callback-pool
  |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&)|
  5)
(define-cpp-converter _cl_ring_addops* static)
;; Definition of class _cl_ring_addops
;; At /home/bruno/cln/include/cl_ring.h:159
(define-cpp-class _cl_ring_addops (<class-proxy>)
  ((zero type: |_cl_ring_element(*)(cl_heap_ring*)| cpp-name: "zero")
   (zerop type:
          |cl_boolean(*)(cl_heap_ring*,const_cl_ring_element&)|
          cpp-name: "zerop")
   (plus type:
         |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&,const_cl_ring_element&)|
         cpp-name: "plus")
   (minus type:
          |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&,const_cl_ring_element&)|
          cpp-name: "minus")
   (uminus type:
           |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&)|
           cpp-name: "uminus")
   (operator= (_cl_ring_addops&) _cl_ring_addops& "operator =" implicitp: t))
  copy-constructor: (_cl_ring_addops&)
  constructor: ()
  type: _cl_ring_addops*
  class: "_cl_ring_addops")
