// Random numbers suitable for testing.
// They contain relatively long strings of ones and zeroes. Better for testing.

#ifndef _TRANDOM_H
#define _TRANDOM_H

#include <cl_number.h>
#include <cl_random.h>

extern cl_I testrandom_I ();
//extern cl_F testrandom_F ();
//extern cl_R testrandom_R ();

#endif /* _TRANDOM_H */
