(define-cpp-type const-cl_byte& static
  id: cl_byte*
  class: cl_byte
  cpp-type-dcl: "const cl_byte &%s"
  cast: "((const cl_byte &)(%s))"
  arg-type: "const cl_byte *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_byte *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_byte *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_I_div_t quotient)
(define-cpp-data-slot-setter cl_I_div_t remainder)
(define-cpp-type cl_I_div_t& static
  id: cl_I_div_t*
  class: cl_I_div_t
  cpp-type-dcl: "cl_I_div_t &%s"
  cast: "((cl_I_div_t &)(%s))"
  arg-type: "cl_I_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_I_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_I_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_I_div_t static
  id: cl_I_div_t*
  class: cl_I_div_t
  cpp-type-dcl: "cl_I_div_t %s"
  cast: "((cl_I_div_t)(%s))"
  arg-type: "cl_I_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_I_div_t *%s"
  res-converter: "new cl_I_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_I_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_I_div_t* static
  id: cl_I_div_t*
  class: cl_I_div_t
  cpp-type-dcl: "cl_I_div_t *%s"
  cast: "((cl_I_div_t *)(%s))"
  callback-res-converter: "(cl_I_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_random_state* static
  id: cl_random_state*
  class: cl_random_state
  cpp-type-dcl: "cl_random_state *%s"
  cast: "((cl_random_state *)(%s))"
  callback-res-converter: "(cl_random_state *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_random_state& static
  id: cl_random_state*
  class: cl_random_state
  cpp-type-dcl: "cl_random_state &%s"
  cast: "((cl_random_state &)(%s))"
  arg-type: "cl_random_state *%s"
  arg-converter: "*(%s)"
  res-type: "cl_random_state *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_random_state *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-variable-setter cl_debugout_stream)
(define-cpp-data-slot-setter cl_read_float_flags default_float_format)
(define-cpp-data-slot-setter cl_read_float_flags default_lfloat_format)
(define-cpp-type cl_read_float_flags& static
  id: cl_read_float_flags*
  class: cl_read_float_flags
  cpp-type-dcl: "cl_read_float_flags &%s"
  cast: "((cl_read_float_flags &)(%s))"
  arg-type: "cl_read_float_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_read_float_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_read_float_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_read_float_flags static
  id: cl_read_float_flags*
  class: cl_read_float_flags
  cpp-type-dcl: "cl_read_float_flags %s"
  cast: "((cl_read_float_flags)(%s))"
  arg-type: "cl_read_float_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_read_float_flags *%s"
  res-converter: "new cl_read_float_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_read_float_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_read_float_flags* static
  id: cl_read_float_flags*
  class: cl_read_float_flags
  cpp-type-dcl: "cl_read_float_flags *%s"
  cast: "((cl_read_float_flags *)(%s))"
  callback-res-converter: "(cl_read_float_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_read_syntax_t enum
  cpp-arg-type: "((cl_read_syntax_t)(%s))"
  id: cl_read_syntax_t)
(define-cpp-type cl_read_lsyntax_t enum
  cpp-arg-type: "((cl_read_lsyntax_t)(%s))"
  id: cl_read_lsyntax_t)
(define-cpp-data-slot-setter cl_read_flags syntax)
(define-cpp-data-slot-setter cl_read_flags lsyntax)
(define-cpp-data-slot-setter cl_read_flags rational_base)
(define-cpp-data-slot-setter cl_read_flags float_flags)
(define-cpp-type cl_read_flags& static
  id: cl_read_flags*
  class: cl_read_flags
  cpp-type-dcl: "cl_read_flags &%s"
  cast: "((cl_read_flags &)(%s))"
  arg-type: "cl_read_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_read_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_read_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_read_flags static
  id: cl_read_flags*
  class: cl_read_flags
  cpp-type-dcl: "cl_read_flags %s"
  cast: "((cl_read_flags)(%s))"
  arg-type: "cl_read_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_read_flags *%s"
  res-converter: "new cl_read_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_read_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_read_flags* static
  id: cl_read_flags*
  class: cl_read_flags
  cpp-type-dcl: "cl_read_flags *%s"
  cast: "((cl_read_flags *)(%s))"
  callback-res-converter: "(cl_read_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_read_flags& static
  id: cl_read_flags*
  class: cl_read_flags
  cpp-type-dcl: "const cl_read_flags &%s"
  cast: "((const cl_read_flags &)(%s))"
  arg-type: "const cl_read_flags *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_read_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_read_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-char** other-proxy
  cpp-type-dcl: "const char **%s"
  id: char**
  cast: "((const char **)(%s))"
  c-callback-arg-type: "void *%s")
(define-cpp-type const-cl_string& static
  id: cl_string*
  class: cl_string
  cpp-type-dcl: "const cl_string &%s"
  cast: "((const cl_string &)(%s))"
  arg-type: "const cl_string *%s"
  arg-converter: "*(%s)"
  res-type: "const cl_string *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(const cl_string *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_string& static
  id: cl_string*
  class: cl_string
  cpp-type-dcl: "cl_string &%s"
  cast: "((cl_string &)(%s))"
  arg-type: "cl_string *%s"
  arg-converter: "*(%s)"
  res-type: "cl_string *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_string *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_string static
  id: cl_string*
  class: cl_string
  cpp-type-dcl: "cl_string %s"
  cast: "((cl_string)(%s))"
  arg-type: "cl_string *%s"
  arg-converter: "*(%s)"
  res-type: "cl_string *%s"
  res-converter: "new cl_string (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_string *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_heap_string* static
  id: cl_heap_string*
  class: cl_heap_string
  cpp-type-dcl: "cl_heap_string *%s"
  cast: "((cl_heap_string *)(%s))"
  callback-res-converter: "(cl_heap_string *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_string* static
  id: cl_string*
  class: cl_string
  cpp-type-dcl: "cl_string *%s"
  cast: "((cl_string *)(%s))"
  callback-res-converter: "(cl_string *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_print_rational_flags rational_base)
(define-cpp-data-slot-setter cl_print_rational_flags rational_readably)
(define-cpp-type cl_print_rational_flags& static
  id: cl_print_rational_flags*
  class: cl_print_rational_flags
  cpp-type-dcl: "cl_print_rational_flags &%s"
  cast: "((cl_print_rational_flags &)(%s))"
  arg-type: "cl_print_rational_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_rational_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_rational_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_rational_flags static
  id: cl_print_rational_flags*
  class: cl_print_rational_flags
  cpp-type-dcl: "cl_print_rational_flags %s"
  cast: "((cl_print_rational_flags)(%s))"
  arg-type: "cl_print_rational_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_rational_flags *%s"
  res-converter: "new cl_print_rational_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_rational_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_rational_flags* static
  id: cl_print_rational_flags*
  class: cl_print_rational_flags
  cpp-type-dcl: "cl_print_rational_flags *%s"
  cast: "((cl_print_rational_flags *)(%s))"
  callback-res-converter: "(cl_print_rational_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_print_float_flags float_readably)
(define-cpp-data-slot-setter cl_print_float_flags default_float_format)
(define-cpp-type cl_print_float_flags& static
  id: cl_print_float_flags*
  class: cl_print_float_flags
  cpp-type-dcl: "cl_print_float_flags &%s"
  cast: "((cl_print_float_flags &)(%s))"
  arg-type: "cl_print_float_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_float_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_float_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_float_flags static
  id: cl_print_float_flags*
  class: cl_print_float_flags
  cpp-type-dcl: "cl_print_float_flags %s"
  cast: "((cl_print_float_flags)(%s))"
  arg-type: "cl_print_float_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_float_flags *%s"
  res-converter: "new cl_print_float_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_float_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_float_flags* static
  id: cl_print_float_flags*
  class: cl_print_float_flags
  cpp-type-dcl: "cl_print_float_flags *%s"
  cast: "((cl_print_float_flags *)(%s))"
  callback-res-converter: "(cl_print_float_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_print_real_flags float_readably)
(define-cpp-data-slot-setter cl_print_real_flags rational_base)
(define-cpp-data-slot-setter cl_print_real_flags default_float_format)
(define-cpp-data-slot-setter cl_print_real_flags rational_readably)
(define-cpp-type cl_print_real_flags& static
  id: cl_print_real_flags*
  class: cl_print_real_flags
  cpp-type-dcl: "cl_print_real_flags &%s"
  cast: "((cl_print_real_flags &)(%s))"
  arg-type: "cl_print_real_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_real_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_real_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_real_flags static
  id: cl_print_real_flags*
  class: cl_print_real_flags
  cpp-type-dcl: "cl_print_real_flags %s"
  cast: "((cl_print_real_flags)(%s))"
  arg-type: "cl_print_real_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_real_flags *%s"
  res-converter: "new cl_print_real_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_real_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_real_flags* static
  id: cl_print_real_flags*
  class: cl_print_real_flags
  cpp-type-dcl: "cl_print_real_flags *%s"
  cast: "((cl_print_real_flags *)(%s))"
  callback-res-converter: "(cl_print_real_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_print_complex_flags complex_readably)
(define-cpp-type cl_print_complex_flags& static
  id: cl_print_complex_flags*
  class: cl_print_complex_flags
  cpp-type-dcl: "cl_print_complex_flags &%s"
  cast: "((cl_print_complex_flags &)(%s))"
  arg-type: "cl_print_complex_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_complex_flags *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_complex_flags *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_print_complex_flags static
  id: cl_print_complex_flags*
  class: cl_print_complex_flags
  cpp-type-dcl: "cl_print_complex_flags %s"
  cast: "((cl_print_complex_flags)(%s))"
  arg-type: "cl_print_complex_flags *%s"
  arg-converter: "*(%s)"
  res-type: "cl_print_complex_flags *%s"
  res-converter: "new cl_print_complex_flags (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_print_complex_flags *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
