// cl_N internals

#ifndef _CL_N_H
#define _CL_N_H

#include "cl_number.h"
#include "cl_macros.h"

nonreturning_function(extern, cl_error_division_by_0, (void));
nonreturning_function(extern, cl_as_error, (const cl_number& obj, const char * typestring, const char * filename, int line));

#endif /* _CL_N_H */
