// cl_exp1().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_real.h"


// Implementation.

#include "cl_F.h"
#include "cl_R_tran.h"

cl_F cl_exp1 (void)
{
	floatformatcase(cl_default_float_format
	,	return cl_SF_exp1;
	,	return cl_FF_exp1;
	,	return cl_DF_exp1;
	,	return cl_exp1(len);
	);
}
