(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(define-cpp-converter cl_boolean enum)
(defenum cl_boolean (cl_false cl_true) cl_boolean)
(defoverloaded cl_pointer_p (cl_uint) cl_boolean "cl_pointer_p")
(defoverloaded cl_immediate_p (cl_uint) cl_boolean "cl_immediate_p")
(defoverloaded cl_tag (cl_uint) cl_uint "cl_tag")
(defoverloaded cl_value (cl_uint) cl_uint "cl_value")
(defoverloaded cl_svalue (cl_uint) cl_sint "cl_svalue")
(defoverloaded cl_combine (cl_uint cl_sint) cl_uint "cl_combine")
(define-cpp-converter cl_heap* static)
;; Definition of class cl_heap
;; At /home/bruno/cln/include/cl_object.h:162
(define-cpp-class cl_heap (<class-proxy>)
  ((refcount type: int cpp-name: "refcount")
   (typetag type: int cpp-name: "typetag")
   (operator= (cl_heap&) cl_heap& "operator =" implicitp: t))
  copy-constructor: (cl_heap&)
  constructor: ()
  type: cl_heap*
  class: "cl_heap")
(define-cpp-converter cl_heap_destructor_function function-type)
(define-callback-pool cl_heap_destructor_function 5)
(defoverloaded cl_register_typetag (int cl_heap_destructor_function)
  void
  "cl_register_typetag")
(defoverloaded cl_free_heap_object (cl_heap*) void "cl_free_heap_object")
(define-cpp-converter cl_heap_dprint_function function-type)
(define-callback-pool cl_heap_dprint_function 5)
(defoverloaded cl_register_typetag_printer (int cl_heap_dprint_function)
  void
  "cl_register_typetag_printer")
(defoverloaded cl_inc_pointer_refcount (cl_heap*)
  void
  "cl_inc_pointer_refcount")
(defoverloaded cl_gc_dec_pointer_refcount (cl_heap*)
  void
  "cl_gc_dec_pointer_refcount")
(defoverloaded cl_rc_dec_pointer_refcount (cl_heap*)
  void
  "cl_rc_dec_pointer_refcount")
(define-cpp-converter cl_gcobject* static)
;; Definition of class cl_gcobject
;; At /home/bruno/cln/include/cl_object.h:330
(define-cpp-class cl_gcobject (<class-proxy>)
  ((operator= (const-cl_gcobject&) cl_gcobject& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (inc_pointer_refcount () void "inc_pointer_refcount")
   (dec_pointer_refcount () void "dec_pointer_refcount")
   (nonpointer_tag () cl_uint "nonpointer_tag")
   (pointer_typetag () int "pointer_typetag")
   (debug_print () void "debug_print")
   (operator-new (size_t cl_gcobject*) void* "operator new" staticp: t)
   (operator-new (size_t) void* "operator new" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_gcobject&)
  type: cl_gcobject*
  class: "cl_gcobject")
(define-cpp-converter cl_gcpointer* static)
;; Definition of class cl_gcpointer
;; At /home/bruno/cln/include/cl_object.h:388
(define-cpp-class cl_gcpointer (<class-proxy>)
  ((operator= (const-cl_gcpointer&) cl_gcpointer& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (inc_pointer_refcount () void "inc_pointer_refcount")
   (dec_pointer_refcount () void "dec_pointer_refcount")
   (nonpointer_tag () cl_uint "nonpointer_tag")
   (pointer_typetag () int "pointer_typetag")
   (debug_print () void "debug_print")
   (operator-new (size_t cl_gcpointer*) void* "operator new" staticp: t)
   (operator-new (size_t) void* "operator new" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_gcpointer&)
  type: cl_gcpointer*
  class: "cl_gcpointer")
(define-cpp-converter cl_rcobject* static)
;; Definition of class cl_rcobject
;; At /home/bruno/cln/include/cl_object.h:446
(define-cpp-class cl_rcobject (<class-proxy>)
  ((operator= (const-cl_rcobject&) cl_rcobject& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (inc_pointer_refcount () void "inc_pointer_refcount")
   (dec_pointer_refcount () void "dec_pointer_refcount")
   (nonpointer_tag () cl_uint "nonpointer_tag")
   (pointer_typetag () int "pointer_typetag")
   (debug_print () void "debug_print")
   (operator-new (size_t cl_rcobject*) void* "operator new" staticp: t)
   (operator-new (size_t) void* "operator new" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_rcobject&)
  type: cl_rcobject*
  class: "cl_rcobject")
(define-cpp-converter cl_rcpointer* static)
;; Definition of class cl_rcpointer
;; At /home/bruno/cln/include/cl_object.h:504
(define-cpp-class cl_rcpointer (<class-proxy>)
  ((operator= (const-cl_rcpointer&) cl_rcpointer& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (inc_pointer_refcount () void "inc_pointer_refcount")
   (dec_pointer_refcount () void "dec_pointer_refcount")
   (nonpointer_tag () cl_uint "nonpointer_tag")
   (pointer_typetag () int "pointer_typetag")
   (debug_print () void "debug_print")
   (operator-new (size_t cl_rcpointer*) void* "operator new" staticp: t)
   (operator-new (size_t) void* "operator new" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_rcpointer&)
  type: cl_rcpointer*
  class: "cl_rcpointer")
(define-cpp-converter |void*(*)(size_t)| function-type)
(define-callback-pool |void*(*)(size_t)| 5)
(define-cpp-variable cl_malloc_hook |void*(*)(size_t)| "cl_malloc_hook")
(define-cpp-converter |void(*)(void*)| function-type)
(define-callback-pool |void(*)(void*)| 5)
(define-cpp-variable cl_free_hook |void(*)(void*)| "cl_free_hook")
(define-cpp-converter cl_number* static)
(define-cpp-converter cl_N* static)
(define-cpp-converter cl_R* static)
(define-cpp-converter cl_F* static)
(define-cpp-converter cl_SF* static)
(define-cpp-converter cl_FF* static)
(define-cpp-converter cl_DF* static)
(define-cpp-converter cl_LF* static)
(define-cpp-converter cl_RA* static)
(define-cpp-converter cl_I* static)
;; Definition of class cl_number
;; At /home/bruno/cln/include/cl_number.h:40
(define-cpp-class cl_number (cl_gcobject)
  ((operator= (const-cl_number&) cl_number& "operator =")
   (operator= (const-cl_N&) cl_number& "operator =")
   (operator= (const-cl_R&) cl_number& "operator =")
   (operator= (const-cl_F&) cl_number& "operator =")
   (operator= (const-cl_SF&) cl_number& "operator =")
   (operator= (const-cl_FF&) cl_number& "operator =")
   (operator= (const-cl_DF&) cl_number& "operator =")
   (operator= (const-cl_LF&) cl_number& "operator =")
   (operator= (const-cl_RA&) cl_number& "operator =")
   (operator= (const-cl_I&) cl_number& "operator =")
   (operator= (long) cl_number& "operator =")
   (operator= (double) cl_number& "operator ="))
  constructor: ()
  copy-constructor: (const-cl_number&)
  constructor: (long)
  constructor: (double)
  type: cl_number*
  class: "cl_number")
;; Definition of class cl_N
;; At /home/bruno/cln/include/cl_number.h:108
(define-cpp-class cl_N (cl_number)
  ((operator= (const-cl_N&) cl_N& "operator =")
   (operator= (const-cl_R&) cl_N& "operator =")
   (operator= (const-cl_F&) cl_N& "operator =")
   (operator= (const-cl_SF&) cl_N& "operator =")
   (operator= (const-cl_FF&) cl_N& "operator =")
   (operator= (const-cl_DF&) cl_N& "operator =")
   (operator= (const-cl_LF&) cl_N& "operator =")
   (operator= (const-cl_RA&) cl_N& "operator =")
   (operator= (const-cl_I&) cl_N& "operator =")
   (operator= (long) cl_N& "operator =")
   (operator= (double) cl_N& "operator =")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_N*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_N&)
  constructor: (long)
  constructor: (double)
  constructor: (const-char*)
  type: cl_N*
  class: "cl_N")
;; Definition of class cl_R
;; At /home/bruno/cln/include/cl_number.h:152
(define-cpp-class cl_R (cl_N)
  ((operator= (const-cl_R&) cl_R& "operator =")
   (operator= (const-cl_F&) cl_R& "operator =")
   (operator= (const-cl_SF&) cl_R& "operator =")
   (operator= (const-cl_FF&) cl_R& "operator =")
   (operator= (const-cl_DF&) cl_R& "operator =")
   (operator= (const-cl_LF&) cl_R& "operator =")
   (operator= (long) cl_R& "operator =")
   (operator= (double) cl_R& "operator =")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_R*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_R&)
  constructor: (long)
  constructor: (double)
  constructor: (const-char*)
  type: cl_R*
  class: "cl_R")
;; Definition of class cl_F
;; At /home/bruno/cln/include/cl_number.h:192
(define-cpp-class cl_F (cl_R)
  ((operator= (const-cl_F&) cl_F& "operator =")
   (operator= (const-cl_SF&) cl_F& "operator =")
   (operator= (const-cl_FF&) cl_F& "operator =")
   (operator= (const-cl_DF&) cl_F& "operator =")
   (operator= (const-cl_LF&) cl_F& "operator =")
   (operator= (double) cl_F& "operator =")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_F*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_F&)
  constructor: (double)
  constructor: (const-char*)
  type: cl_F*
  class: "cl_F")
;; Definition of class cl_SF
;; At /home/bruno/cln/include/cl_number.h:223
(define-cpp-class cl_SF (cl_F)
  ((operator= (const-cl_SF&) cl_SF& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_SF*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_SF&)
  constructor: (const-char*)
  type: cl_SF*
  class: "cl_SF")
;; Definition of class cl_FF
;; At /home/bruno/cln/include/cl_number.h:260
(define-cpp-class cl_FF (cl_F)
  ((operator= (const-cl_FF&) cl_FF& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (operator= (float) cl_FF& "operator =")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_FF*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_FF&)
  constructor: (float)
  constructor: (const-char*)
  type: cl_FF*
  class: "cl_FF")
