// cl_zeta().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_real.h"


// Implementation.

#include "cl_F.h"
#include "cl_R_tran.h"
#include "cl_LF.h"

cl_F cl_zeta (int s)
{
	floatformatcase(cl_default_float_format
	,	return cl_LF_to_SF(cl_zeta(s,LF_minlen));
	,	return cl_LF_to_FF(cl_zeta(s,LF_minlen));
	,	return cl_LF_to_DF(cl_zeta(s,LF_minlen));
	,	return cl_zeta(s,len);
	);
}
