(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded logorc2 (const-cl_I& const-cl_I&) cl_I "logorc2")
(defoverloaded logorc1 (const-cl_I& const-cl_I&) cl_I "logorc1")
(defoverloaded lognot (const-cl_I&) cl_I "lognot")
(define-cpp-converter cl_boole enum)
(defenum cl_boole
  (boole_clr boole_set
             boole_1
             boole_2
             boole_c1
             boole_c2
             boole_and
             boole_ior
             boole_xor
             boole_eqv
             boole_nand
             boole_nor
             boole_andc1
             boole_andc2
             boole_orc1
             boole_orc2)
  cl_boole)
(defoverloaded boole (cl_boole const-cl_I& const-cl_I&) cl_I "boole")
(defoverloaded logtest (const-cl_I& const-cl_I&) cl_boolean "logtest")
(defoverloaded logbitp (uintL const-cl_I&) cl_boolean "logbitp")
(defoverloaded logbitp (const-cl_I& const-cl_I&) cl_boolean "logbitp")
(defoverloaded oddp (const-cl_I&) cl_boolean "oddp")
(defoverloaded evenp (const-cl_I&) cl_boolean "evenp")
(defoverloaded ash (const-cl_I& sintL) cl_I "ash")
(defoverloaded ash (const-cl_I& const-cl_I&) cl_I "ash")
(defoverloaded logcount (const-cl_I&) uintL "logcount")
(defoverloaded integer_length (const-cl_I&) uintL "integer_length")
(defoverloaded ord2 (const-cl_I&) uintL "ord2")
(defoverloaded power2p (const-cl_I&) uintL "power2p")
(defoverloaded |operator\|| (const-cl_I& const-cl_I&) cl_I "operator |")
(defoverloaded operator^ (const-cl_I& const-cl_I&) cl_I "operator ^")
(defoverloaded operator& (const-cl_I& const-cl_I&) cl_I "operator &")
(defoverloaded |operator~| (const-cl_I&) cl_I "operator ~")
(defoverloaded plus1 (const-cl_I&) cl_I "plus1")
(defoverloaded minus1 (const-cl_I&) cl_I "minus1")
(defoverloaded operator+ (const-cl_I& const-cl_I&) cl_I "operator +")
(defoverloaded operator+ (long const-cl_I&) cl_I "operator +")
(defoverloaded operator+ (const-cl_I& long) cl_I "operator +")
(defoverloaded operator- (const-cl_I&) cl_I "operator -")
(defoverloaded operator- (const-cl_I& const-cl_I&) cl_I "operator -")
(defoverloaded operator- (long const-cl_I&) cl_I "operator -")
(defoverloaded operator- (const-cl_I& long) cl_I "operator -")
(defoverloaded operator<< (const-cl_I& sintL) cl_I "operator <<")
(defoverloaded operator<< (const-cl_I& const-cl_I&) cl_I "operator <<")
(defoverloaded operator>> (const-cl_I& sintL) cl_I "operator >>")
(defoverloaded operator>> (const-cl_I& const-cl_I&) cl_I "operator >>")
(defoverloaded cl_equal (const-cl_I& const-cl_I&) cl_boolean "cl_equal")
(defoverloaded cl_compare (const-cl_I& const-cl_I&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_I& const-cl_I&) bool "operator ==")
(defoverloaded operator!= (const-cl_I& const-cl_I&) bool "operator !=")
(defoverloaded operator<= (const-cl_I& const-cl_I&) bool "operator <=")
(defoverloaded operator< (const-cl_I& const-cl_I&) bool "operator <")
(defoverloaded operator>= (const-cl_I& const-cl_I&) bool "operator >=")
(defoverloaded operator> (const-cl_I& const-cl_I&) bool "operator >")
(defoverloaded minusp (const-cl_I&) cl_boolean "minusp")
(defoverloaded zerop (const-cl_I&) cl_boolean "zerop")
(define-cpp-converter cl_byte* static)
;; Definition of class cl_byte
;; At /home/bruno/cln/include/cl_integer.h:265
(define-cpp-class cl_byte (<class-proxy>)
  ((size type: uintL cpp-name: "size")
   (position type: uintL cpp-name: "position")
   (operator= (cl_byte&) cl_byte& "operator =" implicitp: t))
  constructor: (unsigned unsigned)
  copy-constructor: (cl_byte&)
  type: cl_byte*
  class: "cl_byte")
(defoverloaded ldb (const-cl_I& const-cl_byte&) cl_I "ldb")
(defoverloaded ldb_test (const-cl_I& const-cl_byte&) cl_boolean "ldb_test")
(defoverloaded mask_field (const-cl_I& const-cl_byte&) cl_I "mask_field")
(defoverloaded deposit_field (const-cl_I& const-cl_I& const-cl_byte&)
  cl_I
  "deposit_field")
