// print_real().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_output.h"


// Implementation.

void print_real (cl_ostream stream, const cl_print_number_flags& flags, const cl_R& z)
{
	print_real(stream,(cl_print_real_flags&)flags,z);
}
