// cl_default_random_state.

// General includes.
#include "cl_sysdep.h"

CL_PROVIDE(cl_random_def)

// Specification.
#include "cl_random.h"


// Implementation.

cl_random_state cl_default_random_state;

CL_PROVIDE_END(cl_random_def)
