(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(define-cpp-converter
  |_cl_ring_element*(*)(cl_heap_ring**,cl_I**)|
  function-type)
(define-callback-pool |_cl_ring_element(*)(cl_heap_ring*,const-cl_I&)| 5)
(define-cpp-converter
  |_cl_ring_element*(*)(cl_heap_ring**,_cl_ring_element**,cl_I**)|
  function-type)
(define-callback-pool
  |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&,const-cl_I&)|
  5)
(define-cpp-converter _cl_ring_mulops* static)
;; Definition of class _cl_ring_mulops
;; At /home/bruno/cln/include/cl_ring.h:171
(define-cpp-class _cl_ring_mulops (<class-proxy>)
  ((one type: |_cl_ring_element(*)(cl_heap_ring*)| cpp-name: "one")
   (canonhom type:
             |_cl_ring_element(*)(cl_heap_ring*,const-cl_I&)|
             cpp-name: "canonhom")
   (mul type:
        |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&,const_cl_ring_element&)|
        cpp-name: "mul")
   (square type:
           |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&)|
           cpp-name: "square")
   (expt_pos type:
             |_cl_ring_element(*)(cl_heap_ring*,const_cl_ring_element&,const-cl_I&)|
             cpp-name: "expt_pos")
   (operator= (_cl_ring_mulops&) _cl_ring_mulops& "operator =" implicitp: t))
  copy-constructor: (_cl_ring_mulops&)
  constructor: ()
  type: _cl_ring_mulops*
  class: "_cl_ring_mulops")
;; Definition of class cl_heap_ring
;; At /home/bruno/cln/include/cl_ring.h:196
(define-cpp-class cl_heap_ring (cl_heap)
  ((operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (_fprint (ostream& const_cl_ring_element&) void "_fprint")
   (_equal (const_cl_ring_element& const_cl_ring_element&)
           cl_boolean
           "_equal")
   (_zero () _cl_ring_element "_zero")
   (_zerop (const_cl_ring_element&) cl_boolean "_zerop")
   (_plus (const_cl_ring_element& const_cl_ring_element&)
          _cl_ring_element
          "_plus")
   (_minus (const_cl_ring_element& const_cl_ring_element&)
           _cl_ring_element
           "_minus")
   (_uminus (const_cl_ring_element&) _cl_ring_element "_uminus")
   (_one () _cl_ring_element "_one")
   (_canonhom (const-cl_I&) _cl_ring_element "_canonhom")
   (_mul (const_cl_ring_element& const_cl_ring_element&)
         _cl_ring_element
         "_mul")
   (_square (const_cl_ring_element&) _cl_ring_element "_square")
   (_expt_pos (const_cl_ring_element& const-cl_I&)
              _cl_ring_element
              "_expt_pos")
   (fprint (ostream& const-cl_ring_element&) void "fprint")
   (equal (const-cl_ring_element& const-cl_ring_element&) cl_boolean "equal")
   (zero () cl_ring_element "zero")
   (zerop (const-cl_ring_element&) cl_boolean "zerop")
   (plus (const-cl_ring_element& const-cl_ring_element&)
         cl_ring_element
         "plus")
   (minus (const-cl_ring_element& const-cl_ring_element&)
          cl_ring_element
          "minus")
   (uminus (const-cl_ring_element&) cl_ring_element "uminus")
   (one () cl_ring_element "one")
   (canonhom (const-cl_I&) cl_ring_element "canonhom")
   (mul (const-cl_ring_element& const-cl_ring_element&) cl_ring_element "mul")
   (square (const-cl_ring_element&) cl_ring_element "square")
   (expt_pos (const-cl_ring_element& const-cl_I&) cl_ring_element "expt_pos")
   (get_property (const-cl_symbol&) cl_property* "get_property")
   (add_property (cl_property*) void "add_property")
   (operator= (cl_heap_ring&) cl_heap_ring& "operator =" implicitp: t))
  constructor: (_cl_ring_setops* _cl_ring_addops* _cl_ring_mulops*)
  copy-constructor: (cl_heap_ring&)
  type: cl_heap_ring*
  class: "cl_heap_ring")
(defoverloaded fprint (ostream& const-cl_ring_element&) void "fprint")
(defoverloaded operator<< (ostream& const-cl_ring_element&)
  ostream&
  "operator <<")
(defoverloaded operator+ (const-cl_ring_element& const-cl_ring_element&)
  cl_ring_element
  "operator +")
(defoverloaded operator- (const-cl_ring_element&)
  cl_ring_element
  "operator -")
(defoverloaded operator- (const-cl_ring_element& const-cl_ring_element&)
  cl_ring_element
  "operator -")
(defoverloaded operator== (const-cl_ring_element& const-cl_ring_element&)
  bool
  "operator ==")
(defoverloaded operator!= (const-cl_ring_element& const-cl_ring_element&)
  bool
  "operator !=")
(defoverloaded zerop (const-cl_ring_element&) cl_boolean "zerop")
(defoverloaded operator* (const-cl_ring_element& const-cl_ring_element&)
  cl_ring_element
  "operator *")
(defoverloaded square (const-cl_ring_element&) cl_ring_element "square")
(defoverloaded expt_pos (const-cl_ring_element& const-cl_I&)
  cl_ring_element
  "expt_pos")
(defoverloaded operator* (const-cl_I& const-cl_ring_element&)
  cl_ring_element
  "operator *")
(defoverloaded operator* (const-cl_ring_element& const-cl_I&)
  cl_ring_element
  "operator *")
(define-cpp-variable cl_no_ring cl_ring "cl_no_ring")
(define-cpp-converter cl_null_ring* static)
;; Definition of class cl_null_ring
;; At /home/bruno/cln/include/cl_ring.h:383
(define-cpp-class cl_null_ring (cl_ring)
  ((operator= (cl_null_ring&) cl_null_ring& "operator =" implicitp: t))
  type: cl_null_ring*
  class: "cl_null_ring")
(define-cpp-variable cl_0_ring cl_null_ring "cl_0_ring")
(define-cpp-converter cl_heap_number_ring* static)
;; Definition of class cl_heap_number_ring
;; At /home/bruno/cln/include/cl_ring.h:398
(define-cpp-class cl_heap_number_ring (cl_heap_ring)
  ((operator= (cl_heap_number_ring&)
              cl_heap_number_ring&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_heap_number_ring&)
  type: cl_heap_number_ring*
  class: "cl_heap_number_ring")
(defoverloaded cl_identity (const-cl_ring&) const-cl_ring& "cl_identity")
(defoverloaded cl_identity2 (const-cl_ring_element&)
  const-cl_ring_element&
  "cl_identity2")
(defoverloaded cl_identity (const_cl_ring_element&)
  const-cl_gcobject&
  "cl_identity")
(define-cpp-converter cl_modint_ring* static)
(define-cpp-converter cl_heap_modint_ring* static)
;; Definition of class cl_modint_ring
;; At /home/bruno/cln/include/cl_modinteger.h:28
(define-cpp-class cl_modint_ring (cl_ring)
  ((operator= (const-cl_modint_ring&) cl_modint_ring& "operator =")
   (operator-> () cl_heap_modint_ring* "operator ->"))
  constructor: ()
  constructor: (cl_heap_modint_ring*)
  copy-constructor: (const-cl_modint_ring&)
  type: cl_modint_ring*
  class: "cl_modint_ring")
(define-cpp-variable cl_modint0_ring cl_modint_ring "cl_modint0_ring")
(defoverloaded operator== (const-cl_modint_ring& const-cl_modint_ring&)
  bool
  "operator ==")
(defoverloaded operator!= (const-cl_modint_ring& const-cl_modint_ring&)
  bool
  "operator !=")
(defoverloaded operator== (const-cl_modint_ring& cl_heap_modint_ring*)
  bool
  "operator ==")
(defoverloaded operator!= (const-cl_modint_ring& cl_heap_modint_ring*)
  bool
  "operator !=")
(define-cpp-converter cl_composite_condition* static)
;; Definition of class cl_composite_condition
;; At /home/bruno/cln/include/cl_modinteger.h:70
(define-cpp-class cl_composite_condition (cl_condition)
  ((p type: cl_I cpp-name: "p")
   (factor type: cl_I cpp-name: "factor")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (name () const-char* "name")
   (print (ostream&) void "print")
   (operator= (cl_composite_condition&)
              cl_composite_condition&
              "operator ="
              implicitp: t))
  constructor: (const-cl_I&)
  constructor: (const-cl_I& const-cl_I&)
  copy-constructor: (cl_composite_condition&)
  type: cl_composite_condition*
  class: "cl_composite_condition")
(define-cpp-converter _cl_MI* static)
;; Definition of class _cl_MI
;; At /home/bruno/cln/include/cl_modinteger.h:90
(define-cpp-class _cl_MI (<class-proxy>)
  ((rep type: cl_I cpp-name: "rep")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t _cl_MI*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  constructor: (const-cl_heap_modint_ring* const-cl_I&)
  constructor: (const-cl_modint_ring& const-cl_I&)
  copy-constructor: (_cl_MI&)
  type: _cl_MI*
  class: "_cl_MI")
(define-cpp-converter cl_MI* static)
;; Definition of class cl_MI
;; At /home/bruno/cln/include/cl_modinteger.h:109
(define-cpp-class cl_MI (_cl_MI)
  ((ring () const-cl_modint_ring& "ring")
   (debug_print () void "debug_print")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_MI*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  constructor: (const-cl_modint_ring& const-cl_I&)
  constructor: (const-cl_modint_ring& const_cl_MI&)
  copy-constructor: (cl_MI&)
  type: cl_MI*
  class: "cl_MI")
(define-cpp-converter cl_MI_x* static)
;; Definition of class cl_MI_x
;; At /home/bruno/cln/include/cl_modinteger.h:134
(define-cpp-class cl_MI_x (<class-proxy>)
  ((condition type: cl_composite_condition* cpp-name: "condition"))
  constructor: (cl_composite_condition*)
  constructor: (const-cl_MI&)
  copy-constructor: (cl_MI_x&)
  type: cl_MI_x*
  class: "cl_MI_x")
(define-cpp-converter
  |void(*)(cl_heap_modint_ring**,ostream**,_cl_MI**)|
  function-type)
(define-callback-pool |void(*)(cl_heap_modint_ring*,ostream&,const_cl_MI&)| 5)
(define-cpp-converter
  |cl_boolean(*)(cl_heap_modint_ring**,_cl_MI**,_cl_MI**)|
  function-type)
(define-callback-pool
  |cl_boolean(*)(cl_heap_modint_ring*,const_cl_MI&,const_cl_MI&)|
  5)
(define-cpp-converter
  |_cl_MI*(*)(cl_heap_modint_ring**,cl_random_state**)|
  function-type)
