(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded float_precision (const-cl_DF&) uintL "float_precision")
(defoverloaded integer_decode_float (const-cl_DF&)
  cl_idecoded_float
  "integer_decode_float")
(defoverloaded scale_float (const-cl_DF& sintL) cl_DF "scale_float")
(defoverloaded scale_float (const-cl_DF& const-cl_I&) cl_DF "scale_float")
(defoverloaded signum (const-cl_DF&) cl_DF "signum")
(defoverloaded cl_float_approx (const-cl_DF&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_DF&) double "cl_double_approx")
(defoverloaded operator- (const-cl_LF&) cl_LF "operator -")
(defoverloaded cl_compare (const-cl_LF& const-cl_LF&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_LF& const-cl_LF&) bool "operator ==")
(defoverloaded operator!= (const-cl_LF& const-cl_LF&) bool "operator !=")
(defoverloaded operator<= (const-cl_LF& const-cl_LF&) bool "operator <=")
(defoverloaded operator< (const-cl_LF& const-cl_LF&) bool "operator <")
(defoverloaded operator>= (const-cl_LF& const-cl_LF&) bool "operator >=")
(defoverloaded operator> (const-cl_LF& const-cl_LF&) bool "operator >")
(defoverloaded minusp (const-cl_LF&) cl_boolean "minusp")
(defoverloaded zerop (const-cl_LF&) cl_boolean "zerop")
(defoverloaded operator+ (const-cl_LF& const-cl_LF&) cl_LF "operator +")
(defoverloaded operator- (const-cl_LF& const-cl_LF&) cl_LF "operator -")
(defoverloaded operator* (const-cl_LF& const-cl_LF&) cl_LF "operator *")
(defoverloaded operator* (const-cl_LF& const-cl_I&) cl_R "operator *")
(defoverloaded operator* (const-cl_I& const-cl_LF&) cl_R "operator *")
(defoverloaded operator* (long const-cl_LF&) cl_R "operator *")
(defoverloaded operator* (const-cl_LF& long) cl_R "operator *")
(defoverloaded square (const-cl_LF&) cl_LF "square")
(defoverloaded operator/ (const-cl_LF& const-cl_LF&) cl_LF "operator /")
(defoverloaded operator/ (const-cl_LF& const-cl_I&) cl_LF "operator /")
(defoverloaded operator/ (const-cl_LF& long) cl_LF "operator /")
(defoverloaded sqrt (const-cl_LF&) cl_LF "sqrt")
(defoverloaded recip (const-cl_LF&) cl_LF "recip")
(defoverloaded plus1 (const-cl_LF&) cl_LF "plus1")
(defoverloaded minus1 (const-cl_LF&) cl_LF "minus1")
(defoverloaded ftruncate (const-cl_LF&) cl_LF "ftruncate")
(defoverloaded fround (const-cl_LF&) cl_LF "fround")
(defoverloaded ffloor (const-cl_LF&) cl_LF "ffloor")
(defoverloaded fceiling (const-cl_LF&) cl_LF "fceiling")
(define-cpp-converter cl_LF_fdiv_t* static)
;; Definition of class cl_LF_fdiv_t
;; At /home/bruno/cln/include/cl_lfloat.h:133
(define-cpp-class cl_LF_fdiv_t (<class-proxy>)
  ((quotient type: cl_LF cpp-name: "quotient")
   (remainder type: cl_LF cpp-name: "remainder")
   (operator= (cl_LF_fdiv_t&) cl_LF_fdiv_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_LF& const-cl_LF&)
  copy-constructor: (cl_LF_fdiv_t&)
  type: cl_LF_fdiv_t*
  class: "cl_LF_fdiv_t")
(defoverloaded ffloor2 (const-cl_LF&) cl_LF_fdiv_t "ffloor2")
(defoverloaded fceiling2 (const-cl_LF&) cl_LF_fdiv_t "fceiling2")
(defoverloaded ftruncate2 (const-cl_LF&) cl_LF_fdiv_t "ftruncate2")
(defoverloaded fround2 (const-cl_LF&) cl_LF_fdiv_t "fround2")
(define-cpp-converter cl_LF_div_t* static)
;; Definition of class cl_LF_div_t
;; At /home/bruno/cln/include/cl_lfloat.h:176
(define-cpp-class cl_LF_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_LF cpp-name: "remainder")
   (operator= (cl_LF_div_t&) cl_LF_div_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_LF&)
  copy-constructor: (cl_LF_div_t&)
  type: cl_LF_div_t*
  class: "cl_LF_div_t")
(defoverloaded floor2 (const-cl_LF&) cl_LF_div_t "floor2")
(defoverloaded floor1 (const-cl_LF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_LF&) cl_LF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_LF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_LF&) cl_LF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_LF&) cl_I "truncate1")
