(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded expt_pos (const-cl_MI& const-cl_I&) cl_MI "expt_pos")
(defoverloaded recip (const-cl_MI&) cl_MI "recip")
(defoverloaded div (const-cl_MI& const-cl_MI&) cl_MI "div")
(defoverloaded div (const-cl_MI& const-cl_I&) cl_MI "div")
(defoverloaded div (const-cl_I& const-cl_MI&) cl_MI "div")
(defoverloaded expt (const-cl_MI& const-cl_I&) cl_MI "expt")
(defoverloaded operator* (const-cl_I& const-cl_MI&) cl_MI "operator *")
(defoverloaded operator* (const-cl_MI& const-cl_I&) cl_MI "operator *")
(defoverloaded jacobi (sint32 sint32) int "jacobi")
(defoverloaded jacobi (const-cl_I& const-cl_I&) int "jacobi")
(defoverloaded isprobprime (const-cl_I&) cl_boolean "isprobprime")
(defoverloaded nextprobprime (const-cl_R&) cl_I "nextprobprime")
(define-cpp-converter sqrt_mod_p_t* static)
;; Definition of class sqrt_mod_p_t
;; At /home/bruno/cln/include/cl_numtheory.h:46
(define-cpp-class sqrt_mod_p_t (<class-proxy>)
  ((condition type: cl_composite_condition* cpp-name: "condition")
   (solutions type: int cpp-name: "solutions")
   (factor type: cl_I cpp-name: "factor")
   (solution type: cl_MI[] constp: t cpp-name: "solution"))
  constructor: ()
  constructor: (cl_composite_condition*)
  constructor: (int)
  constructor: (int const-cl_MI&)
  constructor: (int const-cl_MI& const-cl_MI&)
  copy-constructor: (sqrt_mod_p_t&)
  type: sqrt_mod_p_t*
  class: "sqrt_mod_p_t")
(defoverloaded sqrt_mod_p (const-cl_modint_ring& const-cl_MI&)
  sqrt_mod_p_t
  "sqrt_mod_p")
(define-cpp-converter cornacchia_t* static)
;; Definition of class cornacchia_t
;; At /home/bruno/cln/include/cl_numtheory.h:69
(define-cpp-class cornacchia_t (<class-proxy>)
  ((condition type: cl_composite_condition* cpp-name: "condition")
   (solutions type: int cpp-name: "solutions")
   (solution_x type: cl_I cpp-name: "solution_x")
   (solution_y type: cl_I cpp-name: "solution_y")
   (operator= (cornacchia_t&) cornacchia_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (cl_composite_condition*)
  constructor: (int)
  constructor: (int const-cl_I& const-cl_I&)
  copy-constructor: (cornacchia_t&)
  type: cornacchia_t*
  class: "cornacchia_t")
(defoverloaded cornacchia1 (const-cl_I& const-cl_I&)
  cornacchia_t
  "cornacchia1")
(defoverloaded cornacchia4 (const-cl_I& const-cl_I&)
  cornacchia_t
  "cornacchia4")
(define-cpp-converter cl_V_any* static)
;; Definition of class cl_V_any
;; At /home/bruno/cln/include/cl_V.h:8
(define-cpp-class cl_V_any (cl_gcpointer)
  ((operator= (const-cl_V_any&) cl_V_any& "operator ="))
  constructor: ()
  copy-constructor: (const-cl_V_any&)
  type: cl_V_any*
  class: "cl_V_any")
(define-cpp-converter cl_univpoly_ring* static)
(define-cpp-converter cl_heap_univpoly_ring* static)
;; Definition of class cl_univpoly_ring
;; At /home/bruno/cln/include/cl_univpoly.h:19
(define-cpp-class cl_univpoly_ring (cl_ring)
  ((operator= (const-cl_univpoly_ring&) cl_univpoly_ring& "operator =")
   (operator-> () cl_heap_univpoly_ring* "operator ->"))
  constructor: ()
  constructor: (cl_heap_univpoly_ring*)
  copy-constructor: (const-cl_univpoly_ring&)
  type: cl_univpoly_ring*
  class: "cl_univpoly_ring")
(defoverloaded operator== (const-cl_univpoly_ring& const-cl_univpoly_ring&)
  bool
  "operator ==")
(defoverloaded operator!= (const-cl_univpoly_ring& const-cl_univpoly_ring&)
  bool
  "operator !=")
(defoverloaded operator== (const-cl_univpoly_ring& cl_heap_univpoly_ring*)
  bool
  "operator ==")
(defoverloaded operator!= (const-cl_univpoly_ring& cl_heap_univpoly_ring*)
  bool
  "operator !=")
(define-cpp-converter _cl_UP* static)
;; Definition of class _cl_UP
;; At /home/bruno/cln/include/cl_univpoly.h:59
(define-cpp-class _cl_UP (<class-proxy>)
  ((rep type: cl_gcpointer cpp-name: "rep")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t _cl_UP*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  constructor: (const-cl_heap_univpoly_ring* const-cl_V_any&)
  constructor: (const-cl_univpoly_ring& const-cl_V_any&)
  copy-constructor: (_cl_UP&)
  type: _cl_UP*
  class: "_cl_UP")
(define-cpp-converter cl_UP* static)
;; Definition of class cl_UP
;; At /home/bruno/cln/include/cl_univpoly.h:77
(define-cpp-class cl_UP (_cl_UP)
  ((ring () const-cl_univpoly_ring& "ring")
   (set_coeff (uintL const-cl_ring_element&) void "set_coeff")
   (finalize () void "finalize")
   (|operator()| (const-cl_ring_element&) cl_ring_element "operator ()")
   (debug_print () void "debug_print")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_UP*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: (const-cl_univpoly_ring& const-cl_V_any&)
  constructor: (const-cl_univpoly_ring& const_cl_UP&)
  copy-constructor: (cl_UP&)
  type: cl_UP*
  class: "cl_UP")
(define-cpp-converter
  |void(*)(cl_heap_univpoly_ring**,ostream**,_cl_UP**)|
  function-type)
(define-callback-pool |void(*)(cl_heap_univpoly_ring*,ostream&,const_cl_UP&)| 5)
(define-cpp-converter
  |cl_boolean(*)(cl_heap_univpoly_ring**,_cl_UP**,_cl_UP**)|
  function-type)
(define-callback-pool
  |cl_boolean(*)(cl_heap_univpoly_ring*,const_cl_UP&,const_cl_UP&)|
  5)
(define-cpp-converter _cl_univpoly_setops* static)
;; Definition of class _cl_univpoly_setops
;; At /home/bruno/cln/include/cl_univpoly.h:110
(define-cpp-class _cl_univpoly_setops (<class-proxy>)
  ((fprint type:
           |void(*)(cl_heap_univpoly_ring*,ostream&,const_cl_UP&)|
           cpp-name: "fprint")
   (equal type:
          |cl_boolean(*)(cl_heap_univpoly_ring*,const_cl_UP&,const_cl_UP&)|
          cpp-name: "equal")
   (operator= (_cl_univpoly_setops&)
              _cl_univpoly_setops&
              "operator ="
              implicitp: t))
  copy-constructor: (_cl_univpoly_setops&)
  constructor: ()
  type: _cl_univpoly_setops*
  class: "_cl_univpoly_setops")
(define-cpp-converter |_cl_UP*(*)(cl_heap_univpoly_ring**)| function-type)
(define-callback-pool |_cl_UP(*)(cl_heap_univpoly_ring*)| 5)
(define-cpp-converter
  |cl_boolean(*)(cl_heap_univpoly_ring**,_cl_UP**)|
  function-type)
(define-callback-pool |cl_boolean(*)(cl_heap_univpoly_ring*,const_cl_UP&)| 5)
(define-cpp-converter
  |_cl_UP*(*)(cl_heap_univpoly_ring**,_cl_UP**,_cl_UP**)|
  function-type)
(define-callback-pool
  |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&,const_cl_UP&)|
  5)
(define-cpp-converter
  |_cl_UP*(*)(cl_heap_univpoly_ring**,_cl_UP**)|
  function-type)
(define-callback-pool |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&)| 5)
(define-cpp-converter _cl_univpoly_addops* static)
;; Definition of class _cl_univpoly_addops
;; At /home/bruno/cln/include/cl_univpoly.h:118
(define-cpp-class _cl_univpoly_addops (<class-proxy>)
  ((zero type: |_cl_UP(*)(cl_heap_univpoly_ring*)| cpp-name: "zero")
   (zerop type:
          |cl_boolean(*)(cl_heap_univpoly_ring*,const_cl_UP&)|
          cpp-name: "zerop")
   (plus type:
         |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&,const_cl_UP&)|
         cpp-name: "plus")
   (minus type:
          |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&,const_cl_UP&)|
          cpp-name: "minus")
   (uminus type:
           |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&)|
           cpp-name: "uminus")
   (operator= (_cl_univpoly_addops&)
              _cl_univpoly_addops&
              "operator ="
              implicitp: t))
  copy-constructor: (_cl_univpoly_addops&)
  constructor: ()
  type: _cl_univpoly_addops*
  class: "_cl_univpoly_addops")
(define-cpp-converter
  |_cl_UP*(*)(cl_heap_univpoly_ring**,cl_I**)|
  function-type)
(define-callback-pool |_cl_UP(*)(cl_heap_univpoly_ring*,const-cl_I&)| 5)
