(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(define-cpp-converter
  |_cl_UP*(*)(cl_heap_univpoly_ring**,_cl_UP**,cl_I**)|
  function-type)
(define-callback-pool
  |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&,const-cl_I&)|
  5)
(define-cpp-converter _cl_univpoly_mulops* static)
;; Definition of class _cl_univpoly_mulops
;; At /home/bruno/cln/include/cl_univpoly.h:129
(define-cpp-class _cl_univpoly_mulops (<class-proxy>)
  ((one type: |_cl_UP(*)(cl_heap_univpoly_ring*)| cpp-name: "one")
   (canonhom type:
             |_cl_UP(*)(cl_heap_univpoly_ring*,const-cl_I&)|
             cpp-name: "canonhom")
   (mul type:
        |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&,const_cl_UP&)|
        cpp-name: "mul")
   (square type:
           |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&)|
           cpp-name: "square")
   (expt_pos type:
             |_cl_UP(*)(cl_heap_univpoly_ring*,const_cl_UP&,const-cl_I&)|
             cpp-name: "expt_pos")
   (operator= (_cl_univpoly_mulops&)
              _cl_univpoly_mulops&
              "operator ="
              implicitp: t))
  copy-constructor: (_cl_univpoly_mulops&)
  constructor: ()
  type: _cl_univpoly_mulops*
  class: "_cl_univpoly_mulops")
(define-cpp-converter
  |sintL(*)(cl_heap_univpoly_ring**,_cl_UP**)|
  function-type)
(define-callback-pool |sintL(*)(cl_heap_univpoly_ring*,const_cl_UP&)| 5)
(define-cpp-converter
  |_cl_UP*(*)(cl_heap_univpoly_ring**,cl_ring_element**,uintL)|
  function-type)
(define-callback-pool
  |_cl_UP(*)(cl_heap_univpoly_ring*,const-cl_ring_element&,uintL)|
  5)
(define-cpp-converter
  |cl_ring_element*(*)(cl_heap_univpoly_ring**,_cl_UP**,uintL)|
  function-type)
(define-callback-pool
  |cl_ring_element(*)(cl_heap_univpoly_ring*,const_cl_UP&,uintL)|
  5)
(define-cpp-converter |_cl_UP*(*)(cl_heap_univpoly_ring**,sintL)| function-type)
(define-callback-pool |_cl_UP(*)(cl_heap_univpoly_ring*,sintL)| 5)
(define-cpp-converter
  |void(*)(cl_heap_univpoly_ring**,_cl_UP**,uintL,cl_ring_element**)|
  function-type)
(define-callback-pool
  |void(*)(cl_heap_univpoly_ring*,_cl_UP&,uintL,const-cl_ring_element&)|
  5)
(define-cpp-converter |void(*)(cl_heap_univpoly_ring**,_cl_UP**)| function-type)
(define-callback-pool |void(*)(cl_heap_univpoly_ring*,_cl_UP&)| 5)
(define-cpp-converter
  |cl_ring_element*(*)(cl_heap_univpoly_ring**,_cl_UP**,cl_ring_element**)|
  function-type)
(define-callback-pool
  |cl_ring_element(*)(cl_heap_univpoly_ring*,const_cl_UP&,const-cl_ring_element&)|
  5)
(define-cpp-converter _cl_univpoly_polyops* static)
;; Definition of class _cl_univpoly_polyops
;; At /home/bruno/cln/include/cl_univpoly.h:141
(define-cpp-class _cl_univpoly_polyops (<class-proxy>)
  ((degree type:
           |sintL(*)(cl_heap_univpoly_ring*,const_cl_UP&)|
           cpp-name: "degree")
   (monomial type:
             |_cl_UP(*)(cl_heap_univpoly_ring*,const-cl_ring_element&,uintL)|
             cpp-name: "monomial")
   (coeff type:
          |cl_ring_element(*)(cl_heap_univpoly_ring*,const_cl_UP&,uintL)|
          cpp-name: "coeff")
   (create type: |_cl_UP(*)(cl_heap_univpoly_ring*,sintL)| cpp-name: "create")
   (set_coeff type:
              |void(*)(cl_heap_univpoly_ring*,_cl_UP&,uintL,const-cl_ring_element&)|
              cpp-name: "set_coeff")
   (finalize type:
             |void(*)(cl_heap_univpoly_ring*,_cl_UP&)|
             cpp-name: "finalize")
   (eval type:
         |cl_ring_element(*)(cl_heap_univpoly_ring*,const_cl_UP&,const-cl_ring_element&)|
         cpp-name: "eval")
   (operator= (_cl_univpoly_polyops&)
              _cl_univpoly_polyops&
              "operator ="
              implicitp: t))
  copy-constructor: (_cl_univpoly_polyops&)
  constructor: ()
  type: _cl_univpoly_polyops*
  class: "_cl_univpoly_polyops")
;; Definition of class cl_heap_univpoly_ring
;; At /home/bruno/cln/include/cl_univpoly.h:171
(define-cpp-class cl_heap_univpoly_ring (cl_heap)
  ((basering type: cl_ring cpp-name: "basering")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (_fprint (ostream& const_cl_UP&) void "_fprint")
   (_equal (const_cl_UP& const_cl_UP&) cl_boolean "_equal")
   (_zero () _cl_UP "_zero")
   (_zerop (const_cl_UP&) cl_boolean "_zerop")
   (_plus (const_cl_UP& const_cl_UP&) _cl_UP "_plus")
   (_minus (const_cl_UP& const_cl_UP&) _cl_UP "_minus")
   (_uminus (const_cl_UP&) _cl_UP "_uminus")
   (_one () _cl_UP "_one")
   (_canonhom (const-cl_I&) _cl_UP "_canonhom")
   (_mul (const_cl_UP& const_cl_UP&) _cl_UP "_mul")
   (_square (const_cl_UP&) _cl_UP "_square")
   (_expt_pos (const_cl_UP& const-cl_I&) _cl_UP "_expt_pos")
   (_degree (const_cl_UP&) sintL "_degree")
   (_monomial (const-cl_ring_element& uintL) _cl_UP "_monomial")
   (_coeff (const_cl_UP& uintL) cl_ring_element "_coeff")
   (_create (sintL) _cl_UP "_create")
   (_set_coeff (_cl_UP& uintL const-cl_ring_element&) void "_set_coeff")
   (_finalize (_cl_UP&) void "_finalize")
   (_eval (const_cl_UP& const-cl_ring_element&) cl_ring_element "_eval")
   (fprint (ostream& const-cl_UP&) void "fprint")
   (equal (const-cl_UP& const-cl_UP&) cl_boolean "equal")
   (zero () cl_UP "zero")
   (zerop (const-cl_UP&) cl_boolean "zerop")
   (plus (const-cl_UP& const-cl_UP&) cl_UP "plus")
   (minus (const-cl_UP& const-cl_UP&) cl_UP "minus")
   (uminus (const-cl_UP&) cl_UP "uminus")
   (one () cl_UP "one")
   (canonhom (const-cl_I&) cl_UP "canonhom")
   (mul (const-cl_UP& const-cl_UP&) cl_UP "mul")
   (square (const-cl_UP&) cl_UP "square")
   (expt_pos (const-cl_UP& const-cl_I&) cl_UP "expt_pos")
   (degree (const-cl_UP&) sintL "degree")
   (monomial (const-cl_ring_element& uintL) cl_UP "monomial")
   (coeff (const-cl_UP& uintL) cl_ring_element "coeff")
   (create (sintL) cl_UP "create")
   (set_coeff (cl_UP& uintL const-cl_ring_element&) void "set_coeff")
   (finalize (cl_UP&) void "finalize")
   (eval (const-cl_UP& const-cl_ring_element&) cl_ring_element "eval")
   (get_property (const-cl_symbol&) cl_property* "get_property")
   (add_property (cl_property*) void "add_property")
   (operator= (cl_heap_univpoly_ring&)
              cl_heap_univpoly_ring&
              "operator ="
              implicitp: t))
  constructor: (const-cl_ring&
                 _cl_univpoly_setops*
                 _cl_univpoly_addops*
                 _cl_univpoly_mulops*
                 _cl_univpoly_polyops*)
  copy-constructor: (cl_heap_univpoly_ring&)
  type: cl_heap_univpoly_ring*
  class: "cl_heap_univpoly_ring")
(defoverloaded cl_find_univpoly_ring (const-cl_ring&)
  cl_univpoly_ring
  "cl_find_univpoly_ring")
(defoverloaded cl_find_univpoly_ring (const-cl_ring& const-cl_symbol&)
  cl_univpoly_ring
  "cl_find_univpoly_ring")
(defoverloaded fprint (ostream& const-cl_UP&) void "fprint")
(defoverloaded operator<< (ostream& const-cl_UP&) ostream& "operator <<")
(defoverloaded operator+ (const-cl_UP& const-cl_UP&) cl_UP "operator +")
(defoverloaded operator- (const-cl_UP&) cl_UP "operator -")
(defoverloaded operator- (const-cl_UP& const-cl_UP&) cl_UP "operator -")
(defoverloaded operator== (const-cl_UP& const-cl_UP&) bool "operator ==")
(defoverloaded operator!= (const-cl_UP& const-cl_UP&) bool "operator !=")
(defoverloaded zerop (const-cl_UP&) cl_boolean "zerop")
(defoverloaded operator* (const-cl_UP& const-cl_UP&) cl_UP "operator *")
(defoverloaded square (const-cl_UP&) cl_UP "square")
(defoverloaded expt_pos (const-cl_UP& const-cl_I&) cl_UP "expt_pos")
(defoverloaded operator* (const-cl_I& const-cl_UP&) cl_UP "operator *")
(defoverloaded operator* (const-cl_UP& const-cl_I&) cl_UP "operator *")
(defoverloaded degree (const-cl_UP&) sintL "degree")
(defoverloaded coeff (const-cl_UP& uintL) cl_ring_element "coeff")
(defoverloaded set_coeff (cl_UP& uintL const-cl_ring_element&)
  void
  "set_coeff")
(defoverloaded finalize (cl_UP&) void "finalize")
(defoverloaded deriv (const-cl_UP&) cl_UP "deriv")
(define-cpp-variable
  cl_no_univpoly_ring
  cl_univpoly_ring
  "cl_no_univpoly_ring")
(define-cpp-converter cl_univpoly_integer_ring* static)
(define-cpp-converter cl_heap_univpoly_integer_ring* static)
;; Definition of class cl_univpoly_integer_ring
;; At /home/bruno/cln/include/cl_univpoly_integer.h:23
(define-cpp-class cl_univpoly_integer_ring (cl_univpoly_ring)
  ((operator= (const-cl_univpoly_integer_ring&)
              cl_univpoly_integer_ring&
              "operator =")
   (operator-> () cl_heap_univpoly_integer_ring* "operator ->"))
  constructor: ()
  copy-constructor: (const-cl_univpoly_integer_ring&)
  type: cl_univpoly_integer_ring*
  class: "cl_univpoly_integer_ring")
(define-cpp-converter cl_UP_I* static)
;; Definition of class cl_UP_I
;; At /home/bruno/cln/include/cl_univpoly_integer.h:39
(define-cpp-class cl_UP_I (cl_UP)
  ((ring () const-cl_univpoly_integer_ring& "ring")
   (set_coeff (uintL const-cl_I&) void "set_coeff")
   (finalize () void "finalize")
   (|operator()| (const-cl_I&) cl_I "operator ()")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_UP_I*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  copy-constructor: (cl_UP_I&)
  type: cl_UP_I*
  class: "cl_UP_I")
;; Definition of class cl_heap_univpoly_integer_ring
;; At /home/bruno/cln/include/cl_univpoly_integer.h:55
(define-cpp-class cl_heap_univpoly_integer_ring (cl_heap_univpoly_ring)
  ((operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (fprint (ostream& const-cl_UP_I&) void "fprint")
   (equal (const-cl_UP_I& const-cl_UP_I&) cl_boolean "equal")
   (zero () cl_UP_I "zero")
   (zerop (const-cl_UP_I&) cl_boolean "zerop")
   (plus (const-cl_UP_I& const-cl_UP_I&) cl_UP_I "plus")
   (minus (const-cl_UP_I& const-cl_UP_I&) cl_UP_I "minus")
   (uminus (const-cl_UP_I&) cl_UP_I "uminus")
   (one () cl_UP_I "one")
   (canonhom (const-cl_I&) cl_UP_I "canonhom")
   (mul (const-cl_UP_I& const-cl_UP_I&) cl_UP_I "mul")
   (square (const-cl_UP_I&) cl_UP_I "square")
   (expt_pos (const-cl_UP_I& const-cl_I&) cl_UP_I "expt_pos")
   (degree (const-cl_UP_I&) sintL "degree")
   (monomial (const-cl_I& uintL) cl_UP_I "monomial")
   (coeff (const-cl_UP_I& uintL) cl_I "coeff")
   (create (sintL) cl_UP_I "create")
   (set_coeff (cl_UP_I& uintL const-cl_I&) void "set_coeff")
   (finalize (cl_UP_I&) void "finalize")
   (eval (const-cl_UP_I& const-cl_I&) cl_I "eval")
   (operator= (cl_heap_univpoly_integer_ring&)
              cl_heap_univpoly_integer_ring&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_heap_univpoly_integer_ring&)
  type: cl_heap_univpoly_integer_ring*
  class: "cl_heap_univpoly_integer_ring")
(defoverloaded operator+ (const-cl_UP_I& const-cl_UP_I&) cl_UP_I "operator +")
(defoverloaded operator- (const-cl_UP_I&) cl_UP_I "operator -")
