(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded dpb (const-cl_I& const-cl_I& const-cl_byte&) cl_I "dpb")
(defoverloaded operator* (const-cl_I& const-cl_I&) cl_I "operator *")
(defoverloaded operator* (long const-cl_I&) cl_I "operator *")
(defoverloaded operator* (const-cl_I& long) cl_I "operator *")
(defoverloaded square (const-cl_I&) cl_I "square")
(defoverloaded expt_pos (const-cl_I& uintL) cl_I "expt_pos")
(defoverloaded expt_pos (const-cl_I& const-cl_I&) cl_I "expt_pos")
(defoverloaded factorial (uintL) cl_I "factorial")
(defoverloaded binomial (uintL uintL) cl_I "binomial")
(define-cpp-converter cl_I_div_t* static)
;; Definition of class cl_I_div_t
;; At /home/bruno/cln/include/cl_integer.h:330
(define-cpp-class cl_I_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_I cpp-name: "remainder")
   (operator= (cl_I_div_t&) cl_I_div_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_I&)
  copy-constructor: (cl_I_div_t&)
  type: cl_I_div_t*
  class: "cl_I_div_t")
(defoverloaded exquopos (const-cl_I& const-cl_I&) cl_I "exquopos")
(defoverloaded exquo (const-cl_I& const-cl_I&) cl_I "exquo")
(defoverloaded mod (const-cl_I& const-cl_I&) cl_I "mod")
(defoverloaded rem (const-cl_I& const-cl_I&) cl_I "rem")
(defoverloaded floor2 (const-cl_I& const-cl_I&) cl_I_div_t "floor2")
(defoverloaded floor1 (const-cl_I& const-cl_I&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_I& const-cl_I&) cl_I_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_I& const-cl_I&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_I& const-cl_I&) cl_I_div_t "truncate2")
(defoverloaded truncate1 (const-cl_I& const-cl_I&) cl_I "truncate1")
(defoverloaded round2 (const-cl_I& const-cl_I&) cl_I_div_t "round2")
(defoverloaded round1 (const-cl_I& const-cl_I&) cl_I "round1")
(defoverloaded xgcd (const-cl_I& const-cl_I& cl_I* cl_I*) cl_I "xgcd")
(defoverloaded isqrt (const-cl_I& cl_I*) cl_boolean "isqrt")
(defoverloaded isqrt (const-cl_I&) cl_I "isqrt")
(defoverloaded sqrtp (const-cl_I& cl_I*) cl_boolean "sqrtp")
(defoverloaded rootp (const-cl_I& uintL cl_I*) cl_boolean "rootp")
(defoverloaded rootp (const-cl_I& const-cl_I& cl_I*) cl_boolean "rootp")
(defoverloaded signum (const-cl_I&) cl_I "signum")
(defoverloaded mul_10_plus_x (const-cl_I& unsigned-char) cl_I "mul_10_plus_x")
(defoverloaded cl_recip2adic (uintL const-cl_I&) cl_I "cl_recip2adic")
(defoverloaded cl_div2adic (uintL const-cl_I& const-cl_I&) cl_I "cl_div2adic")
(defoverloaded cl_float_approx (const-cl_I&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_I&) double "cl_double_approx")
(defoverloaded zerop (const-cl_N&) cl_boolean "zerop")
(defoverloaded complex (const-cl_R& const-cl_R&) cl_N "complex")
(defoverloaded realpart (const-cl_N&) cl_R "realpart")
(defoverloaded imagpart (const-cl_N&) cl_R "imagpart")
(defoverloaded conjugate (const-cl_N&) cl_N "conjugate")
(defoverloaded operator- (const-cl_N&) cl_N "operator -")
(defoverloaded operator+ (const-cl_N& const-cl_N&) cl_N "operator +")
(defoverloaded operator- (const-cl_N& const-cl_N&) cl_N "operator -")
(defoverloaded operator* (const-cl_N& const-cl_N&) cl_N "operator *")
(defoverloaded square (const-cl_N&) cl_N "square")
(defoverloaded operator/ (const-cl_N& const-cl_N&) cl_N "operator /")
(defoverloaded recip (const-cl_N&) cl_N "recip")
(defoverloaded plus1 (const-cl_N&) cl_N "plus1")
(defoverloaded minus1 (const-cl_N&) cl_N "minus1")
(defoverloaded signum (const-cl_N&) cl_N "signum")
