// cl_I internals for ash

#ifndef _CL_I_ASH_H
#define _CL_I_ASH_H

#include "cl_number.h"
#include "cl_macros.h"

nonreturning_function(extern, cl_ash_error, (const cl_I& badamount));

#endif /* _CL_I_ASH_H */
