(define-cpp-type sintL long
  cpp-type-dcl: "sintL %s"
  cpp-arg-type: "((sintL)(%s))"
  cast: "((sintL)(%s))")
(define-cpp-variable-setter cl_inhibit_floating_point_underflow)
(define-cpp-type uintC unsigned
  cpp-type-dcl: "uintC %s"
  cpp-arg-type: "((uintC)(%s))"
  cast: "((uintC)(%s))")
(define-cpp-data-slot-setter cl_R_div_t quotient)
(define-cpp-data-slot-setter cl_R_div_t remainder)
(define-cpp-type cl_R_div_t& static
  id: cl_R_div_t*
  class: cl_R_div_t
  cpp-type-dcl: "cl_R_div_t &%s"
  cast: "((cl_R_div_t &)(%s))"
  arg-type: "cl_R_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_R_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_R_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_R_div_t static
  id: cl_R_div_t*
  class: cl_R_div_t
  cpp-type-dcl: "cl_R_div_t %s"
  cast: "((cl_R_div_t)(%s))"
  arg-type: "cl_R_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_R_div_t *%s"
  res-converter: "new cl_R_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_R_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_R_div_t* static
  id: cl_R_div_t*
  class: cl_R_div_t
  cpp-type-dcl: "cl_R_div_t *%s"
  cast: "((cl_R_div_t *)(%s))"
  callback-res-converter: "(cl_R_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_R_fdiv_t quotient)
(define-cpp-data-slot-setter cl_R_fdiv_t remainder)
(define-cpp-type cl_R_fdiv_t& static
  id: cl_R_fdiv_t*
  class: cl_R_fdiv_t
  cpp-type-dcl: "cl_R_fdiv_t &%s"
  cast: "((cl_R_fdiv_t &)(%s))"
  arg-type: "cl_R_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_R_fdiv_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_R_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_R_fdiv_t static
  id: cl_R_fdiv_t*
  class: cl_R_fdiv_t
  cpp-type-dcl: "cl_R_fdiv_t %s"
  cast: "((cl_R_fdiv_t)(%s))"
  arg-type: "cl_R_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_R_fdiv_t *%s"
  res-converter: "new cl_R_fdiv_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_R_fdiv_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_R_fdiv_t* static
  id: cl_R_fdiv_t*
  class: cl_R_fdiv_t
  cpp-type-dcl: "cl_R_fdiv_t *%s"
  cast: "((cl_R_fdiv_t *)(%s))"
  callback-res-converter: "(cl_R_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_cos_sin_t cos)
(define-cpp-data-slot-setter cl_cos_sin_t sin)
(define-cpp-type cl_cos_sin_t& static
  id: cl_cos_sin_t*
  class: cl_cos_sin_t
  cpp-type-dcl: "cl_cos_sin_t &%s"
  cast: "((cl_cos_sin_t &)(%s))"
  arg-type: "cl_cos_sin_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_cos_sin_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_cos_sin_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_cos_sin_t static
  id: cl_cos_sin_t*
  class: cl_cos_sin_t
  cpp-type-dcl: "cl_cos_sin_t %s"
  cast: "((cl_cos_sin_t)(%s))"
  arg-type: "cl_cos_sin_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_cos_sin_t *%s"
  res-converter: "new cl_cos_sin_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_cos_sin_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_cos_sin_t* static
  id: cl_cos_sin_t*
  class: cl_cos_sin_t
  cpp-type-dcl: "cl_cos_sin_t *%s"
  cast: "((cl_cos_sin_t *)(%s))"
  callback-res-converter: "(cl_cos_sin_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_cosh_sinh_t cosh)
(define-cpp-data-slot-setter cl_cosh_sinh_t sinh)
(define-cpp-type cl_cosh_sinh_t& static
  id: cl_cosh_sinh_t*
  class: cl_cosh_sinh_t
  cpp-type-dcl: "cl_cosh_sinh_t &%s"
  cast: "((cl_cosh_sinh_t &)(%s))"
  arg-type: "cl_cosh_sinh_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_cosh_sinh_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_cosh_sinh_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_cosh_sinh_t static
  id: cl_cosh_sinh_t*
  class: cl_cosh_sinh_t
  cpp-type-dcl: "cl_cosh_sinh_t %s"
  cast: "((cl_cosh_sinh_t)(%s))"
  arg-type: "cl_cosh_sinh_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_cosh_sinh_t *%s"
  res-converter: "new cl_cosh_sinh_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_cosh_sinh_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_cosh_sinh_t* static
  id: cl_cosh_sinh_t*
  class: cl_cosh_sinh_t
  cpp-type-dcl: "cl_cosh_sinh_t *%s"
  cast: "((cl_cosh_sinh_t *)(%s))"
  callback-res-converter: "(cl_cosh_sinh_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_SF_fdiv_t quotient)
(define-cpp-data-slot-setter cl_SF_fdiv_t remainder)
(define-cpp-type cl_SF_fdiv_t& static
  id: cl_SF_fdiv_t*
  class: cl_SF_fdiv_t
  cpp-type-dcl: "cl_SF_fdiv_t &%s"
  cast: "((cl_SF_fdiv_t &)(%s))"
  arg-type: "cl_SF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_SF_fdiv_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_SF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_SF_fdiv_t static
  id: cl_SF_fdiv_t*
  class: cl_SF_fdiv_t
  cpp-type-dcl: "cl_SF_fdiv_t %s"
  cast: "((cl_SF_fdiv_t)(%s))"
  arg-type: "cl_SF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_SF_fdiv_t *%s"
  res-converter: "new cl_SF_fdiv_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_SF_fdiv_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_SF_fdiv_t* static
  id: cl_SF_fdiv_t*
  class: cl_SF_fdiv_t
  cpp-type-dcl: "cl_SF_fdiv_t *%s"
  cast: "((cl_SF_fdiv_t *)(%s))"
  callback-res-converter: "(cl_SF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_SF_div_t quotient)
(define-cpp-data-slot-setter cl_SF_div_t remainder)
(define-cpp-type cl_SF_div_t& static
  id: cl_SF_div_t*
  class: cl_SF_div_t
  cpp-type-dcl: "cl_SF_div_t &%s"
  cast: "((cl_SF_div_t &)(%s))"
  arg-type: "cl_SF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_SF_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_SF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_SF_div_t static
  id: cl_SF_div_t*
  class: cl_SF_div_t
  cpp-type-dcl: "cl_SF_div_t %s"
  cast: "((cl_SF_div_t)(%s))"
  arg-type: "cl_SF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_SF_div_t *%s"
  res-converter: "new cl_SF_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_SF_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_SF_div_t* static
  id: cl_SF_div_t*
  class: cl_SF_div_t
  cpp-type-dcl: "cl_SF_div_t *%s"
  cast: "((cl_SF_div_t *)(%s))"
  callback-res-converter: "(cl_SF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_decoded_sfloat mantissa)
(define-cpp-data-slot-setter cl_decoded_sfloat exponent)
(define-cpp-data-slot-setter cl_decoded_sfloat sign)
(define-cpp-type cl_decoded_sfloat& static
  id: cl_decoded_sfloat*
  class: cl_decoded_sfloat
  cpp-type-dcl: "cl_decoded_sfloat &%s"
  cast: "((cl_decoded_sfloat &)(%s))"
  arg-type: "cl_decoded_sfloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_sfloat *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_sfloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_sfloat static
  id: cl_decoded_sfloat*
  class: cl_decoded_sfloat
  cpp-type-dcl: "cl_decoded_sfloat %s"
  cast: "((cl_decoded_sfloat)(%s))"
  arg-type: "cl_decoded_sfloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_sfloat *%s"
  res-converter: "new cl_decoded_sfloat (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_sfloat *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_sfloat* static
  id: cl_decoded_sfloat*
  class: cl_decoded_sfloat
  cpp-type-dcl: "cl_decoded_sfloat *%s"
  cast: "((cl_decoded_sfloat *)(%s))"
  callback-res-converter: "(cl_decoded_sfloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_FF_fdiv_t quotient)
(define-cpp-data-slot-setter cl_FF_fdiv_t remainder)
(define-cpp-type cl_FF_fdiv_t& static
  id: cl_FF_fdiv_t*
  class: cl_FF_fdiv_t
  cpp-type-dcl: "cl_FF_fdiv_t &%s"
  cast: "((cl_FF_fdiv_t &)(%s))"
  arg-type: "cl_FF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_FF_fdiv_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_FF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_FF_fdiv_t static
  id: cl_FF_fdiv_t*
  class: cl_FF_fdiv_t
  cpp-type-dcl: "cl_FF_fdiv_t %s"
  cast: "((cl_FF_fdiv_t)(%s))"
  arg-type: "cl_FF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_FF_fdiv_t *%s"
  res-converter: "new cl_FF_fdiv_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_FF_fdiv_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_FF_fdiv_t* static
  id: cl_FF_fdiv_t*
  class: cl_FF_fdiv_t
  cpp-type-dcl: "cl_FF_fdiv_t *%s"
  cast: "((cl_FF_fdiv_t *)(%s))"
  callback-res-converter: "(cl_FF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_FF_div_t quotient)
(define-cpp-data-slot-setter cl_FF_div_t remainder)
(define-cpp-type cl_FF_div_t& static
  id: cl_FF_div_t*
  class: cl_FF_div_t
  cpp-type-dcl: "cl_FF_div_t &%s"
  cast: "((cl_FF_div_t &)(%s))"
  arg-type: "cl_FF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_FF_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_FF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_FF_div_t static
  id: cl_FF_div_t*
  class: cl_FF_div_t
  cpp-type-dcl: "cl_FF_div_t %s"
  cast: "((cl_FF_div_t)(%s))"
  arg-type: "cl_FF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_FF_div_t *%s"
  res-converter: "new cl_FF_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_FF_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_FF_div_t* static
  id: cl_FF_div_t*
  class: cl_FF_div_t
  cpp-type-dcl: "cl_FF_div_t *%s"
  cast: "((cl_FF_div_t *)(%s))"
  callback-res-converter: "(cl_FF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_decoded_ffloat mantissa)
