(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded fprinthexadecimal (ostream& const-cl_I&)
  void
  "fprinthexadecimal")
(defoverloaded operator<< (ostream& const-cl_number&) ostream& "operator <<")
(defoverloaded operator>> (istream& cl_N&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_R&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_RA&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_I&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_F&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_SF&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_FF&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_DF&) istream& "operator >>")
(defoverloaded operator>> (istream& cl_LF&) istream& "operator >>")
(define-cpp-converter cl_read_float_flags* static)
;; Definition of class cl_read_float_flags
;; At /home/bruno/cln/include/cl_input.h:10
(define-cpp-class cl_read_float_flags (<class-proxy>)
  ((default_float_format
     type: cl_float_format_t
     cpp-name: "default_float_format")
   (default_lfloat_format
     type: cl_float_format_t
     cpp-name: "default_lfloat_format")
   (operator= (cl_read_float_flags&)
              cl_read_float_flags&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_read_float_flags&)
  constructor: ()
  type: cl_read_float_flags*
  class: "cl_read_float_flags")
(define-cpp-converter cl_read_syntax_t enum)
(defenum cl_read_syntax_t
  (syntax_integer
    syntax_ratio
    syntax_rational
    syntax_sfloat
    syntax_ffloat
    syntax_dfloat
    syntax_lfloat
    syntax_float
    syntax_real
    syntax_complex
    syntax_number
    syntax_maybe_bad)
  cl_read_syntax_t)
(define-cpp-converter cl_read_lsyntax_t enum)
(defenum cl_read_lsyntax_t
  (lsyntax_standard lsyntax_algebraic lsyntax_commonlisp lsyntax_all)
  cl_read_lsyntax_t)
(define-cpp-converter cl_read_flags* static)
;; Definition of class cl_read_flags
;; At /home/bruno/cln/include/cl_input.h:46
(define-cpp-class cl_read_flags (<class-proxy>)
  ((syntax type: cl_read_syntax_t cpp-name: "syntax")
   (lsyntax type: cl_read_lsyntax_t cpp-name: "lsyntax")
   (rational_base type: unsigned cpp-name: "rational_base")
   (float_flags type: cl_read_float_flags cpp-name: "float_flags")
   (operator= (cl_read_flags&) cl_read_flags& "operator =" implicitp: t))
  copy-constructor: (cl_read_flags&)
  constructor: ()
  type: cl_read_flags*
  class: "cl_read_flags")
(define-cpp-converter char** other-proxy)
(defoverloaded read_number (const-cl_read_flags&
                             const-char*
                             const-char*
                             const-char**)
  cl_number
  "read_number")
(defoverloaded read_number (istream& const-cl_read_flags&)
  cl_number
  "read_number")
(define-cpp-converter cl_string* static)
(define-cpp-converter cl_heap_string* static)
;; Definition of class cl_string
;; At /home/bruno/cln/include/cl_string.h:33
(define-cpp-class cl_string (cl_gcpointer)
  ((asciz () const-char* "asciz")
   (length () unsigned-long "length")
   (operator[] (unsigned-long) char "operator []")
   (operator= (const-cl_string&) cl_string& "operator =")
   (operator= (const-char*) cl_string& "operator ="))
  constructor: ()
  copy-constructor: (const-cl_string&)
  constructor: (const-char*)
  constructor: (const-char* unsigned-long)
  constructor: (cl_heap_string*)
  type: cl_string*
  class: "cl_string")
(defoverloaded strlen (const-cl_string&) unsigned-long "strlen")
(defoverloaded asciz (const-char*) const-char* "asciz")
(defoverloaded asciz (const-cl_string&) const-char* "asciz")
(defoverloaded hashcode (const-cl_string&) unsigned-long "hashcode")
(defoverloaded fprint (ostream& const-cl_string&) void "fprint")
(defoverloaded operator<< (ostream& const-cl_string&) ostream& "operator <<")
(defoverloaded cl_fget (istream& optional: char) cl_string "cl_fget")
(defoverloaded cl_fgetline (istream& optional: char) cl_string "cl_fgetline")
(defoverloaded cl_fget (istream& int optional: char) cl_string "cl_fget")
(defoverloaded cl_fgetline (istream& int optional: char)
  cl_string
  "cl_fgetline")
(defoverloaded operator>> (istream& cl_string&) istream& "operator >>")
(define-cpp-converter cl_print_rational_flags* static)
;; Definition of class cl_print_rational_flags
;; At /home/bruno/cln/include/cl_output.h:11
(define-cpp-class cl_print_rational_flags (<class-proxy>)
  ((rational_base type: unsigned cpp-name: "rational_base")
   (rational_readably type: cl_boolean cpp-name: "rational_readably")
   (operator= (cl_print_rational_flags&)
              cl_print_rational_flags&
              "operator ="
              implicitp: t))
  constructor: ()
  copy-constructor: (cl_print_rational_flags&)
  type: cl_print_rational_flags*
  class: "cl_print_rational_flags")
(define-cpp-converter cl_print_float_flags* static)
;; Definition of class cl_print_float_flags
;; At /home/bruno/cln/include/cl_output.h:23
(define-cpp-class cl_print_float_flags (<class-proxy>)
  ((float_readably type: cl_boolean cpp-name: "float_readably")
   (default_float_format
     type: cl_float_format_t
     cpp-name: "default_float_format")
   (operator= (cl_print_float_flags&)
              cl_print_float_flags&
              "operator ="
              implicitp: t))
  constructor: ()
  copy-constructor: (cl_print_float_flags&)
  type: cl_print_float_flags*
  class: "cl_print_float_flags")
(define-cpp-converter cl_print_real_flags* static)
;; Definition of class cl_print_real_flags
;; At /home/bruno/cln/include/cl_output.h:34
(define-cpp-class cl_print_real_flags (cl_print_rational_flags
                                        cl_print_float_flags)
  ((float_readably type: cl_boolean cpp-name: "float_readably")
   (rational_base type: unsigned cpp-name: "rational_base")
   (default_float_format
     type: cl_float_format_t
     cpp-name: "default_float_format")
   (rational_readably type: cl_boolean cpp-name: "rational_readably")
   (operator= (cl_print_real_flags&)
              cl_print_real_flags&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_print_real_flags&)
  constructor: ()
  type: cl_print_real_flags*
  class: "cl_print_real_flags"
  supertypes: (cl_print_rational_flags* cl_print_float_flags*))
(define-cpp-converter cl_print_complex_flags* static)
;; Definition of class cl_print_complex_flags
;; At /home/bruno/cln/include/cl_output.h:36
(define-cpp-class cl_print_complex_flags (<class-proxy>)
  ((complex_readably type: cl_boolean cpp-name: "complex_readably")
   (operator= (cl_print_complex_flags&)
              cl_print_complex_flags&
              "operator ="
              implicitp: t))
  constructor: ()
  copy-constructor: (cl_print_complex_flags&)
  type: cl_print_complex_flags*
  class: "cl_print_complex_flags")
(define-cpp-converter cl_print_number_flags* static)
;; Definition of class cl_print_number_flags
;; At /home/bruno/cln/include/cl_output.h:44
(define-cpp-class cl_print_number_flags (cl_print_real_flags
                                          cl_print_complex_flags)
  ((complex_readably type: cl_boolean cpp-name: "complex_readably")
   (float_readably type: cl_boolean cpp-name: "float_readably")
   (rational_base type: unsigned cpp-name: "rational_base")
   (default_float_format
     type: cl_float_format_t
     cpp-name: "default_float_format")
   (rational_readably type: cl_boolean cpp-name: "rational_readably")
   (operator= (cl_print_number_flags&)
              cl_print_number_flags&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_print_number_flags&)
  constructor: ()
  type: cl_print_number_flags*
  class: "cl_print_number_flags"
  supertypes: (cl_print_real_flags* cl_print_complex_flags*))
(define-cpp-converter cl_print_vector_syntax_t enum)
(defenum cl_print_vector_syntax_t
  (vsyntax_algebraic vsyntax_pretty vsyntax_commonlisp)
  cl_print_vector_syntax_t)
(define-cpp-converter cl_print_vector_flags* static)
;; Definition of class cl_print_vector_flags
;; At /home/bruno/cln/include/cl_output.h:52
(define-cpp-class cl_print_vector_flags (<class-proxy>)
  ((vector_syntax type: cl_print_vector_syntax_t cpp-name: "vector_syntax")
   (operator= (cl_print_vector_flags&)
              cl_print_vector_flags&
              "operator ="
              implicitp: t))
  constructor: ()
  copy-constructor: (cl_print_vector_flags&)
  type: cl_print_vector_flags*
  class: "cl_print_vector_flags")
