(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded fceiling (const-cl_F&) cl_F "fceiling")
(defoverloaded ftruncate (const-cl_F&) cl_F "ftruncate")
(defoverloaded fround (const-cl_F&) cl_F "fround")
(define-cpp-converter cl_F_fdiv_t* static)
;; Definition of class cl_F_fdiv_t
;; At /home/bruno/cln/include/cl_float.h:89
(define-cpp-class cl_F_fdiv_t (<class-proxy>)
  ((quotient type: cl_F cpp-name: "quotient")
   (remainder type: cl_F cpp-name: "remainder")
   (operator= (cl_F_fdiv_t&) cl_F_fdiv_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_F& const-cl_F&)
  copy-constructor: (cl_F_fdiv_t&)
  type: cl_F_fdiv_t*
  class: "cl_F_fdiv_t")
(defoverloaded ffloor2 (const-cl_F&) cl_F_fdiv_t "ffloor2")
(defoverloaded fceiling2 (const-cl_F&) cl_F_fdiv_t "fceiling2")
(defoverloaded ftruncate2 (const-cl_F&) cl_F_fdiv_t "ftruncate2")
(defoverloaded fround2 (const-cl_F&) cl_F_fdiv_t "fround2")
(define-cpp-converter cl_F_div_t* static)
;; Definition of class cl_F_div_t
;; At /home/bruno/cln/include/cl_float.h:112
(define-cpp-class cl_F_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_F cpp-name: "remainder")
   (operator= (cl_F_div_t&) cl_F_div_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_F&)
  copy-constructor: (cl_F_div_t&)
  type: cl_F_div_t*
  class: "cl_F_div_t")
(defoverloaded floor2 (const-cl_F&) cl_F_div_t "floor2")
(defoverloaded floor1 (const-cl_F&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_F&) cl_F_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_F&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_F&) cl_F_div_t "truncate2")
(defoverloaded truncate1 (const-cl_F&) cl_I "truncate1")
(defoverloaded round2 (const-cl_F&) cl_F_div_t "round2")
(defoverloaded round1 (const-cl_F&) cl_I "round1")
(defoverloaded floor2 (const-cl_F& const-cl_F&) cl_F_div_t "floor2")
(defoverloaded floor1 (const-cl_F& const-cl_F&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_F& const-cl_F&) cl_F_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_F& const-cl_F&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_F& const-cl_F&) cl_F_div_t "truncate2")
(defoverloaded truncate1 (const-cl_F& const-cl_F&) cl_I "truncate1")
(defoverloaded round2 (const-cl_F& const-cl_F&) cl_F_div_t "round2")
(defoverloaded round1 (const-cl_F& const-cl_F&) cl_I "round1")
(define-cpp-variable
  cl_default_float_format
  cl_float_format_t
  "cl_default_float_format")
(defoverloaded cl_float_format (uintL) cl_float_format_t "cl_float_format")
(defoverloaded cl_float (const-cl_F& const-cl_F&) cl_F "cl_float")
(defoverloaded cl_float (const-cl_F& cl_float_format_t) cl_F "cl_float")
(defoverloaded cl_float (const-cl_I& const-cl_F&) cl_F "cl_float")
(defoverloaded cl_float (const-cl_I& cl_float_format_t) cl_F "cl_float")
(defoverloaded cl_float (const-cl_RA& const-cl_F&) cl_F "cl_float")
(defoverloaded cl_float (const-cl_RA& cl_float_format_t) cl_F "cl_float")
(defoverloaded cl_float (int const-cl_F&) cl_F "cl_float")
(defoverloaded cl_float (int cl_float_format_t) cl_F "cl_float")
(defoverloaded cl_float (double const-cl_F&) cl_F "cl_float")
(defoverloaded cl_float (double cl_float_format_t) cl_F "cl_float")
(define-cpp-converter cl_decoded_float* static)
;; Definition of class cl_decoded_float
;; At /home/bruno/cln/include/cl_float.h:222
(define-cpp-class cl_decoded_float (<class-proxy>)
  ((mantissa type: cl_F cpp-name: "mantissa")
   (exponent type: cl_I cpp-name: "exponent")
   (sign type: cl_F cpp-name: "sign")
   (operator= (cl_decoded_float&) cl_decoded_float& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_F& const-cl_I& const-cl_F&)
  copy-constructor: (cl_decoded_float&)
  type: cl_decoded_float*
  class: "cl_decoded_float")
(defoverloaded decode_float (const-cl_F&) cl_decoded_float "decode_float")
(defoverloaded float_exponent (const-cl_F&) sintL "float_exponent")
(defoverloaded float_radix (const-cl_F&) sintL "float_radix")
(defoverloaded float_sign (const-cl_F&) cl_F "float_sign")
(defoverloaded float_sign (const-cl_F& const-cl_F&) cl_F "float_sign")
(defoverloaded float_digits (const-cl_F&) uintL "float_digits")
(defoverloaded float_precision (const-cl_F&) uintL "float_precision")
(defoverloaded cl_float_format (const-cl_F&)
  cl_float_format_t
  "cl_float_format")
(defoverloaded integer_decode_float (const-cl_F&)
  cl_idecoded_float
  "integer_decode_float")
