(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded operator- (const-cl_UP_I& const-cl_UP_I&) cl_UP_I "operator -")
(defoverloaded operator* (const-cl_UP_I& const-cl_UP_I&) cl_UP_I "operator *")
(defoverloaded square (const-cl_UP_I&) cl_UP_I "square")
(defoverloaded expt_pos (const-cl_UP_I& const-cl_I&) cl_UP_I "expt_pos")
(defoverloaded operator* (const-cl_I& const-cl_UP_I&) cl_UP_I "operator *")
(defoverloaded operator* (const-cl_UP_I& const-cl_I&) cl_UP_I "operator *")
(defoverloaded coeff (const-cl_UP_I& uintL) cl_I "coeff")
(defoverloaded set_coeff (cl_UP_I& uintL const-cl_I&) void "set_coeff")
(defoverloaded finalize (cl_UP_I&) void "finalize")
(defoverloaded deriv (const-cl_UP_I&) cl_UP_I "deriv")
(defoverloaded cl_tschebychev (sintL) cl_UP_I "cl_tschebychev")
(defoverloaded cl_hermite (sintL) cl_UP_I "cl_hermite")
(defoverloaded cl_laguerre (sintL) cl_UP_I "cl_laguerre")
(define-cpp-converter cl_univpoly_rational_ring* static)
(define-cpp-converter cl_heap_univpoly_rational_ring* static)
;; Definition of class cl_univpoly_rational_ring
;; At /home/bruno/cln/include/cl_univpoly_rational.h:23
(define-cpp-class cl_univpoly_rational_ring (cl_univpoly_ring)
  ((operator= (const-cl_univpoly_rational_ring&)
              cl_univpoly_rational_ring&
              "operator =")
   (operator-> () cl_heap_univpoly_rational_ring* "operator ->"))
  constructor: ()
  copy-constructor: (const-cl_univpoly_rational_ring&)
  type: cl_univpoly_rational_ring*
  class: "cl_univpoly_rational_ring")
(define-cpp-converter cl_UP_RA* static)
;; Definition of class cl_UP_RA
;; At /home/bruno/cln/include/cl_univpoly_rational.h:39
(define-cpp-class cl_UP_RA (cl_UP)
  ((ring () const-cl_univpoly_rational_ring& "ring")
   (set_coeff (uintL const-cl_RA&) void "set_coeff")
   (finalize () void "finalize")
   (|operator()| (const-cl_RA&) cl_RA "operator ()")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_UP_RA*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  copy-constructor: (cl_UP_RA&)
  type: cl_UP_RA*
  class: "cl_UP_RA")
;; Definition of class cl_heap_univpoly_rational_ring
;; At /home/bruno/cln/include/cl_univpoly_rational.h:55
(define-cpp-class cl_heap_univpoly_rational_ring (cl_heap_univpoly_ring)
  ((operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (fprint (ostream& const-cl_UP_RA&) void "fprint")
   (equal (const-cl_UP_RA& const-cl_UP_RA&) cl_boolean "equal")
   (zero () cl_UP_RA "zero")
   (zerop (const-cl_UP_RA&) cl_boolean "zerop")
   (plus (const-cl_UP_RA& const-cl_UP_RA&) cl_UP_RA "plus")
   (minus (const-cl_UP_RA& const-cl_UP_RA&) cl_UP_RA "minus")
   (uminus (const-cl_UP_RA&) cl_UP_RA "uminus")
   (one () cl_UP_RA "one")
   (canonhom (const-cl_I&) cl_UP_RA "canonhom")
   (mul (const-cl_UP_RA& const-cl_UP_RA&) cl_UP_RA "mul")
   (square (const-cl_UP_RA&) cl_UP_RA "square")
   (expt_pos (const-cl_UP_RA& const-cl_I&) cl_UP_RA "expt_pos")
   (degree (const-cl_UP_RA&) sintL "degree")
   (monomial (const-cl_RA& uintL) cl_UP_RA "monomial")
   (coeff (const-cl_UP_RA& uintL) cl_RA "coeff")
   (create (sintL) cl_UP_RA "create")
   (set_coeff (cl_UP_RA& uintL const-cl_RA&) void "set_coeff")
   (finalize (cl_UP_RA&) void "finalize")
   (eval (const-cl_UP_RA& const-cl_RA&) cl_RA "eval")
   (operator= (cl_heap_univpoly_rational_ring&)
              cl_heap_univpoly_rational_ring&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_heap_univpoly_rational_ring&)
  type: cl_heap_univpoly_rational_ring*
  class: "cl_heap_univpoly_rational_ring")
(defoverloaded operator+ (const-cl_UP_RA& const-cl_UP_RA&)
  cl_UP_RA
  "operator +")
(defoverloaded operator- (const-cl_UP_RA&) cl_UP_RA "operator -")
(defoverloaded operator- (const-cl_UP_RA& const-cl_UP_RA&)
  cl_UP_RA
  "operator -")
(defoverloaded operator* (const-cl_UP_RA& const-cl_UP_RA&)
  cl_UP_RA
  "operator *")
(defoverloaded square (const-cl_UP_RA&) cl_UP_RA "square")
(defoverloaded expt_pos (const-cl_UP_RA& const-cl_I&) cl_UP_RA "expt_pos")
(defoverloaded operator* (const-cl_I& const-cl_UP_RA&) cl_UP_RA "operator *")
(defoverloaded operator* (const-cl_UP_RA& const-cl_I&) cl_UP_RA "operator *")
(defoverloaded coeff (const-cl_UP_RA& uintL) cl_RA "coeff")
(defoverloaded set_coeff (cl_UP_RA& uintL const-cl_RA&) void "set_coeff")
(defoverloaded finalize (cl_UP_RA&) void "finalize")
(defoverloaded deriv (const-cl_UP_RA&) cl_UP_RA "deriv")
(defoverloaded cl_legendre (sintL) cl_UP_RA "cl_legendre")
(define-cpp-converter cl_univpoly_real_ring* static)
(define-cpp-converter cl_heap_univpoly_real_ring* static)
;; Definition of class cl_univpoly_real_ring
;; At /home/bruno/cln/include/cl_univpoly_real.h:23
(define-cpp-class cl_univpoly_real_ring (cl_univpoly_ring)
  ((operator= (const-cl_univpoly_real_ring&)
              cl_univpoly_real_ring&
              "operator =")
   (operator-> () cl_heap_univpoly_real_ring* "operator ->"))
  constructor: ()
  copy-constructor: (const-cl_univpoly_real_ring&)
  type: cl_univpoly_real_ring*
  class: "cl_univpoly_real_ring")
(define-cpp-converter cl_UP_R* static)
;; Definition of class cl_UP_R
;; At /home/bruno/cln/include/cl_univpoly_real.h:39
(define-cpp-class cl_UP_R (cl_UP)
  ((ring () const-cl_univpoly_real_ring& "ring")
   (set_coeff (uintL const-cl_R&) void "set_coeff")
   (finalize () void "finalize")
   (|operator()| (const-cl_R&) cl_R "operator ()")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_UP_R*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  copy-constructor: (cl_UP_R&)
  type: cl_UP_R*
  class: "cl_UP_R")
;; Definition of class cl_heap_univpoly_real_ring
;; At /home/bruno/cln/include/cl_univpoly_real.h:55
(define-cpp-class cl_heap_univpoly_real_ring (cl_heap_univpoly_ring)
  ((operator-new (size_t) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t)
   (fprint (ostream& const-cl_UP_R&) void "fprint")
   (equal (const-cl_UP_R& const-cl_UP_R&) cl_boolean "equal")
   (zero () cl_UP_R "zero")
   (zerop (const-cl_UP_R&) cl_boolean "zerop")
   (plus (const-cl_UP_R& const-cl_UP_R&) cl_UP_R "plus")
   (minus (const-cl_UP_R& const-cl_UP_R&) cl_UP_R "minus")
   (uminus (const-cl_UP_R&) cl_UP_R "uminus")
   (one () cl_UP_R "one")
   (canonhom (const-cl_I&) cl_UP_R "canonhom")
   (mul (const-cl_UP_R& const-cl_UP_R&) cl_UP_R "mul")
   (square (const-cl_UP_R&) cl_UP_R "square")
   (expt_pos (const-cl_UP_R& const-cl_I&) cl_UP_R "expt_pos")
   (degree (const-cl_UP_R&) sintL "degree")
   (monomial (const-cl_R& uintL) cl_UP_R "monomial")
   (coeff (const-cl_UP_R& uintL) cl_R "coeff")
   (create (sintL) cl_UP_R "create")
   (set_coeff (cl_UP_R& uintL const-cl_R&) void "set_coeff")
   (finalize (cl_UP_R&) void "finalize")
   (eval (const-cl_UP_R& const-cl_R&) cl_R "eval")
   (operator= (cl_heap_univpoly_real_ring&)
              cl_heap_univpoly_real_ring&
              "operator ="
              implicitp: t))
  copy-constructor: (cl_heap_univpoly_real_ring&)
  type: cl_heap_univpoly_real_ring*
  class: "cl_heap_univpoly_real_ring")
(defoverloaded operator+ (const-cl_UP_R& const-cl_UP_R&) cl_UP_R "operator +")
(defoverloaded operator- (const-cl_UP_R&) cl_UP_R "operator -")
(defoverloaded operator- (const-cl_UP_R& const-cl_UP_R&) cl_UP_R "operator -")
(defoverloaded operator* (const-cl_UP_R& const-cl_UP_R&) cl_UP_R "operator *")
(defoverloaded square (const-cl_UP_R&) cl_UP_R "square")
(defoverloaded expt_pos (const-cl_UP_R& const-cl_I&) cl_UP_R "expt_pos")
(defoverloaded operator* (const-cl_I& const-cl_UP_R&) cl_UP_R "operator *")
(defoverloaded operator* (const-cl_UP_R& const-cl_I&) cl_UP_R "operator *")
(defoverloaded coeff (const-cl_UP_R& uintL) cl_R "coeff")
(defoverloaded set_coeff (cl_UP_R& uintL const-cl_R&) void "set_coeff")
(defoverloaded finalize (cl_UP_R&) void "finalize")
(defoverloaded deriv (const-cl_UP_R&) cl_UP_R "deriv")
