/* include/cl_config.h.  Generated automatically by configure.  */
// Defines CPU and compiler dependent macros

#ifndef _CL_CONFIG_H
#define _CL_CONFIG_H

/* These definitions are adjusted by `configure' automatically. */

/* CPU */
#ifndef __i386__
#undef __i386__
#endif
#ifndef __m68k__
#undef __m68k__
#endif
#ifndef __mips__
#define __mips__ 1
#endif
#ifndef __sparc__
#undef __sparc__
#endif
#ifndef __alpha__
#undef __alpha__
#endif
#ifndef __hppa__
#undef __hppa__
#endif
#ifndef __arm__
#undef __arm__
#endif
#ifndef __rs6000__
#undef __rs6000__
#endif
#ifndef __m88k__
#undef __m88k__
#endif
#ifndef __convex__
#undef __convex__
#endif


/* compiler characteristics */

/* CL_BOOL */
/* Define if your compiler understands the bool type. */
#undef HAVE_BOOL
/* Define this as `int' if your compiler doesn't understand the bool type. */
#define bool int

/* CL_LONGLONG */
/* Define if your compiler supports the `long long' type. */
#define HAVE_LONGLONG 1

/* CL_LONGDOUBLE */
/* Define if your compiler supports the `long double' type. */
#define HAVE_LONGDOUBLE 1


/* header files */

/* CL_STDLIB_H */
/* Define if you have <stdlib.h>. */
#define HAVE_STDLIB_H 1

/* CL_UNISTD_H */
/* Define if you have <unistd.h>. */
#define HAVE_UNISTD_H 1


/* typedefs */

/* CL_CLOCK_T */
#define CLOCK_T clock_t


/* functions and declarations */

/* CL_ALLOCA */
/* Define if you have <alloca.h> and it should be used (not Ultrix). */
#define HAVE_ALLOCA_H 1
/* Define if you need to link with an external alloca.o when using alloca(). */
#undef NO_ALLOCA

/* CL_GETTIMEOFDAY */
/* Define if you have the gettimeofday() function. */
#define HAVE_GETTIMEOFDAY 1
/* Define if the declaration of gettimeofday() needs dots. */
#define GETTIMEOFDAY_DOTS 1
/* Define as the type of `tzp' in gettimeofday() declaration. */
#undef GETTIMEOFDAY_TZP_T

/* CL_FTIME */
/* Define if you have the ftime() function. */
#undef HAVE_FTIME

/* CL_TIMES_CLOCK */
/* Define if you have the times() function and it returns the real time,
   but don't have the gettimeofday() or ftime() function. */
#undef HAVE_TIMES_CLOCK

/* CL_RUSAGE */
/* Define if you have <sys/resource.h>. */
#define HAVE_SYS_RESOURCE_H 1
/* Define if you also have <sys/time.h>, the getrusage() function,
   the struct rusage type, and <sys/resource.h> defines RUSAGE_SELF. */
#define HAVE_GETRUSAGE 1
/* Define if you have <sys/times.h>. */
#define HAVE_SYS_TIMES_H 1


/* compiler characteristics */

/* CL_CHAR_UNSIGNED */
#ifndef __CHAR_UNSIGNED__
#define __CHAR_UNSIGNED__ 1
#endif

/* CL_MACHINE */
/* see cl_intparam.h */
/* see cl_floatparam.h */


/* interfacing to GNU gmp */

/* Define if you wish to use the fast GNU gmp low-level functions. */
#define CL_USE_GMP 1


#endif /* _CL_CONFIG_H */

