(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
;; Definition of class cl_DF
;; At /home/bruno/cln/include/cl_number.h:302
(define-cpp-class cl_DF (cl_F)
  ((operator= (const-cl_DF&) cl_DF& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (operator= (double) cl_DF& "operator =")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_DF*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_DF&)
  constructor: (double)
  constructor: (const-char*)
  type: cl_DF*
  class: "cl_DF")
;; Definition of class cl_LF
;; At /home/bruno/cln/include/cl_number.h:332
(define-cpp-class cl_LF (cl_F)
  ((operator= (const-cl_LF&) cl_LF& "operator =")
   (pointer_p () cl_boolean "pointer_p")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_LF*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_LF&)
  constructor: (const-char*)
  type: cl_LF*
  class: "cl_LF")
;; Definition of class cl_RA
;; At /home/bruno/cln/include/cl_number.h:366
(define-cpp-class cl_RA (cl_R)
  ((operator= (const-cl_RA&) cl_RA& "operator =")
   (operator= (const-cl_I&) cl_RA& "operator =")
   (operator= (long) cl_RA& "operator =")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_RA*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_RA&)
  constructor: (long)
  constructor: (const-char*)
  type: cl_RA*
  class: "cl_RA")
;; Definition of class cl_I
;; At /home/bruno/cln/include/cl_number.h:399
(define-cpp-class cl_I (cl_RA)
  ((operator= (const-cl_I&) cl_I& "operator =")
   (operator= (long) cl_I& "operator =")
   (operator-new (size_t) void* "operator new" staticp: t)
   (operator-new (size_t cl_I*) void* "operator new" staticp: t)
   (operator-delete (void*) void "operator delete" staticp: t))
  constructor: ()
  copy-constructor: (const-cl_I&)
  constructor: (long)
  constructor: (const-char*)
  type: cl_I*
  class: "cl_I")
(define-cpp-variable cl_FF_0 cl_FF "cl_FF_0")
(define-cpp-variable cl_DF_0 cl_DF "cl_DF_0")
(define-cpp-variable cl_LF_0 cl_LF "cl_LF_0")
(defoverloaded as_cl_N (const-cl_number& const-char* int)
  const-cl_N&
  "as_cl_N")
(defoverloaded as_cl_R (const-cl_number& const-char* int)
  const-cl_R&
  "as_cl_R")
(defoverloaded as_cl_F (const-cl_number& const-char* int)
  const-cl_F&
  "as_cl_F")
(defoverloaded as_cl_SF (const-cl_number& const-char* int)
  const-cl_SF&
  "as_cl_SF")
(defoverloaded as_cl_FF (const-cl_number& const-char* int)
  const-cl_FF&
  "as_cl_FF")
(defoverloaded as_cl_DF (const-cl_number& const-char* int)
  const-cl_DF&
  "as_cl_DF")
(defoverloaded as_cl_LF (const-cl_number& const-char* int)
  const-cl_LF&
  "as_cl_LF")
(defoverloaded as_cl_RA (const-cl_number& const-char* int)
  const-cl_RA&
  "as_cl_RA")
(defoverloaded as_cl_I (const-cl_number& const-char* int)
  const-cl_I&
  "as_cl_I")
(defoverloaded as_cl_N (const-cl_N& const-char* int) const-cl_N& "as_cl_N")
(defoverloaded as_cl_R (const-cl_R& const-char* int) const-cl_R& "as_cl_R")
(defoverloaded as_cl_F (const-cl_F& const-char* int) const-cl_F& "as_cl_F")
(defoverloaded as_cl_SF (const-cl_SF& const-char* int)
  const-cl_SF&
  "as_cl_SF")
(defoverloaded as_cl_FF (const-cl_FF& const-char* int)
  const-cl_FF&
  "as_cl_FF")
(defoverloaded as_cl_DF (const-cl_DF& const-char* int)
  const-cl_DF&
  "as_cl_DF")
(defoverloaded as_cl_LF (const-cl_LF& const-char* int)
  const-cl_LF&
  "as_cl_LF")
(defoverloaded as_cl_RA (const-cl_RA& const-char* int)
  const-cl_RA&
  "as_cl_RA")
(defoverloaded as_cl_I (const-cl_I& const-char* int) const-cl_I& "as_cl_I")
(define-cpp-converter cl_idecoded_float* static)
;; Definition of class cl_idecoded_float
;; At /home/bruno/cln/include/cl_number.h:795
(define-cpp-class cl_idecoded_float (<class-proxy>)
  ((mantissa type: cl_I cpp-name: "mantissa")
   (exponent type: cl_I cpp-name: "exponent")
   (sign type: cl_I cpp-name: "sign")
   (operator= (cl_idecoded_float&)
              cl_idecoded_float&
              "operator ="
              implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_I& const-cl_I&)
  copy-constructor: (cl_idecoded_float&)
  type: cl_idecoded_float*
  class: "cl_idecoded_float")
(defoverloaded cl_identity (const-cl_number&) const-cl_number& "cl_identity")
(define-cpp-converter cl_float_format_t enum)
(defenum cl_float_format_t
  (cl_float_format_sfloat
    cl_float_format_ffloat
    cl_float_format_dfloat
    cl_float_format_lfloat_min)
  cl_float_format_t)
(defoverloaded operator- (const-cl_F&) cl_F "operator -")
(defoverloaded operator+ (const-cl_F& const-cl_F&) cl_F "operator +")
(defoverloaded operator- (const-cl_F& const-cl_F&) cl_F "operator -")
(defoverloaded operator* (const-cl_F& const-cl_F&) cl_F "operator *")
(defoverloaded square (const-cl_F&) cl_F "square")
(defoverloaded operator/ (const-cl_F& const-cl_F&) cl_F "operator /")
(defoverloaded sqrt (const-cl_F&) cl_F "sqrt")
(defoverloaded recip (const-cl_F&) cl_F "recip")
(defoverloaded plus1 (const-cl_F&) cl_F "plus1")
(defoverloaded minus1 (const-cl_F&) cl_F "minus1")
(defoverloaded zerop (const-cl_F&) cl_boolean "zerop")
(defoverloaded minusp (const-cl_F&) cl_boolean "minusp")
(defoverloaded cl_compare (const-cl_F& const-cl_F&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_F& const-cl_F&) bool "operator ==")
(defoverloaded operator!= (const-cl_F& const-cl_F&) bool "operator !=")
(defoverloaded operator<= (const-cl_F& const-cl_F&) bool "operator <=")
(defoverloaded operator< (const-cl_F& const-cl_F&) bool "operator <")
(defoverloaded operator>= (const-cl_F& const-cl_F&) bool "operator >=")
(defoverloaded operator> (const-cl_F& const-cl_F&) bool "operator >")
(defoverloaded ffloor (const-cl_F&) cl_F "ffloor")
