(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded operator* (const-cl_RA& long) cl_RA "operator *")
(defoverloaded square (const-cl_RA&) cl_RA "square")
(defoverloaded operator/ (const-cl_RA& const-cl_RA&) cl_RA "operator /")
(defoverloaded operator/ (long const-cl_RA&) cl_RA "operator /")
(defoverloaded operator/ (const-cl_RA& long) cl_RA "operator /")
(define-cpp-converter cl_RA_div_t* static)
;; Definition of class cl_RA_div_t
;; At /home/bruno/cln/include/cl_rational.h:132
(define-cpp-class cl_RA_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_RA cpp-name: "remainder")
   (operator= (cl_RA_div_t&) cl_RA_div_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_RA&)
  copy-constructor: (cl_RA_div_t&)
  type: cl_RA_div_t*
  class: "cl_RA_div_t")
(defoverloaded floor2 (const-cl_RA&) cl_RA_div_t "floor2")
(defoverloaded floor1 (const-cl_RA&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_RA&) cl_RA_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_RA&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_RA&) cl_RA_div_t "truncate2")
(defoverloaded truncate1 (const-cl_RA&) cl_I "truncate1")
(defoverloaded round2 (const-cl_RA&) cl_RA_div_t "round2")
(defoverloaded round1 (const-cl_RA&) cl_I "round1")
(defoverloaded floor2 (const-cl_RA& const-cl_RA&) cl_RA_div_t "floor2")
(defoverloaded floor1 (const-cl_RA& const-cl_RA&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_RA& const-cl_RA&) cl_RA_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_RA& const-cl_RA&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_RA& const-cl_RA&) cl_RA_div_t "truncate2")
(defoverloaded truncate1 (const-cl_RA& const-cl_RA&) cl_I "truncate1")
(defoverloaded round2 (const-cl_RA& const-cl_RA&) cl_RA_div_t "round2")
(defoverloaded round1 (const-cl_RA& const-cl_RA&) cl_I "round1")
(defoverloaded signum (const-cl_RA&) cl_RA "signum")
(defoverloaded expt_pos (const-cl_RA& uintL) cl_RA "expt_pos")
(defoverloaded expt_pos (const-cl_RA& const-cl_I&) cl_RA "expt_pos")
(defoverloaded expt (const-cl_RA& sintL) cl_RA "expt")
(defoverloaded expt (const-cl_RA& const-cl_I&) cl_RA "expt")
(defoverloaded sqrtp (const-cl_RA& cl_RA*) cl_boolean "sqrtp")
(defoverloaded sqrt (const-cl_RA&) cl_R "sqrt")
(defoverloaded rootp (const-cl_RA& uintL cl_RA*) cl_boolean "rootp")
(defoverloaded rootp (const-cl_RA& const-cl_I& cl_RA*) cl_boolean "rootp")
(defoverloaded logp (const-cl_I& const-cl_I& cl_RA*) cl_boolean "logp")
(defoverloaded logp (const-cl_RA& const-cl_RA& cl_RA*) cl_boolean "logp")
(defoverloaded cl_float_approx (const-cl_RA&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_RA&) double "cl_double_approx")
(defoverloaded cl_I_to_UL (const-cl_I&) uint32 "cl_I_to_UL")
(defoverloaded cl_I_to_L (const-cl_I&) sint32 "cl_I_to_L")
(defoverloaded cl_I_to_int (const-cl_I&) int "cl_I_to_int")
(defoverloaded cl_I_to_uint (const-cl_I&) unsigned "cl_I_to_uint")
(defoverloaded cl_I_to_long (const-cl_I&) long "cl_I_to_long")
(defoverloaded cl_I_to_ulong (const-cl_I&) unsigned-long "cl_I_to_ulong")
(defoverloaded logior (const-cl_I& const-cl_I&) cl_I "logior")
(defoverloaded logxor (const-cl_I& const-cl_I&) cl_I "logxor")
(defoverloaded logand (const-cl_I& const-cl_I&) cl_I "logand")
(defoverloaded logeqv (const-cl_I& const-cl_I&) cl_I "logeqv")
(defoverloaded lognand (const-cl_I& const-cl_I&) cl_I "lognand")
(defoverloaded lognor (const-cl_I& const-cl_I&) cl_I "lognor")
(defoverloaded logandc2 (const-cl_I& const-cl_I&) cl_I "logandc2")
(defoverloaded logandc1 (const-cl_I& const-cl_I&) cl_I "logandc1")
