(define-cpp-data-slot-setter cl_decoded_ffloat exponent)
(define-cpp-data-slot-setter cl_decoded_ffloat sign)
(define-cpp-type cl_decoded_ffloat& static
  id: cl_decoded_ffloat*
  class: cl_decoded_ffloat
  cpp-type-dcl: "cl_decoded_ffloat &%s"
  cast: "((cl_decoded_ffloat &)(%s))"
  arg-type: "cl_decoded_ffloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_ffloat *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_ffloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_ffloat static
  id: cl_decoded_ffloat*
  class: cl_decoded_ffloat
  cpp-type-dcl: "cl_decoded_ffloat %s"
  cast: "((cl_decoded_ffloat)(%s))"
  arg-type: "cl_decoded_ffloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_ffloat *%s"
  res-converter: "new cl_decoded_ffloat (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_ffloat *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_ffloat* static
  id: cl_decoded_ffloat*
  class: cl_decoded_ffloat
  cpp-type-dcl: "cl_decoded_ffloat *%s"
  cast: "((cl_decoded_ffloat *)(%s))"
  callback-res-converter: "(cl_decoded_ffloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_DF_fdiv_t quotient)
(define-cpp-data-slot-setter cl_DF_fdiv_t remainder)
(define-cpp-type cl_DF_fdiv_t& static
  id: cl_DF_fdiv_t*
  class: cl_DF_fdiv_t
  cpp-type-dcl: "cl_DF_fdiv_t &%s"
  cast: "((cl_DF_fdiv_t &)(%s))"
  arg-type: "cl_DF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_DF_fdiv_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_DF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_DF_fdiv_t static
  id: cl_DF_fdiv_t*
  class: cl_DF_fdiv_t
  cpp-type-dcl: "cl_DF_fdiv_t %s"
  cast: "((cl_DF_fdiv_t)(%s))"
  arg-type: "cl_DF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_DF_fdiv_t *%s"
  res-converter: "new cl_DF_fdiv_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_DF_fdiv_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_DF_fdiv_t* static
  id: cl_DF_fdiv_t*
  class: cl_DF_fdiv_t
  cpp-type-dcl: "cl_DF_fdiv_t *%s"
  cast: "((cl_DF_fdiv_t *)(%s))"
  callback-res-converter: "(cl_DF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_DF_div_t quotient)
(define-cpp-data-slot-setter cl_DF_div_t remainder)
(define-cpp-type cl_DF_div_t& static
  id: cl_DF_div_t*
  class: cl_DF_div_t
  cpp-type-dcl: "cl_DF_div_t &%s"
  cast: "((cl_DF_div_t &)(%s))"
  arg-type: "cl_DF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_DF_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_DF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_DF_div_t static
  id: cl_DF_div_t*
  class: cl_DF_div_t
  cpp-type-dcl: "cl_DF_div_t %s"
  cast: "((cl_DF_div_t)(%s))"
  arg-type: "cl_DF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_DF_div_t *%s"
  res-converter: "new cl_DF_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_DF_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_DF_div_t* static
  id: cl_DF_div_t*
  class: cl_DF_div_t
  cpp-type-dcl: "cl_DF_div_t *%s"
  cast: "((cl_DF_div_t *)(%s))"
  callback-res-converter: "(cl_DF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_decoded_dfloat mantissa)
(define-cpp-data-slot-setter cl_decoded_dfloat exponent)
(define-cpp-data-slot-setter cl_decoded_dfloat sign)
(define-cpp-type cl_decoded_dfloat& static
  id: cl_decoded_dfloat*
  class: cl_decoded_dfloat
  cpp-type-dcl: "cl_decoded_dfloat &%s"
  cast: "((cl_decoded_dfloat &)(%s))"
  arg-type: "cl_decoded_dfloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_dfloat *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_dfloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_dfloat static
  id: cl_decoded_dfloat*
  class: cl_decoded_dfloat
  cpp-type-dcl: "cl_decoded_dfloat %s"
  cast: "((cl_decoded_dfloat)(%s))"
  arg-type: "cl_decoded_dfloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_dfloat *%s"
  res-converter: "new cl_decoded_dfloat (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_dfloat *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_dfloat* static
  id: cl_decoded_dfloat*
  class: cl_decoded_dfloat
  cpp-type-dcl: "cl_decoded_dfloat *%s"
  cast: "((cl_decoded_dfloat *)(%s))"
  callback-res-converter: "(cl_decoded_dfloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_LF_fdiv_t quotient)
(define-cpp-data-slot-setter cl_LF_fdiv_t remainder)
(define-cpp-type cl_LF_fdiv_t& static
  id: cl_LF_fdiv_t*
  class: cl_LF_fdiv_t
  cpp-type-dcl: "cl_LF_fdiv_t &%s"
  cast: "((cl_LF_fdiv_t &)(%s))"
  arg-type: "cl_LF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_LF_fdiv_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_LF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_LF_fdiv_t static
  id: cl_LF_fdiv_t*
  class: cl_LF_fdiv_t
  cpp-type-dcl: "cl_LF_fdiv_t %s"
  cast: "((cl_LF_fdiv_t)(%s))"
  arg-type: "cl_LF_fdiv_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_LF_fdiv_t *%s"
  res-converter: "new cl_LF_fdiv_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_LF_fdiv_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_LF_fdiv_t* static
  id: cl_LF_fdiv_t*
  class: cl_LF_fdiv_t
  cpp-type-dcl: "cl_LF_fdiv_t *%s"
  cast: "((cl_LF_fdiv_t *)(%s))"
  callback-res-converter: "(cl_LF_fdiv_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_LF_div_t quotient)
(define-cpp-data-slot-setter cl_LF_div_t remainder)
(define-cpp-type cl_LF_div_t& static
  id: cl_LF_div_t*
  class: cl_LF_div_t
  cpp-type-dcl: "cl_LF_div_t &%s"
  cast: "((cl_LF_div_t &)(%s))"
  arg-type: "cl_LF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_LF_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_LF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_LF_div_t static
  id: cl_LF_div_t*
  class: cl_LF_div_t
  cpp-type-dcl: "cl_LF_div_t %s"
  cast: "((cl_LF_div_t)(%s))"
  arg-type: "cl_LF_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_LF_div_t *%s"
  res-converter: "new cl_LF_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_LF_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_LF_div_t* static
  id: cl_LF_div_t*
  class: cl_LF_div_t
  cpp-type-dcl: "cl_LF_div_t *%s"
  cast: "((cl_LF_div_t *)(%s))"
  callback-res-converter: "(cl_LF_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_decoded_lfloat mantissa)
(define-cpp-data-slot-setter cl_decoded_lfloat exponent)
(define-cpp-data-slot-setter cl_decoded_lfloat sign)
(define-cpp-type cl_decoded_lfloat& static
  id: cl_decoded_lfloat*
  class: cl_decoded_lfloat
  cpp-type-dcl: "cl_decoded_lfloat &%s"
  cast: "((cl_decoded_lfloat &)(%s))"
  arg-type: "cl_decoded_lfloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_lfloat *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_lfloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_lfloat static
  id: cl_decoded_lfloat*
  class: cl_decoded_lfloat
  cpp-type-dcl: "cl_decoded_lfloat %s"
  cast: "((cl_decoded_lfloat)(%s))"
  arg-type: "cl_decoded_lfloat *%s"
  arg-converter: "*(%s)"
  res-type: "cl_decoded_lfloat *%s"
  res-converter: "new cl_decoded_lfloat (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_decoded_lfloat *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_decoded_lfloat* static
  id: cl_decoded_lfloat*
  class: cl_decoded_lfloat
  cpp-type-dcl: "cl_decoded_lfloat *%s"
  cast: "((cl_decoded_lfloat *)(%s))"
  callback-res-converter: "(cl_decoded_lfloat *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-data-slot-setter cl_RA_div_t quotient)
(define-cpp-data-slot-setter cl_RA_div_t remainder)
(define-cpp-type cl_RA_div_t& static
  id: cl_RA_div_t*
  class: cl_RA_div_t
  cpp-type-dcl: "cl_RA_div_t &%s"
  cast: "((cl_RA_div_t &)(%s))"
  arg-type: "cl_RA_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_RA_div_t *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_RA_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_RA_div_t static
  id: cl_RA_div_t*
  class: cl_RA_div_t
  cpp-type-dcl: "cl_RA_div_t %s"
  cast: "((cl_RA_div_t)(%s))"
  arg-type: "cl_RA_div_t *%s"
  arg-converter: "*(%s)"
  res-type: "cl_RA_div_t *%s"
  res-converter: "new cl_RA_div_t (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_RA_div_t *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_RA_div_t* static
  id: cl_RA_div_t*
  class: cl_RA_div_t
  cpp-type-dcl: "cl_RA_div_t *%s"
  cast: "((cl_RA_div_t *)(%s))"
  callback-res-converter: "(cl_RA_div_t *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type uint32 unsigned-long
  cpp-type-dcl: "uint32 %s"
  cpp-arg-type: "((uint32)(%s))"
  cast: "((uint32)(%s))")
(define-cpp-type sint32 long
  cpp-type-dcl: "sint32 %s"
  cpp-arg-type: "((sint32)(%s))"
  cast: "((sint32)(%s))")
(define-cpp-type cl_boole enum cpp-arg-type: "((cl_boole)(%s))" id: cl_boole)
(define-cpp-data-slot-setter cl_byte size)
(define-cpp-data-slot-setter cl_byte position)
(define-cpp-type cl_byte& static
  id: cl_byte*
  class: cl_byte
  cpp-type-dcl: "cl_byte &%s"
  cast: "((cl_byte &)(%s))"
  arg-type: "cl_byte *%s"
  arg-converter: "*(%s)"
  res-type: "cl_byte *%s"
  res-converter: "&%s"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_byte *)(%s)"
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_byte static
  id: cl_byte*
  class: cl_byte
  cpp-type-dcl: "cl_byte %s"
  cast: "((cl_byte)(%s))"
  arg-type: "cl_byte *%s"
  arg-converter: "*(%s)"
  res-type: "cl_byte *%s"
  res-converter: "new cl_byte (%s)"
  callback-arg-converter: "&%s"
  callback-res-converter: "*(cl_byte *)(%s)"
  c-callback-arg-converter: none
  c-callback-res-converter: none
  c-callback-arg-type: "void *%s")
(define-cpp-type cl_byte* static
  id: cl_byte*
  class: cl_byte
  cpp-type-dcl: "cl_byte *%s"
  cast: "((cl_byte *)(%s))"
  callback-res-converter: "(cl_byte *)(%s)"
  c-callback-arg-type: "void *%s")
