(cpp-source "#include \"/home/bruno/cln/include/cln.h\"")
(defoverloaded sqrt (const-cl_FF&) cl_FF "sqrt")
(defoverloaded recip (const-cl_FF&) cl_FF "recip")
(defoverloaded plus1 (const-cl_FF&) cl_FF "plus1")
(defoverloaded minus1 (const-cl_FF&) cl_FF "minus1")
(defoverloaded ftruncate (const-cl_FF&) cl_FF "ftruncate")
(defoverloaded fround (const-cl_FF&) cl_FF "fround")
(defoverloaded ffloor (const-cl_FF&) cl_FF "ffloor")
(defoverloaded fceiling (const-cl_FF&) cl_FF "fceiling")
(define-cpp-converter cl_FF_fdiv_t* static)
;; Definition of class cl_FF_fdiv_t
;; At /home/bruno/cln/include/cl_ffloat.h:98
(define-cpp-class cl_FF_fdiv_t (<class-proxy>)
  ((quotient type: cl_FF cpp-name: "quotient")
   (remainder type: cl_FF cpp-name: "remainder")
   (operator= (cl_FF_fdiv_t&) cl_FF_fdiv_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_FF& const-cl_FF&)
  copy-constructor: (cl_FF_fdiv_t&)
  type: cl_FF_fdiv_t*
  class: "cl_FF_fdiv_t")
(defoverloaded ffloor2 (const-cl_FF&) cl_FF_fdiv_t "ffloor2")
(defoverloaded fceiling2 (const-cl_FF&) cl_FF_fdiv_t "fceiling2")
(defoverloaded ftruncate2 (const-cl_FF&) cl_FF_fdiv_t "ftruncate2")
(defoverloaded fround2 (const-cl_FF&) cl_FF_fdiv_t "fround2")
(define-cpp-converter cl_FF_div_t* static)
;; Definition of class cl_FF_div_t
;; At /home/bruno/cln/include/cl_ffloat.h:125
(define-cpp-class cl_FF_div_t (<class-proxy>)
  ((quotient type: cl_I cpp-name: "quotient")
   (remainder type: cl_FF cpp-name: "remainder")
   (operator= (cl_FF_div_t&) cl_FF_div_t& "operator =" implicitp: t))
  constructor: ()
  constructor: (const-cl_I& const-cl_FF&)
  copy-constructor: (cl_FF_div_t&)
  type: cl_FF_div_t*
  class: "cl_FF_div_t")
(defoverloaded floor2 (const-cl_FF&) cl_FF_div_t "floor2")
(defoverloaded floor1 (const-cl_FF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_FF&) cl_FF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_FF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_FF&) cl_FF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_FF&) cl_I "truncate1")
(defoverloaded round2 (const-cl_FF&) cl_FF_div_t "round2")
(defoverloaded round1 (const-cl_FF&) cl_I "round1")
(defoverloaded floor2 (const-cl_FF& const-cl_FF&) cl_FF_div_t "floor2")
(defoverloaded floor1 (const-cl_FF& const-cl_FF&) cl_I "floor1")
(defoverloaded ceiling2 (const-cl_FF& const-cl_FF&) cl_FF_div_t "ceiling2")
(defoverloaded ceiling1 (const-cl_FF& const-cl_FF&) cl_I "ceiling1")
(defoverloaded truncate2 (const-cl_FF& const-cl_FF&) cl_FF_div_t "truncate2")
(defoverloaded truncate1 (const-cl_FF& const-cl_FF&) cl_I "truncate1")
(defoverloaded round2 (const-cl_FF& const-cl_FF&) cl_FF_div_t "round2")
(defoverloaded round1 (const-cl_FF& const-cl_FF&) cl_I "round1")
(define-cpp-converter cl_decoded_ffloat* static)
;; Definition of class cl_decoded_ffloat
;; At /home/bruno/cln/include/cl_ffloat.h:203
(define-cpp-class cl_decoded_ffloat (<class-proxy>)
  ((mantissa type: cl_FF cpp-name: "mantissa")
   (exponent type: cl_I cpp-name: "exponent")
   (sign type: cl_FF cpp-name: "sign")
   (operator= (cl_decoded_ffloat&)
              cl_decoded_ffloat&
              "operator ="
              implicitp: t))
  constructor: ()
  constructor: (const-cl_FF& const-cl_I& const-cl_FF&)
  copy-constructor: (cl_decoded_ffloat&)
  type: cl_decoded_ffloat*
  class: "cl_decoded_ffloat")
(defoverloaded decode_float (const-cl_FF&) cl_decoded_ffloat "decode_float")
(defoverloaded float_exponent (const-cl_FF&) sintL "float_exponent")
(defoverloaded float_radix (const-cl_FF&) sintL "float_radix")
(defoverloaded float_sign (const-cl_FF&) cl_FF "float_sign")
(defoverloaded float_digits (const-cl_FF&) uintL "float_digits")
(defoverloaded float_precision (const-cl_FF&) uintL "float_precision")
(defoverloaded integer_decode_float (const-cl_FF&)
  cl_idecoded_float
  "integer_decode_float")
(defoverloaded scale_float (const-cl_FF& sintL) cl_FF "scale_float")
(defoverloaded scale_float (const-cl_FF& const-cl_I&) cl_FF "scale_float")
(defoverloaded signum (const-cl_FF&) cl_FF "signum")
(defoverloaded cl_float_approx (const-cl_FF&) float "cl_float_approx")
(defoverloaded cl_double_approx (const-cl_FF&) double "cl_double_approx")
(defoverloaded operator- (const-cl_DF&) cl_DF "operator -")
(defoverloaded cl_compare (const-cl_DF& const-cl_DF&) cl_signean "cl_compare")
(defoverloaded operator== (const-cl_DF& const-cl_DF&) bool "operator ==")
(defoverloaded operator!= (const-cl_DF& const-cl_DF&) bool "operator !=")
