// Global variables in CLN

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_float.h"


// Implementation.

cl_boolean cl_inhibit_floating_point_underflow = cl_false;

cl_float_format_t cl_default_float_format      = cl_float_format_ffloat;

