// copy().

// General includes.
#include "cl_sysdep.h"

// Specification.
#define CL_GV_NO_RANGECHECKS
#include "cl_GV_integer.h"


// Implementation.

cl_GV_I copy (const cl_GV_I& v)
{
	var uintL len = v.length();
	var cl_GV_I w = cl_GV_I(len,v.maxbits());
	for (var uintL i = 0; i < len; i++)
		w[i] = v[i];
	return w;
}
