// fprintdecimal().

// General includes.
#include "cl_sysdep.h"

// Specification.
#include "cl_io.h"


// Implementation.

#if defined(CL_IO_STDIO)

void fprintdecimal (cl_ostream stream, int x)
{
	fprintf(stream,"%d",x);
}

#endif
